## NOTES:
## Pressure in Paris lines, rain in Danish inches = 26.154 mm

library(dataresqc)
library(XLConnect)
library(lubridate)
library(plyr)

## Create station code
name_folder <- strsplit(getwd(), "/")[[1]][8]
name_group <- strsplit(getwd(), "/")[[1]][6]
station_code <- paste(name_group, name_folder, sep = "_")

## Create output directory (if necessary)
if (!name_folder %in% list.dirs("../../4_Formatted/",full.names=FALSE)) {
  dir.create(paste0("../../4_Formatted/", name_folder))
}

## Read inventory
inventory <- readWorksheetFromFile("/scratch3/PALAEO-RA/DataRescue/Inventory_PALAEO-RA_digi.xls",
                                   sheet=1, colTypes="character")

## Define function to fill missing values with the value on the previous row
fill_variable <- function(timeseries, strings) { 
  for (i in 2:length(timeseries)) {
    if (timeseries[i] %in% strings) {
      if(!timeseries[i-1] %in% strings) {
        timeseries[i] <- timeseries[i-1]
      } 
    }
  }
  return(timeseries)
}

## Define wind directions
directions <- c("N", "NNE", "NE", "ENE", "E", "ESE", "SE", "SSE", "S",
                "SSW", "SW", "WSW", "W", "WNW", "NW", "NNW")


## 1829-1834

files <- list.files("../../3_Revised/Accra/", pattern="subdaily.xls",
                    full.names=TRUE)

## Read metadata
metadata <- inventory[which(inventory$Code == station_code & 
                              inventory$Resolution == "subdaily"), ]

## Loop on files
out <- list()
for (i in 1:length(files)) {
  
  ## Read data
  template <- readWorksheetFromFile(files[i],
                                    sheet=1, startRow=8, header=FALSE, colTypes="character")
  
  ## Time
  if (i == 1) {
    hours <- c(rep(c(6,9,12,16,22), each=nrow(template)))
  } else {
    hours <- c(rep(c(7,9,12,16,21), each=nrow(template)))
  }
  minutes <- rep(0, 5*nrow(template))
  meta_time <- paste0("orig.time=", hours)
  
  ## Initialize data frame
  out[[i]] <- data.frame(year = rep(as.integer(template[,1]), 5), 
                         month = rep(as.integer(template[,2]), 5), 
                         day = rep(as.integer(template[,3]), 5), 
                         hour = hours, 
                         minute = minutes)
  
  ## Pressure
  pres_orig <- c(template[,4:8], recursive=TRUE)
  out[[i]]$p <- round(convert_pressure(as.numeric(pres_orig) + 330, 27.07/12, 
                                       as.numeric(metadata$Latitude), 
                                       as.numeric(metadata$Altitude)), 
                      1)
  out[[i]]$p_meta <- paste0(meta_time, "|orig=330+", pres_orig, "Pl")

  ## Wind direction
  wdir <- c(template[,c(10,12,14,16,18)], recursive=TRUE)
  out[[i]]$dd <- round(22.5 * (match(toupper(wdir), directions) - 1), 0)
  out[[i]]$dd_meta <- paste0(meta_time, "|orig=", wdir)
  
  ## Wind force
  wforce <- c(template[,c(11,13,15,17,19)], recursive=TRUE)
  out[[i]]$wind_force <- as.numeric(wforce)
  out[[i]]$wind_force_meta <- paste0(meta_time, "|orig=", wforce)
  
  ## Precipitation
  rain <- template[,20]
  rain[which(is.na(rain))] <- 0
  rain[which(rain=="NA")] <- NA
  out[[i]]$rr <- c(round(as.numeric(rain) * 26.154, 1), rep(NA, 4*nrow(template)))
  out[[i]]$rr_meta <- c(paste0("orig=", rain, "Din"), rep(NA, 4*nrow(template)))
  
}

## Additional pressure observations
template <- readWorksheetFromFile("../../3_Revised/Accra/Africa_T3_GH_Accra_1829-1832_subdaily-pressure.xls",
                                  sheet=1, startRow=8, header=FALSE, colTypes="character")
times <- strptime(template[,4], format="%Y-%m-%d %H:%M:%S")
hours <- hour(times)
minutes <- minute(times)
meta_time <- paste0("orig.time=", substr(template[,4],12,19))
out[[3]] <- data.frame(year = as.integer(fill_variable(template[,1], NA)), 
                       month = as.integer(fill_variable(template[,2], NA)),
                       day = as.integer(fill_variable(template[,3], NA)), 
                       hour = hours, 
                       minute = minutes)
out[[3]]$p <- round(convert_pressure(as.numeric(template[,5]) + 330, 27.07/12, 
                                     as.numeric(metadata$Latitude), 
                                     as.numeric(metadata$Altitude)), 
                    1)
out[[3]]$p_meta <- paste0(meta_time, "|orig=330+", template[,5], "Pl")

## Merge data frames
out <- Reduce(rbind.fill, out)
out <- out[order(out$year,out$month,out$day,out$hour),]

## Write SEF files
write_sef(out[,c("year","month","day","hour","minute","p")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "p",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "hPa",
          stat = "point",
          metaHead = paste0("Observer=", metadata$Observer, " | PGC=Y | PTC=Y"),
          meta = out$p_meta,
          period = 0,
          time_offset = as.numeric(metadata$Longitude) * 12 / 180,
          keep_na = TRUE)

write_sef(out[,c("year","month","day","hour","minute","dd")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "dd",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "degree",
          stat = "point",
          metaHead = paste0("Observer=", metadata$Observer),
          meta = out$dd_meta,
          period = 0,
          time_offset = as.numeric(metadata$Longitude) * 12 / 180,
          keep_na = FALSE)

write_sef(out[,c("year","month","day","hour","minute","wind_force")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "wind_force",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "1-5",
          stat = "point",
          metaHead = paste0("Observer=", metadata$Observer),
          meta = out$wind_force_meta,
          period = 0,
          time_offset = as.numeric(metadata$Longitude) * 12 / 180,
          keep_na = FALSE)

out$hour <- NA
out$minute <- NA

write_sef(out[,c("year","month","day","hour","minute","rr")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "rr",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "mm",
          stat = "sum",
          metaHead = paste0("Observer=", metadata$Observer),
          meta = out$rr_meta,
          period = "day",
          keep_na = FALSE)


## 1838-1842

## Read metadata
metadata <- inventory[which(inventory$Code == station_code & 
                              inventory$Resolution == "daily"), ]

## Read data
template <- readWorksheetFromFile("../../3_Revised/Accra/Africa_T3_GH_Accra_1838-1842_daily.xls",
                                  sheet=1, startRow=8, header=FALSE, colTypes="character")

## Pressure
hours <- rep(24, nrow(template))
minutes <- rep(0, nrow(template))
out <- data.frame(year = as.integer(template[,1]), 
                  month = as.integer(template[,2]), 
                  day = as.integer(template[,3]), 
                  hour = hours, 
                  minute = minutes)
out$p <- round(convert_pressure(as.numeric(template[,4]) + 330, 27.07/12, 
                                as.numeric(metadata$Latitude), 
                                as.numeric(metadata$Altitude)), 
               1)
out$p_meta <- paste0("orig=330+", template[,4], "Pl")
out <- out[which(!is.na(out$year)),]
write_sef(out[,c("year","month","day","hour","minute","p")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "p",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "hPa",
          stat = "mean",
          metaHead = paste0("Observer=", metadata$Observer, " | PGC=Y | PTC=Y"),
          meta = out$p_meta,
          period = "day",
          keep_na = TRUE,
          note = "daily")

## Precipitation
hours1 <- sapply(template[,6], function(x) as.numeric(strsplit(x,"-")[[1]][1]))
hours2 <- sapply(template[,6], function(x) as.numeric(strsplit(x,"-")[[1]][2]))
j <- which(!is.na(hours1) & is.na(hours2))
hours2[j] <- hours1[j]
j <- which(hours2>12)
k <- which(hours2<=12)
hours2[j] <- hours2[j] - 12
hours2[k] <- hours2[k] + 12
minutes <- as.integer((hours2 - as.integer(hours2)) * 60)
out <- data.frame(year = as.integer(fill_variable(template[,1], NA)), 
                  month = as.integer(fill_variable(template[,2], NA)), 
                  day = as.integer(fill_variable(template[,3], NA)), 
                  hour = as.integer(hours2), 
                  minute = minutes)
rain <- template[,7]
rain[which(is.na(rain))] <- 0
rain[which(rain%in%c("NA","|"))] <- NA
out$rr <- round(as.numeric(rain) * 27.07/12, 1)
out$rr_meta <- paste0("orig.time=", template[,6],"|orig=", rain, "Pl")
write_sef(out[,c("year","month","day","hour","minute","rr")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "rr",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "mm",
          stat = "sum",
          metaHead = paste0("Observer=", metadata$Observer),
          meta = out$rr_meta,
          period = "p",
          keep_na = FALSE)


## Check SEF files
for (f in list.files(paste0("../../4_Formatted/", name_folder), full.names=TRUE)) {
  print(f)
  check_sef(f)
}