## NOTES:
## Pressure was reduced using the thermometer's values
## Weather description was not formatted

library(dataresqc)
library(XLConnect)

## Create output directory (if necessary) and station code
name_folder <- strsplit(getwd(), "/")[[1]][8]
name_group <- strsplit(getwd(), "/")[[1]][6]
if (!name_folder %in% list.dirs("../../4_Formatted/",full.names=FALSE)) {
  dir.create(paste0("../../4_Formatted/", name_folder))
}
station_code <- paste(name_group, name_folder, sep = "_")

## Read inventory
inventory <- readWorksheetFromFile("/scratch3/PALAEO-RA/DataRescue/Inventory_PALAEO-RA_digi.xls",
                                   sheet=1, colTypes="character")

## Define function to fill missing values with the value on the previous row
fill_variable <- function(timeseries, strings) { 
  for (i in 2:length(timeseries)) {
    if (timeseries[i] %in% strings) {
      if(!timeseries[i-1] %in% strings) {
        timeseries[i] <- timeseries[i-1]
      } 
    }
  }
  return(timeseries)
}


## Read metadata
metadata <- inventory[which(inventory$Code == station_code), ]

## Loop on files
files <- list.files("../../3_Revised/Freetown/", full.names=TRUE)
out <- list()
for (i in 1:length(files)) {
  
  ## Read data
  template <- readWorksheetFromFile(paste0("../../3_Revised/Freetown/",files[i]),
                                    sheet=1, startRow=8, header=FALSE, colTypes="character")

  ## Time
  for (j in 1:3) template[,j] <- fill_variable(template[,j], NA)
  hours <- as.integer(substr(template[,4],12,13))
  minutes <- as.integer(substr(template[,4],15,16))
  hours[which(hours==0)] <- 24 # Fix midnight times
  meta_time <- paste0("orig.time=", substr(template[,4],12,16))
  k <- which(nchar(template[,4]) < 18)
  meta_time[k] <- paste0("orig.time=", template[k,4])
  
  ## Initialize data frame
  out[[i]] <- data.frame(year= as.integer(template[,1]), 
                         month = as.integer(template[,2]), 
                         day = as.integer(template[,3]), 
                         hour = hours, 
                         minute = minutes)
  
  ## Pressure
  atb <- (as.numeric(template[,9])-32) * 5/9
  pres <- convert_pressure(as.numeric(template[,8]), 25.4, as.numeric(metadata$Latitude),
                           as.numeric(metadata$Altitude), atb)
  out[[i]]$p <- round(pres, 1)
  out[[i]]$p_meta <- paste0(meta_time, "|orig=", template[,8], "in|atb=", template[,9], "F")

  ## Temperature
  out[[i]]$ta <- round(atb, 1)
  out[[i]]$ta_meta <- paste0(meta_time, "|orig=", template[,9], "F")
  
  ## Wind direction
  directions <- c("N", "NbE", "NNE", "NEbN", "NE", "NEbE", "ENE", "EbN", "E", 
                  "EbS", "ESE", "SEbE", "SE", "SEbS", "SSE", "SbE", "S",
                  "SbW", "SSW", "SWbS", "SW", "SWbW", "WSW", "WbS", "W",
                  "WbN", "WNW", "NWbW", "NW", "NWbN", "NNW", "NbW")
  wdir <- 11.25 * (match(template[,5],directions) - 1)
  wdir[grep("south",template[,5],ignore.case=TRUE)] <- 180
  wdir[grep("north",template[,5],ignore.case=TRUE)] <- 0
  wdir[grep("east",template[,5],ignore.case=TRUE)] <- 90
  wdir[grep("west",template[,5],ignore.case=TRUE)] <- 270
  out[[i]]$dd <- round(wdir, 0)
  out[[i]]$dd_meta <- paste0(meta_time, "|orig=", template[,5])
  
  ## Wind force
  out[[i]]$w <- as.integer(template[,6])
  out[[i]]$w_meta <- paste0(meta_time, "|orig=", template[,6])
  
}

## Merge data frames
out <- Reduce(rbind, out)
out <- out[order(out$year,out$month,out$day,out$hour,na.last=FALSE),]

## Write SEF files
write_sef(out[,c("year","month","day","hour","minute","p")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "p",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "hPa",
          stat = "point",
          metaHead = paste0("Observer=", metadata$Observer, " | PGC=Y | PTC=Y"),
          meta = out$p_meta,
          period = 0,
          time_offset = as.numeric(metadata$Longitude) * 12 / 180,
          keep_na = FALSE)

write_sef(out[,c("year","month","day","hour","minute","ta")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "ta",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "C",
          stat = "point",
          metaHead = paste0("Observer=", metadata$Observer),
          meta = out$ta_meta,
          period = 0,
          time_offset = as.numeric(metadata$Longitude) * 12 / 180,
          keep_na = FALSE)

write_sef(out[,c("year","month","day","hour","minute","dd")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "dd",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "degree",
          stat = "point",
          metaHead = paste0("Observer=", metadata$Observer),
          meta = out$dd_meta,
          period = 0,
          time_offset = as.numeric(metadata$Longitude) * 12 / 180,
          keep_na = TRUE)

write_sef(out[,c("year","month","day","hour","minute","w")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "w",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "Beaufort",
          stat = "point",
          metaHead = paste0("Observer=", metadata$Observer),
          meta = out$w_meta,
          period = 0,
          time_offset = as.numeric(metadata$Longitude) * 12 / 180,
          keep_na = TRUE)


## Check SEF files
for (f in list.files(paste0("../../4_Formatted/", name_folder), full.names=TRUE)) {
  print(f)
  check_sef(f)
}