## 

library(dataresqc)
library(XLConnect)

## Create station code
name_folder <- strsplit(getwd(), "/")[[1]][8]
name_group <- strsplit(getwd(), "/")[[1]][6]
station_code <- paste(name_group, name_folder, sep = "_")

## Create output directory (if necessary)
if (!name_folder %in% list.dirs("../../4_Formatted/",full.names=FALSE)) {
  dir.create(paste0("../../4_Formatted/", name_folder))
}

## Read inventory
inventory <- readWorksheetFromFile("/scratch3/PALAEO-RA/DataRescue/Inventory_PALAEO-RA_digi.xls",
                                 sheet=1, colTypes="character")

## Define function to fill missing values with the value on the previous row
fill_variable <- function(timeseries, strings) { 
  for (i in 2:length(timeseries)) {
    if (timeseries[i] %in% strings) {
      if(!timeseries[i-1] %in% strings) {
        timeseries[i] <- timeseries[i-1]
      } 
    }
  }
  return(timeseries)
}

## Read data and metadata
template <- readWorksheetFromFile("../../3_Revised/Kingston/Americas_T3_JM_Kingston_1785-1792_monthly.xlsx",
                                  sheet=1, startRow=9, header=FALSE)
metadata <- inventory[which(inventory$Code == station_code), ]
template[,1] <- fill_variable(template[,1], NA)

## Initialize data frame
out <- data.frame(year=as.integer(template[,1]), 
                  month=as.integer(template[,2]), 
                  day=rep("", nrow(template)), 
                  hour=rep("", nrow(template)), 
                  minute=rep("", nrow(template)))

## Temperature (Fahrenheit)
temp <- round(rowMeans(template[,9:11]), 1)
out$ta <- round((temp-32)*5/9, 1)
out$ta_meta <- paste0("orig=", temp, "F")

## Tmax
tmax <- apply(template[,3:5], 1, max)
out$tx <- round((tmax-32)*5/9, 1)
out$tx_meta <- paste0("orig=", tmax, "F")

## Tmin
tmin <- apply(template[,6:8], 1, min)
out$tn <- round((tmin-32)*5/9, 1)
out$tn_meta <- paste0("orig=", tmin, "F")

## Wet days
out$nr <- as.integer(template[,12])

## Write SEF files
write_sef(out[, c("year","month","day","hour","minute","ta")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "ta",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "C",
          stat = "mean",
          metaHead = paste0("Observer=", metadata$Observer, " | Obs.times=sunrise,1-2PM,8-9PM"),
          meta = out$ta_meta,
          period = "month",
          note = "monthly",
          keep_na = TRUE)

write_sef(out[, c("year","month","day","hour","minute","tx")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "ta",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "C",
          stat = "maximum",
          metaHead = paste0("Observer=", metadata$Observer, " | Obs.times=sunrise,1-2PM,8-9PM"),
          meta = out$tx_meta,
          period = "month",
          note = "monthly-max",
          keep_na = TRUE)

write_sef(out[, c("year","month","day","hour","minute","tn")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "ta",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "C",
          stat = "minimum",
          metaHead = paste0("Observer=", metadata$Observer, " | Obs.times=sunrise,1-2PM,8-9PM"),
          meta = out$tn_meta,
          period = "month",
          note = "monthly-min",
          keep_na = TRUE)
          
write_sef(out[, c("year","month","day","hour","minute","nr")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "nr",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "days",
          stat = "sum",
          metaHead = paste0("Observer=", metadata$Observer),
          period = "month",
          keep_na = TRUE)


## Check SEF files
for (f in list.files(paste0("../../4_Formatted/", name_folder), full.names=TRUE)) check_sef(f)