##
library(dataresqc)
library(XLConnect)

## Create station code
name_folder <- strsplit(getwd(), "/")[[1]][8]
name_group <- strsplit(getwd(), "/")[[1]][6]
station_code <- paste(name_group, name_folder, sep = "_")

## Create output directory (if necessary)
if (!name_folder %in% list.dirs("../../4_Formatted/",full.names=FALSE)) {
  dir.create(paste0("../../4_Formatted/", name_folder))
}

## Read inventory
inventory <- readWorksheetFromFile("/scratch3/PALAEO-RA/DataRescue/Inventory_PALAEO-RA_digi.xls",
                                   sheet=1, colTypes="character")

## Define function to fill missing values with the value on the previous row
fill_variable <- function(timeseries, strings) { 
  for (i in 2:length(timeseries)) {
    if (timeseries[i] %in% strings) {
      if(!timeseries[i-1] %in% strings) {
        timeseries[i] <- timeseries[i-1]
      } 
    }
  }
  return(timeseries)
}

## Read metadata
metadata <- inventory[which(inventory$Code == station_code), ]

## Read data
template <- readWorksheetFromFile("../../3_Revised/Maranhao/Americas_T3_BR_Maranhao_1821_subdaily.xls",
                                  sheet=1, startRow=8, header=FALSE, colTypes="character")

## Initialize data frame
out <- data.frame(year = rep(as.integer(template[,1]), 3), 
                  month = rep(as.integer(template[,2]), 3), 
                  day = rep(as.integer(template[,3]), 3), 
                  hour = rep(NA, 3*nrow(template)), 
                  minute = rep(NA, 3*nrow(template)))

## Time
meta_time <- paste0("orig.time=", rep(c("Manhan","Tarde","Noite"),each=nrow(template)))

## Pressure
pres <- as.integer(c(template[,c(4,7,10)],recursive=TRUE)) +
  as.integer(c(template[,c(5,8,11)],recursive=TRUE)) / 10 +
  as.integer(c(template[,c(6,9,12)],recursive=TRUE)) / 100
out$p <- round(convert_pressure(pres, 25.4, as.numeric(metadata$Latitude), 
                                as.numeric(metadata$Altitude)), 1)
out$p_meta <- paste0(meta_time, "|orig=", c(template[,c(4,7,10)],recursive=TRUE), ".",
                     c(template[,c(5,8,11)],recursive=TRUE), ".",
                     c(template[,c(6,9,12)],recursive=TRUE), "in")

## Temperature
temp <- (as.integer(c(template[,13:15],recursive=TRUE)) - 32) * 5/9
out$ta <- round(temp, 1)
out$ta_meta <- paste0(meta_time, "|orig=", c(template[,13:15],recursive=TRUE), "F")

## Precipitation
for (j in 16:24) template[which(is.na(template[,j])),j] <- 0
prec <- as.integer(c(template[,c(16,19,22)],recursive=TRUE)) +
  as.integer(c(template[,c(17,20,23)],recursive=TRUE)) / 12 +
  as.integer(c(template[,c(18,21,24)],recursive=TRUE)) / 120
out$rr <- round(prec * 25.4, 1)
out$rr_meta <- paste0(meta_time, "|orig=", c(template[,c(16,19,22)],recursive=TRUE), ".",
                     c(template[,c(17,20,23)],recursive=TRUE), ".",
                     c(template[,c(18,21,24)],recursive=TRUE), "in")


## Write SEF files
out <- out[order(out$year,out$month,out$day), ]
write_sef(out[,c("year","month","day","hour","minute","p")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "p",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "hPa",
          stat = "point",
          metaHead = paste0("Observer=", metadata$Observer, " | PGC=Y | PTC=N"),
          meta = out$p_meta,
          period = 0,
          time_offset = as.numeric(metadata$Longitude) * 12 / 180,
          keep_na = TRUE)

write_sef(out[,c("year","month","day","hour","minute","ta")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "ta",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "C",
          stat = "point",
          metaHead = paste0("Observer=", metadata$Observer),
          meta = out$ta_meta,
          period = 0,
          time_offset = as.numeric(metadata$Longitude) * 12 / 180,
          keep_na = TRUE)

write_sef(out[,c("year","month","day","hour","minute","rr")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "rr",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "mm",
          stat = "sum",
          metaHead = paste0("Observer=", metadata$Observer),
          meta = out$rr_meta,
          period = "p",
          keep_na = FALSE)


## Check SEF files
for (f in list.files(paste0("../../4_Formatted/", name_folder), full.names=TRUE)) {
  print(f)
  check_sef(f)
}