## NOTE: The data under 'Therm. register' were not formatted, nor were the cloud types

library(dataresqc)
library(XLConnect)
library(lubridate)
library(plyr)

## Create station code
name_folder <- strsplit(getwd(), "/")[[1]][8]
name_group <- strsplit(getwd(), "/")[[1]][6]
station_code <- paste(name_group, name_folder, sep = "_")

## Create output directory (if necessary)
if (!name_folder %in% list.dirs("../../4_Formatted/",full.names=FALSE)) {
  dir.create(paste0("../../4_Formatted/", name_folder))
}

## Read inventory
inventory <- readWorksheetFromFile("/scratch3/PALAEO-RA/DataRescue/Inventory_PALAEO-RA_digi.xls",
                                   sheet=1, colTypes="character")

## Define function to fill missing values with the value on the previous row
fill_variable <- function(timeseries, strings) { 
  for (i in 2:length(timeseries)) {
    if (timeseries[i] %in% strings) {
      if(!timeseries[i-1] %in% strings) {
        timeseries[i] <- timeseries[i-1]
      } 
    }
  }
  return(timeseries)
}

## Define wind directions
directions <- c("N", "N by E", "NNE", "NE by N", "NE", "NE by E", "ENE", "E by N", "E",
                "E by S", "ESE", "SE by E", "SE", "SE by S", "SSE", "S by E", "S",
                "S by W", "SSW", "SW by S", "SW", "SW by W", "WSW", "W by S", "W",
                "W by N", "WNW", "NW by W", "NW", "NW by N", "NNW", "N by W")

## Read metadata
metadata <- inventory[which(inventory$Code == station_code), ]

## Loop over files
out <- list()
files <- list.files("../../3_Revised/Santiago/", full.names=TRUE)
for (i in 1:2) {
  
  ## Read data
  template <- readWorksheetFromFile(files[i], sheet=1, startRow=8, header=FALSE, colTypes="character")

  ## Time
  for (k in 1:3) template[,k] <- fill_variable(template[,k], NA)
  if (i == 2) {
    k <- which(substr(template[,4],12,16)=="00:00")
    template[k,4] <- sub("00:00", "24:00", template[k,4])
  }
  l <- which(nchar(template[,4]) < 12)
  template[l,4] <- paste("1899-12-31", template[l,4])
  hours <- hour(strptime(template[,4],format="%Y-%m-%d %H:%M:%S"))
  minutes <- minute(strptime(template[,4],format="%Y-%m-%d %H:%M:%S"))
  dates <- ISOdate(template[,1], template[,2], template[,3])
  if (i == 2) dates[k] <- dates[k] + 24*3600
  meta_time <- paste0("orig.date=", template[,1], "-", template[,2], "-", template[,3], 
                      "|orig.time=", substr(template[,4],12,16))
  
  ## Initialize data frame
  out[[i]] <- data.frame(year = year(dates), 
                         month = month(dates), 
                         day = day(dates), 
                         hour = hours, 
                         minute = minutes)
  
  ## Pressure
  template[,5] <- fill_variable(template[,5], NA)
  pres <- as.numeric(template[,5]) + as.numeric(template[,6])/1000
  atb <- (as.numeric(template[,7]) - 32) * 5/9
  pres <- convert_pressure(pres, 25.4, as.numeric(metadata$Latitude), 
                           as.numeric(metadata$Altitude), atb)
  out[[i]]$p <- round(pres, 2)
  psub <- format(template[,6], width=3, justify="right")
  psub[which(nchar(psub)>3)] <- substr(psub[which(nchar(psub)>3)], 2, 4)
  out[[i]]$p_meta <- paste0(meta_time, "|orig=", template[,5], ".", 
                            gsub(" ","0",psub), "in|atb=", template[,7], "F")
  
  ## Temperature
  out[[i]]$ta <- round((as.numeric(template[,8])-32)*5/9, 1)
  out[[i]]$ta_meta <- paste0(meta_time, "|orig=", template[,8], "F")
  
  ## Wet bulb
  out[[i]]$tb <- round((as.numeric(template[,9])-32)*5/9, 1)
  out[[i]]$tb_meta <- paste0(meta_time, "|orig=", template[,9], "F")

  ## Dry bulb
  out[[i]]$tdry <- round((as.numeric(template[,10])-32)*5/9, 1)
  out[[i]]$tdry_meta <- paste0(meta_time, "|orig=", template[,10], "F")
  
  ## Wind direction
  j <- ifelse(i==1, 15, 16)
  out[[i]]$dd <- round(11.25 * (match(template[,j], directions) - 1), 0)
  out[[i]]$dd_meta <- paste0(meta_time, "|orig=", template[,j])
  
  ## Wind force
  out[[i]]$w <- as.numeric(template[,j+1])
  out[[i]]$w_meta <- paste0(meta_time, "|orig=", template[,j+1])
  
  ## Cloud cover
  cc <- strsplit(template[,j+2], " ")
  out[[i]]$n <- as.numeric(unlist(lapply(cc, function(x) x[length(x)]))) * 10
  out[[i]]$n_meta <- paste0(meta_time, "|orig=", template[,j+2])
  
  ## Daily data (Tmax, Tmin, Wet bulb min, Precipitation)
  if (i == 2) {
    out[[i]]$tx <- round((as.numeric(template[,19])-32)*5/9, 1)
    out[[i]]$tx_meta <- paste0(meta_time, "|orig=", template[,19], "F")
    out[[i]]$tn <- round((as.numeric(template[,20])-32)*5/9, 1)
    out[[i]]$tn_meta <- paste0(meta_time, "|orig=", template[,20], "F")    
    out[[i]]$tb_min <- round((as.numeric(template[,21])-32)*5/9, 1)
    out[[i]]$tb_min_meta <- paste0(meta_time, "|orig=", template[,21], "F")
    prcp <- template[,22]
    prcp[which(is.na(prcp) & hours%in%0:2)] <- 0
    out[[i]]$rr <- round(as.numeric(prcp)*25.4, 1)
    out[[i]]$rr_meta <- paste0(meta_time, "|orig=", template[,22], "in")
  }
  
}

## Merge data frames
out <- Reduce(rbind.fill, out)
out$time <- ISOdatetime(out$year,out$month,out$day,out$hour,out$minute,0)
out[duplicated(out$time), c("p","ta","tb","tdry","dd","w","n")] <- NA
out <- out[order(out$year,out$month,out$day,out$hour,out$minute), ]

## Write SEF files
write_sef(out[, c("year","month","day","hour","minute","ta")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "ta",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "C",
          stat = "point",
          metaHead = paste0("Observer=", metadata$Observer, " | Instrument=Standard"),
          meta = out$ta_meta,
          period = 0,
          time_offset = as.numeric(metadata$Longitude) * 12 / 180)

write_sef(out[, c("year","month","day","hour","minute","p")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "p",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "hPa",
          stat = "point",
          metaHead = paste0("Observer=", metadata$Observer, " | PTC=Y | PGC=Y"),
          meta = out$p_meta,
          period = 0,
          time_offset = as.numeric(metadata$Longitude) * 12 / 180)

write_sef(out[, c("year","month","day","hour","minute","tb")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "tb",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "C",
          stat = "point",
          metaHead = paste0("Observer=", metadata$Observer),
          meta = out$tb_meta,
          period = 0,
          time_offset = as.numeric(metadata$Longitude) * 12 / 180)

write_sef(out[, c("year","month","day","hour","minute","tdry")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "ta",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "C",
          stat = "point",
          metaHead = paste0("Observer=", metadata$Observer, " | Instrument=Psychrometer (dry bulb)"),
          meta = out$tdry_meta,
          period = 0,
          time_offset = as.numeric(metadata$Longitude) * 12 / 180,
          note = "dry")

write_sef(out[, c("year","month","day","hour","minute","dd")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "dd",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "degree",
          stat = "point",
          metaHead = paste0("Observer=", metadata$Observer),
          meta = out$dd_meta,
          period = 0,
          time_offset = as.numeric(metadata$Longitude) * 12 / 180)

write_sef(out[, c("year","month","day","hour","minute","w")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "w",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "0-10",
          stat = "point",
          metaHead = paste0("Observer=", metadata$Observer),
          meta = out$w_meta,
          period = 0,
          time_offset = as.numeric(metadata$Longitude) * 12 / 180)

write_sef(out[, c("year","month","day","hour","minute","n")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "n",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "%",
          stat = "point",
          metaHead = paste0("Observer=", metadata$Observer),
          meta = out$n_meta,
          period = 0,
          time_offset = as.numeric(metadata$Longitude) * 12 / 180)

write_sef(out[, c("year","month","day","hour","minute","tx")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "Tx",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "C",
          stat = "maximum",
          metaHead = paste0("Observer=", metadata$Observer),
          meta = out$tx_meta,
          period = "day",
          time_offset = as.numeric(metadata$Longitude) * 12 / 180)

write_sef(out[, c("year","month","day","hour","minute","tn")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "Tn",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "C",
          stat = "minimum",
          metaHead = paste0("Observer=", metadata$Observer),
          meta = out$tn_meta,
          period = "day",
          time_offset = as.numeric(metadata$Longitude) * 12 / 180)

write_sef(out[, c("year","month","day","hour","minute","tb_min")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "tb",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "C",
          stat = "minimum",
          metaHead = paste0("Observer=", metadata$Observer),
          meta = out$tb_min_meta,
          period = "day",
          time_offset = as.numeric(metadata$Longitude) * 12 / 180,
          note = "min")

write_sef(out[, c("year","month","day","hour","minute","rr")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "rr",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "mm",
          stat = "sum",
          metaHead = paste0("Observer=", metadata$Observer),
          meta = out$rr_meta,
          period = "p",
          time_offset = as.numeric(metadata$Longitude) * 12 / 180)


## Check SEF files
for (f in list.files(paste0("../../4_Formatted/", name_folder), full.names=TRUE)) {
  print(f)
  check_sef(f)
}
