## NOTES: 
## For monthly data, only the mid-range between monthly minimum and maximum was formatted
## (ignoring noon temperature observations, which are clearly biased even in the shadow)
## Monthly indoor temperature and temperature in the Sun were not formatted

library(dataresqc)
library(XLConnect)

## Create station code
name_folder <- strsplit(getwd(), "/")[[1]][8]
name_group <- strsplit(getwd(), "/")[[1]][6]
station_code <- paste(name_group, name_folder, sep = "_")

## Create output directory (if necessary)
if (!name_folder %in% list.dirs("../../4_Formatted/",full.names=FALSE)) {
  dir.create(paste0("../../4_Formatted/", name_folder))
}

## Read inventory
inventory <- readWorksheetFromFile("/scratch3/PALAEO-RA/DataRescue/Inventory_PALAEO-RA_digi.xls",
                                 sheet=1, colTypes="character")

## Define function to fill missing values with the value on the previous row
fill_variable <- function(timeseries, strings) { 
  for (i in 2:length(timeseries)) {
    if (timeseries[i] %in% strings) {
      if(!timeseries[i-1] %in% strings) {
        timeseries[i] <- timeseries[i-1]
      } 
    }
  }
  return(timeseries)
}


## SUBDAILY

## 1788-1790
template <- readWorksheetFromFile("../../3_Revised/StBarthelemy/Americas_T3_FR_StBarthelemy_1788-1790_subdaily.xls",
                                  sheet=1, startRow=8, header=FALSE, colTypes="character")
metadata <- inventory[which(inventory$Code == station_code & inventory$Resolution == "subdaily"), ]

## Time
template[,1] <- fill_variable(template[,1], NA)
template[,2] <- fill_variable(template[,2], NA)
times <- rep(c("6AM","1PM","8PM"), each=nrow(template))
hours <- rep(c(6,13,20), each=nrow(template))
minutes <- rep(0, 3*nrow(template))
dates <- as.Date(paste(template[,1], template[,2], template[,3], sep="-"))
meta_time <- paste0("orig.date=", dates, "|orig.time=", times)

## Initialize data frame
out1 <- data.frame(year=as.integer(template[,1]), 
                   month=as.integer(template[,2]), 
                   day=as.integer(template[,3]), 
                   hour=hours, 
                   minute=minutes)

## Temperature
temp <- c(template[,c(4,6,8)], recursive=TRUE)
out1$ta <- round(as.numeric(temp) * 1.25, 1)
out1$ta_meta <- paste0(meta_time, "|orig=", temp, "R")

## Pressure
pres <- c(template[,10:12], recursive=TRUE)
atb <- c(template[,c(5,7,9)], recursive=TRUE)
out1$p <- round(convert_pressure(as.numeric(pres),29.69,as.numeric(metadata$Latitude),
                                 as.numeric(metadata$Altitude),1.25*as.numeric(atb)), 1)
out1$p_meta <- paste0(meta_time, "|orig=", pres, "Swed.in|atb=", atb, "R")
j <- which(is.na(atb))
out1$p[j] <- round(convert_pressure(as.numeric(pres[j]),29.69,as.numeric(metadata$Latitude),
                                    as.numeric(metadata$Altitude)), 1)
out1$p_meta[j] <- paste0(meta_time[j], "|orig=", pres[j], "Swed.in|PTC=N")

## Wind direction
directions <- c("N", "NNO", "NO", "ONO", "O", "OSO", "SO", "SSO", "S",
                "SSW", "SW", "WSW", "W", "WNW", "NW", "NNW")
wdir <- 22.5 * (match(toupper(template[,13]), directions) - 1)
out1$dd <- c(round(wdir,0), rep(NA,2*nrow(template)))
out1$dd_meta <- paste0("orig=", c(template[,13],rep("",2*nrow(template))))


## 1791-1793
template <- readWorksheetFromFile("../../3_Revised/StBarthelemy/Americas_T3_FR_StBarthelemy_1791-1793_subdaily.xls",
                                  sheet=1, startRow=8, header=FALSE, colTypes="character")

## Time
template[,1] <- fill_variable(template[,1], NA)
template[,2] <- fill_variable(template[,2], NA)
template[,3] <- fill_variable(template[,3], NA)
times <- paste(template[,5], template[,6], sep=":")
pm <- grep("em", template[,4])
hours <- as.integer(template[,5])
hours[pm] <- hours[pm] + 12
minutes <- as.integer(template[,6])
times[which(minutes<0)] <- paste0(template[which(minutes<0),5], ":0", 
                                  template[which(minutes<0),6])
dates <- as.Date(paste(template[,1], template[,2], template[,3], sep="-"))
meta_time <- paste0("orig.date=", dates, "|orig.time=", times, template[,4])

## Initialize data frame
out2 <- data.frame(year=as.integer(template[,1]), 
                   month=as.integer(template[,2]), 
                   day=as.integer(template[,3]), 
                   hour=hours, 
                   minute=minutes)

## Temperature
out2$ta <- round(as.numeric(template[,8]) * 1.25, 1)
out2$ta_meta <- paste0(meta_time, "|orig=", template[,8], "R")

## Pressure
pres <- convert_pressure(as.numeric(template[,7]), 29.69, as.numeric(metadata$Latitude),
                         as.numeric(metadata$Altitude))
out2$p <- round(pres, 1)
out2$p_meta <- paste0(meta_time, "|orig=", template[,7], "Swed.in|PTC=N")

## Wind direction
wdir <- 22.5 * (match(toupper(template[,9]), directions) - 1)
out2$dd <- round(wdir,0)
out2$dd_meta <- paste0(meta_time, "|orig=", template[,9])

## Merge data frames
out <- rbind(out1, out2)
out <- out[order(out$year,out$month,out$day,out$hour,out$minute), ]

## Write SEF files
write_sef(out[, c("year","month","day","hour","minute","ta")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "ta",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "C",
          stat = "point",
          metaHead = paste0("Observer=", metadata$Observer),
          meta = out$ta_meta,
          period = 0,
          time_offset = as.numeric(metadata$Longitude) * 12 / 180,
          keep_na = TRUE)

write_sef(out[, c("year","month","day","hour","minute","p")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "p",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "hPa",
          stat = "point",
          metaHead = paste0("Observer=", metadata$Observer, " | PTC=Y | PGC=Y"),
          meta = out$p_meta,
          period = 0,
          time_offset = as.numeric(metadata$Longitude) * 12 / 180,
          keep_na = TRUE)

out$hour[which(out$year<1791)] <- NA
out$minute[which(out$year<1791)] <- NA
write_sef(out[which(out$dd_meta!="orig="), c("year","month","day","hour","minute","dd")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "dd",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "degree",
          stat = "point",
          metaHead = paste0("Observer=", metadata$Observer),
          meta = out$dd_meta[which(out$dd_meta!="orig=")],
          period = 0,
          time_offset = as.numeric(metadata$Longitude) * 12 / 180,
          keep_na = TRUE)


## MONTHLY
template <- readWorksheetFromFile("../../3_Revised/StBarthelemy/Americas_T3_FR_StBarthelemy_1787-1793_monthly.xls",
                                  sheet=1, startRow=8, header=FALSE, colTypes="character")
metadata <- inventory[which(inventory$Code == station_code & inventory$Resolution == "monthly"), ]
template[,1] <- fill_variable(template[,1], NA)

## Initialize data frame
out <- data.frame(year=as.integer(template[,1]), 
                  month=as.integer(template[,2]), 
                  day=rep("",nrow(template)), 
                  hour=rep("",nrow(template)), 
                  minute=rep("",nrow(template)))

## Temperature
tmin <- apply(template[,c(4,14)], 1, function(x) min(as.numeric(x),na.rm=TRUE))
tmax <- apply(template[,c(3,13)], 1, function(x) max(as.numeric(x),na.rm=TRUE))
temp <- round((tmin + tmax) / 2, 2)
temp[which(is.nan(temp))] <- NA
out$ta <- round(temp * 1.25, 1)
out$ta_meta <- paste0("orig=", temp, "R")

## Pressure
pmin <- apply(template[,c(18,20,22,24)], 1, function(x) min(as.numeric(x),na.rm=TRUE))
pmax <- apply(template[,c(17,19,21,23)], 1, function(x) max(as.numeric(x),na.rm=TRUE))
pres <- round((pmin + pmax) / 2, 3)
pres[which(is.nan(pres))] <- NA
out$p <- round(convert_pressure(pres, 29.69, as.numeric(metadata$Latitude), 
                                as.numeric(metadata$Altitude)), 1)
out$p_meta <- paste0("orig=", pres, "Swed.in")

## Wet days
out$nr <- as.integer(template[,42])
  
## Write SEF files
write_sef(out[, c("year","month","day","hour","minute","ta")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "ta",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "C",
          stat = "mid_range",
          metaHead = paste0("Observer=", metadata$Observer, " | Obs.times=morning,evening"),
          meta = out$ta_meta,
          period = "month",
          note = "monthly",
          keep_na = TRUE)

write_sef(out[, c("year","month","day","hour","minute","p")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "p",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "hPa",
          stat = "mid_range",
          metaHead = paste0("Observer=", metadata$Observer, 
                            " | PTC=N | PGC=Y | Obs.times=morning,noon,evening"),
          meta = out$p_meta,
          period = "month",
          note = "monthly",
          keep_na = TRUE)

write_sef(out[, c("year","month","day","hour","minute","nr")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "nr",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "days",
          stat = "sum",
          metaHead = paste0("Observer=", metadata$Observer),
          period = "month",
          keep_na = TRUE)


## Check SEF files
for (f in list.files(paste0("../../4_Formatted/", name_folder), full.names=TRUE)) {
  print(f)
  check_sef(f)
}