## NOTE: Etat du Ciel was not formatted

library(dataresqc)
library(XLConnect)
library(plyr)

## Create station code
name_folder <- strsplit(getwd(), "/")[[1]][8]
name_group <- strsplit(getwd(), "/")[[1]][6]
station_code <- paste(name_group, name_folder, sep = "_")

## Create output directory (if necessary)
if (!name_folder %in% list.dirs("../../4_Formatted/",full.names=FALSE)) {
  dir.create(paste0("../../4_Formatted/", name_folder))
}

## Read inventory and metadata
inventory <- readWorksheetFromFile("/scratch3/PALAEO-RA/DataRescue/Inventory_PALAEO-RA_digi.xls",
                                 sheet=1, colTypes="character")
metadata <- inventory[which(inventory$Code == station_code & inventory$Resolution=="subdaily"), ]

## Define function to fill missing values with the value on the previous row
fill_variable <- function(timeseries, strings) {
  for (i in 2:length(timeseries)) {
    if (timeseries[i] %in% strings) {
      if(!timeseries[i-1] %in% strings) {
        timeseries[i] <- timeseries[i-1]
      }
    }
  }
  return(timeseries)
}


## Read data
template <- readWorksheetFromFile("../../3_Revised/Baghdad/Asia_T3_IQ_Baghdad_1782-1784_subdaily.xlsx",
                                  sheet=1, startRow=7, header=FALSE, colTypes="character")

## Time
for (i in 1:3) template[,i] <- fill_variable(template[,i], NA)
hours <- as.integer(template[,4])
minutes <- as.integer((as.numeric(template[,4]) - hours) * 60)
hours[grep("midi",template[,5])] <- 12
minutes[grep("midi",template[,5])] <- 0
hours[grep("minuit",template[,5])] <- 24
minutes[grep("minuit",template[,5])] <- 0
i_pm <- grep("soir", template[,5])
hours[i_pm] <- hours[i_pm] + 12
template[which(is.na(template[,4])), 4] <- "" 
meta_time <- paste0("orig.time=", template[,4], template[,5])

## Pressure
pres <- as.integer(template[,6]) + as.numeric(template[,7])/12
pres <- convert_pressure(pres, 27.07, as.numeric(metadata$Latitude), as.numeric(metadata$Altitude))
p_meta <- paste0(meta_time, "|orig=", template[,6], ".", template[,7], "Pin")

## Temperature
temp <- as.numeric(template[,8]) * 1.25
ta_meta <- paste0(meta_time, "|orig=", template[,8], "R")
  
## Wind direction
directions <- c("N", "NNO", "NO", "ONO", "O", "OSO", "SO", "SSO", "S",
                "SSW", "SW", "WSW", "W", "WNW", "NW", "NNW")
wdir <- 22.5 * (match(toupper(template[,9]), directions) - 1)
dd_meta <- paste0(meta_time, "|orig=", template[,9])

## Create output data frame
out <- data.frame(Year = as.integer(template[,1]),
                  Month = as.integer(template[,2]),
                  Day = as.integer(template[,3]),
                  Hour = hours,
                  Minute = minutes,
                  ta = round(temp, 1), ta_meta,
                  p = round(pres, 1), p_meta,
                  dd = round(wdir, 0), dd_meta,
                  stringsAsFactors = FALSE)

## Write SEF files
write_sef(out[, c(1:6)],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "ta",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "C",
          stat = "point",
          meta = out$ta_meta,
          metaHead = paste0("Observer=", metadata$Observer, " | Exposure=NE"),
          period = 0,
          time_offset = as.numeric(metadata$Longitude)*12/180,
          keep_na = TRUE)

write_sef(out[, c(1:5,8)],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "p",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "hPa",
          stat = "point",
          meta = out$p_meta,
          metaHead = paste0("Observer=", metadata$Observer, " | PTC=N | PGC=Y"),
          period = 0,
          time_offset = as.numeric(metadata$Longitude)*12/180,
          keep_na = TRUE)

write_sef(out[, c(1:5,10)],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "dd",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "degree",
          stat = "point",
          meta = out$dd_meta,
          metaHead = paste0("Observer=", metadata$Observer),
          period = 0,
          time_offset = as.numeric(metadata$Longitude)*12/180,
          keep_na = TRUE)


## Check SEF files
for (f in list.files(paste0("../../4_Formatted/", name_folder), full.names=TRUE)) check_sef(f)
