## NOTE: Observations made out of town (Jan-Feb 1777) are left out

library(dataresqc)
library(XLConnect)
library(plyr)

## Create station code
name_folder <- strsplit(getwd(), "/")[[1]][8]
name_group <- strsplit(getwd(), "/")[[1]][6]
station_code <- paste(name_group, name_folder, sep = "_")

## Create output directory (if necessary)
if (!name_folder %in% list.dirs("../../4_Formatted/",full.names=FALSE)) {
  dir.create(paste0("../../4_Formatted/", name_folder))
}

## Read inventory and metadata
inventory <- readWorksheetFromFile("/scratch3/PALAEO-RA/DataRescue/Inventory_PALAEO-RA_digi.xls",
                                 sheet=1, colTypes="character")
metadata <- inventory[which(inventory$Code == station_code & inventory$Resolution=="subdaily"), ]

## Define function to fill missing values with the value on the previous row
fill_variable <- function(timeseries, strings) {
  for (i in 2:length(timeseries)) {
    if (timeseries[i] %in% strings) {
      if(!timeseries[i-1] %in% strings) {
        timeseries[i] <- timeseries[i-1]
      }
    }
  }
  return(timeseries)
}


## 1776-1777
template <- readWorksheetFromFile("../../3_Revised/Chennai/Asia_T3_IN_Chennai_1776-1777_subdaily.xls",
                                  sheet=1, startRow=8, header=FALSE, colTypes="character")

## Time
for (i in 1:3) template[,i] <- fill_variable(template[,i], NA)
ko <- which(template[,6] == "-")
template <- template[-ko,] # Remove observations out of town
dates <- as.Date(paste(template[,1], template[,2], template[,3], sep="-"))
hours <- as.integer(template[,4])
minutes <- as.integer((as.numeric(template[,4]) - hours) * 60)
i_am <- which(hours>12)
i_pm <- which(hours<=12)
hours[i_am] <- hours[i_am] - 12
hours[i_pm] <- hours[i_pm] + 12
meta_time <- paste0("orig.date=", dates, "|orig.time=noon+", template[,4])

## Temperature
temp <- (as.numeric(template[,5]) - 32) * 5/9
ta_meta <- paste0(meta_time, "|orig=", template[,5], "F")

## Pressure
pres <- as.integer(template[,7]) + (as.numeric(template[,7])-as.integer(template[,7]))*5
atb <- (as.numeric(template[,6]) - 32) * 5/9
pres <- convert_pressure(pres, 25.4, as.numeric(metadata$Latitude), as.numeric(metadata$Altitude), atb)
p_meta <- paste0(meta_time, "|orig=", template[,7], "in|atb=", template[,6], "F")
  
## Wind direction
directions <- c("N", "NNE", "NE", "ENE", "E", "ESE", "SE", "SSE", "S",
                "SSW", "SW", "WSW", "W", "WNW", "NW", "NNW")
wdir <- 22.5 * (match(toupper(template[,9]), directions) - 1)
dd_meta <- paste0(meta_time, "|orig=", template[,9])

## Wind force
w_meta <- paste0(meta_time, "|orig=", template[,8])

## Create output data frame
out1 <- data.frame(Year = as.integer(template[,1]),
                  Month = as.integer(template[,2]),
                  Day = as.integer(template[,3]),
                  Hour = hours,
                  Minute = minutes,
                  ta = round(temp, 1), ta_meta,
                  p = round(pres, 1), p_meta,
                  dd = round(wdir, 0), dd_meta,
                  w = as.integer(template[,8]), w_meta,
                  stringsAsFactors = FALSE)


## 1777-1778
template <- readWorksheetFromFile("../../3_Revised/Chennai/Asia_T3_IN_Chennai_1777-1778_subdaily.xls",
                                  sheet=1, startRow=8, header=FALSE, colTypes="character")

## Time
for (i in 1:3) template[,i] <- fill_variable(template[,i], NA)
dates <- as.Date(paste(template[,1], template[,2], template[,3], sep="-"))
hours <- as.integer(template[,4])
minutes <- as.integer((as.numeric(template[,4]) - hours) * 60)
i_am <- which(hours>12)
i_pm <- which(hours<=12)
hours[i_am] <- hours[i_am] - 12
hours[i_pm] <- hours[i_pm] + 12
meta_time <- paste0("orig.date=", dates, "|orig.time=noon+", template[,4])

## Temperature
temp <- (as.numeric(template[,5]) - 32) * 5/9
ta_meta <- paste0(meta_time, "|orig=", template[,5], "F")

## Pressure
pres <- as.integer(template[,7]) + (as.numeric(template[,7])-as.integer(template[,7]))*5
atb <- (as.numeric(template[,6]) - 32) * 5/9
pres <- convert_pressure(pres, 25.4, as.numeric(metadata$Latitude), as.numeric(metadata$Altitude), atb)
p_meta <- paste0(meta_time, "|orig=", template[,7], "in|atb=", template[,6], "F")

## Precipitation
rainy_dates <- unique(dates[which(!is.na(template[,8]))])
template[which(is.na(template[,8]) & !dates%in%rainy_dates & !duplicated(dates)),8] <- 0
template[which(template[,8]==0 & dates%in%rainy_dates)] <- NA
prec <- as.numeric(template[,8]) * 25.4
rr_meta <- paste0(meta_time, "|orig=", template[,8], "in")

## Wind direction
wdir <- 22.5 * (match(toupper(template[,10]), directions) - 1)
wdir[which(template[,10]=="S by W")] <- 191
wdir[which(template[,10]=="SE by E")] <- 124
dd_meta <- paste0(meta_time, "|orig=", template[,10])

## Wind force
w_meta <- paste0(meta_time, "|orig=", template[,9])

## Create output data frame
out2 <- data.frame(Year = as.integer(template[,1]),
                   Month = as.integer(template[,2]),
                   Day = as.integer(template[,3]),
                   Hour = hours,
                   Minute = minutes,
                   ta = round(temp, 1), ta_meta,
                   p = round(pres, 1), p_meta,
                   rr = round(prec, 1), rr_meta,
                   dd = round(wdir, 0), dd_meta,
                   w = as.integer(sub("°","",template[,9])), w_meta,
                   stringsAsFactors = FALSE)


## Merge data frames
out <- rbind.fill(out1, out2)


## Write SEF files
write_sef(out[, c(1:6)],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "ta",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "C",
          stat = "point",
          meta = out$ta_meta,
          metaHead = paste0("Observer=", metadata$Observer),
          period = 0,
          time_offset = as.numeric(metadata$Longitude)*12/180,
          keep_na = TRUE)

write_sef(out[, c(1:5,8)],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "p",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "hPa",
          stat = "point",
          meta = out$p_meta,
          metaHead = paste0("Observer=", metadata$Observer, " | PTC=Y | PGC=Y"),
          period = 0,
          time_offset = as.numeric(metadata$Longitude)*12/180,
          keep_na = TRUE)

write_sef(out[, c(1:5,10)],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "dd",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "degree",
          stat = "point",
          meta = out$dd_meta,
          metaHead = paste0("Observer=", metadata$Observer),
          period = 0,
          time_offset = as.numeric(metadata$Longitude)*12/180,
          keep_na = TRUE)

write_sef(out[, c(1:5,12)],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "wind_force",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "0-3",
          stat = "point",
          meta = out$w_meta,
          metaHead = paste0("Observer=", metadata$Observer),
          period = 0,
          time_offset = as.numeric(metadata$Longitude)*12/180,
          keep_na = TRUE)

write_sef(out[, c(1:5,14)],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "rr",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "mm",
          stat = "sum",
          meta = out$rr_meta,
          metaHead = paste0("Observer=", metadata$Observer),
          period = "p1day",
          time_offset = as.numeric(metadata$Longitude)*12/180,
          keep_na = FALSE)


## Check SEF files
for (f in list.files(paste0("../../4_Formatted/", name_folder), full.names=TRUE)) check_sef(f)
