##
library(dataresqc)
library(XLConnect)
library(lubridate)
library(suncalc)

## Create station code
name_folder <- strsplit(getwd(), "/")[[1]][8]
name_group <- strsplit(getwd(), "/")[[1]][6]
station_code <- paste(name_group, name_folder, sep = "_")

## Create output directory (if necessary)
if (!name_folder %in% list.dirs("../../4_Formatted/",full.names=FALSE)) {
  dir.create(paste0("../../4_Formatted/", name_folder))
}

## Read inventory
inventory <- readWorksheetFromFile("/scratch3/PALAEO-RA/DataRescue/Inventory_PALAEO-RA_digi.xls",
                                   sheet=1, colTypes="character")

## Define function to fill missing values with the value on the previous row
fill_variable <- function(timeseries, strings) { 
  for (i in 2:length(timeseries)) {
    if (timeseries[i] %in% strings) {
      if(!timeseries[i-1] %in% strings) {
        timeseries[i] <- timeseries[i-1]
      } 
    }
  }
  return(timeseries)
}

## Read metadata
metadata <- inventory[which(inventory$Code == station_code), ]

## Read data
template <- readWorksheetFromFile("../../3_Revised/Kathmandu/Asia_T3_NP_Kathmandu_1802-1803_subdaily.xls",
                                  sheet=1, startRow=8, header=FALSE, colTypes="character")
template[,4] <- fill_variable(template[,4], NA)
template <- template[which(template[,4]=="Kathmandu"), ] # exlude observations taken outside of Kathmandu

## Initialize data frame
out <- data.frame(year = rep(as.integer(template[,1]), 4), 
                  month = rep(as.integer(template[,2]), 4), 
                  day = rep(as.integer(template[,3]), 4), 
                  hour = rep(c(12,15,21,4), each=nrow(template)), 
                  minute = rep(0, 4*nrow(template)))

## Time
dates <- as.Date(paste(out$year,out$month,out$day,sep="-"))
dates[which(out$hour==4)] <- dates[which(out$hour==4)] + 1
out$year[which(out$hour==4)] <- year(dates[which(out$hour==4)])
out$month[which(out$hour==4)] <- month(dates[which(out$hour==4)])
out$day[which(out$hour==4)] <- day(dates[which(out$hour==4)])
j <- which((out$year > 1802 | out$month > 5) & out$hour==4)
sunrise <- getSunlightTimes(dates[j], as.numeric(metadata$Latitude), 
                            as.numeric(metadata$Longitude), keep="sunrise", tz="GMT")$sunrise
sunrise <- sunrise + 3600*as.numeric(metadata$Longitude)*12/180
out$hour[j] <- hour(sunrise)
out$minute[j] <- minute(sunrise)
meta_time <- paste0("orig.date=", dates, "|orig.time=", 
                    rep(c("noon","3PM","9PM","4AM"),each=nrow(template)))
meta_time[j] <- sub("4AM", "sunrise", meta_time[j])

## Pressure
pres <- as.numeric(c(template[,10:13],recursive=TRUE))
out$p <- round(convert_pressure(pres, 25.4, as.numeric(metadata$Latitude), 
                                as.numeric(metadata$Altitude)), 1)
out$p_meta <- paste0(meta_time, "|orig=", c(template[,10:13],recursive=TRUE), "in")

## Temperature
temp <- (as.integer(c(template[,6:9],recursive=TRUE)) - 32) * 5/9
out$ta <- round(temp, 1)
out$ta_meta <- paste0(meta_time, "|orig=", c(template[,6:9],recursive=TRUE), "F")

## Precipitation
out$rr <- round(c(as.numeric(template[,14]),rep(NA,3*nrow(template)))*25.4, 1)
out$rr_meta <- paste0("orig=", c(template[,14],rep(NA,3*nrow(template))), "in")

## Order data frame
out <- out[order(out$year,out$month,out$day,out$hour,out$minute),]

## Write SEF files
write_sef(out[,c("year","month","day","hour","minute","p")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "p",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "hPa",
          stat = "point",
          metaHead = paste0("Observer=", metadata$Observer, " | PGC=Y | PTC=N"),
          meta = out$p_meta,
          period = 0,
          time_offset = as.numeric(metadata$Longitude) * 12 / 180,
          keep_na = TRUE)

write_sef(out[,c("year","month","day","hour","minute","ta")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "ta",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "C",
          stat = "point",
          metaHead = paste0("Observer=", metadata$Observer),
          meta = out$ta_meta,
          period = 0,
          time_offset = as.numeric(metadata$Longitude) * 12 / 180,
          keep_na = TRUE)

out$hour <- NA
out$minute <- NA
write_sef(out[,c("year","month","day","hour","minute","rr")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "rr",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "mm",
          stat = "sum",
          metaHead = paste0("Observer=", metadata$Observer),
          meta = out$rr_meta,
          period = "day",
          keep_na = FALSE)


## Check SEF files
for (f in list.files(paste0("../../4_Formatted/", name_folder), full.names=TRUE)) {
  print(f)
  check_sef(f)
}