## NOTES:
## - We assumed that the observation time was 6AM, unless otherwise indicated
## - Only the barometer on the North wall was formatted
## - Additional wind directions were not formatted

library(dataresqc)
library(XLConnect)

## Create station code
name_folder <- strsplit(getwd(), "/")[[1]][8]
name_group <- strsplit(getwd(), "/")[[1]][6]
station_code <- paste(name_group, name_folder, sep = "_")

## Create output directory (if necessary)
if (!name_folder %in% list.dirs("../../4_Formatted/",full.names=FALSE)) {
  dir.create(paste0("../../4_Formatted/", name_folder))
}

## Read inventory
inventory <- readWorksheetFromFile("/scratch3/PALAEO-RA/DataRescue/Inventory_PALAEO-RA_digi.xls",
                                   sheet=1, colTypes="character")

## Define function to fill missing values with the value on the previous row
fill_variable <- function(timeseries, strings) { 
  for (i in 2:length(timeseries)) {
    if (timeseries[i] %in% strings) {
      if(!timeseries[i-1] %in% strings) {
        timeseries[i] <- timeseries[i-1]
      } 
    }
  }
  return(timeseries)
}

## Define wind directions
directions <- c("N", "NNO", "NO", "ONO", "O", "OSO", "SO", "SSO", "S", 
                "SSW", "SW", "WSW", "W", "WNW", "NW", "NNW")

## Read metadata
metadata <- inventory[which(inventory$Code == station_code), ]

## Read data
template <- readWorksheetFromFile("../../3_Revised/Boehringen/Europe_T3_DE_Boehringen_1763-1782_subdaily.xls", 
                                  sheet=1, startRow=8, header=FALSE, colTypes="character")
n <- nrow(template)

## Time
for (i in 1:3) template[,i] <- fill_variable(template[,i], NA)
hours <- rep(6, n)
minutes <- rep(0, n)
hours[which(template[,4]=="a.6")] <- 18
hours[which(template[,4]=="a.9")] <- 21
hours[which(template[,4]=="m.12")] <- 12
meta_time <- paste0("orig.time=", template[,4])

## Initialize data frame
out <- data.frame(year = as.integer(template[,1]), 
                  month = as.integer(template[,2]), 
                  day = as.integer(template[,3]), 
                  hour = hours, 
                  minute = minutes)

## Pressure
pres <- convert_pressure(as.numeric(template[,5]), 25.4, 
                         as.numeric(metadata$Latitude), as.numeric(metadata$Altitude))
out$p <- round(pres, 1)
out$p_meta <- paste0(meta_time, "|orig=", template[,5], "in")

## Temperature
out$ta <- round(as.numeric(template[,12])*1.25, 1)
out$ta_meta <- paste0(meta_time, "|orig=", template[,12], "R")

## Wind
out$dd <- round(22.5 * (match(toupper(template[,9]), directions) - 1), 0)
out$dd_meta <- paste0(meta_time, "|orig=", template[,9])

## Write SEF files
write_sef(out[, c("year","month","day","hour","minute","ta")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "ta",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "C",
          stat = "point",
          metaHead = paste0("Observer=", metadata$Observer),
          meta = out$ta_meta,
          period = 0,
          time_offset = as.numeric(metadata$Longitude) * 12 / 180,
          keep_na = FALSE)

write_sef(out[, c("year","month","day","hour","minute","p")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "p",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "hPa",
          stat = "point",
          metaHead = paste0("Observer=", metadata$Observer, " | PTC=N | PGC=Y"),
          meta = out$p_meta,
          period = 0,
          time_offset = as.numeric(metadata$Longitude) * 12 / 180,
          keep_na = TRUE)

write_sef(out[, c("year","month","day","hour","minute","dd")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "dd",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "degree",
          stat = "point",
          metaHead = paste0("Observer=", metadata$Observer),
          meta = out$dd_meta,
          period = 0,
          time_offset = as.numeric(metadata$Longitude) * 12 / 180,
          keep_na = TRUE)


## Check SEF files
for (f in list.files(paste0("../../4_Formatted/", name_folder), full.names=TRUE)) {
  print(f)
  check_sef(f)
}