## NOTE: Precipitation was already formatted in the 'Giornale Astrometeorologico' dataset, so is left out here

library(dataresqc)
library(XLConnect)

## Create station code
name_folder <- strsplit(getwd(), "/")[[1]][8]
name_group <- strsplit(getwd(), "/")[[1]][6]
station_code <- paste(name_group, name_folder, sep = "_")

## Create output directory (if necessary)
if (!name_folder %in% list.dirs("../../4_Formatted/",full.names=FALSE)) {
  dir.create(paste0("../../4_Formatted/", name_folder))
}

## Read inventory
inventory <- readWorksheetFromFile("/scratch3/PALAEO-RA/DataRescue/Inventory_PALAEO-RA_digi.xls",
                                   sheet=1, colTypes="character")

## Read metadata
metadata <- inventory[which(inventory$Code == station_code), ]

## Read data
files <- list.files("../../3_Revised/Florence/", full.names=TRUE)
template <- readWorksheetFromFile(files, sheet=1, startRow=8, header=FALSE, colTypes="character")


## MONTHLY ##

## Initialize data frame
days <- hours <- minutes <- rep("", nrow(template))
out <- data.frame(year = as.integer(template[,1]), 
                  month = as.integer(template[,2]), 
                  day = days, 
                  hour = hours, 
                  minute = minutes)

## Pressure
pres <- as.numeric(template[,9]) + as.numeric(template[,10])/12
out$p <- round(convert_pressure(pres, 27.07, as.numeric(metadata$Latitude), 
                                as.numeric(metadata$Altitude)), 1)
out$p_meta <- paste0("orig=", template[,9], ".", template[,10], "Pin")

## Wet days
out$nr <- as.numeric(template[,18])
out$nr_meta <- paste0("orig=", template[,18])

## Write SEF files
write_sef(out[, c("year","month","day","hour","minute","p")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "p",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "hPa",
          stat = "mean",
          metaHead = paste0("Observer=", metadata$Observer, " | PTC=N | PGC=Y",
                            " | Obs.times=8,12,17,24"),
          meta = out$p_meta,
          period = "month",
          note = "monthly",
          keep_na = TRUE)

write_sef(out[, c("year","month","day","hour","minute","nr")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "nr",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "days",
          stat = "sum",
          metaHead = paste0("Observer=", metadata$Observer),
          meta = out$nr_meta,
          period = "month",
          keep_na = TRUE)


## SUB-DAILY ##

## Function to extract observation times
get_times <- function(yea, mon, tim, v) {
  obstimes <- c(8,12,14,24)
  if (v == "p") obstimes[3] <- 17
  times1 <- trimws(sapply(tim, function(x) strsplit(x,"e")[[1]][1]))
  times2 <- trimws(sapply(tim, function(x) strsplit(x,"e")[[1]][2]))
  days1 <- as.integer(substr(times1, 1, 2))
  hours1 <- array(dim=length(days1))
  hours1[grep("m",times1)] <- obstimes[1]
  hours1[grep("d",times1)] <- obstimes[2]
  hours1[grep("s",times1)] <- obstimes[3]
  hours1[grep("n",times1)] <- obstimes[4]
  i <- which(!is.na(times2))
  days2 <- as.integer(substr(times2[i], 1, 2))
  days2[which(is.na(days2))] <- days1[i[which(is.na(days2))]]
  hours2 <- array(dim=length(days2))
  hours2[grep("m",times2[i])] <- obstimes[1]
  hours2[grep("d",times2[i])] <- obstimes[2]
  hours2[grep("s",times2[i])] <- obstimes[3]
  hours2[grep("n",times2[i])] <- obstimes[4]
  y <- data.frame(year = as.integer(c(yea, yea[i])), 
                  month = as.integer(c(mon, mon[i])), 
                  day = c(days1, days2), 
                  hour = c(hours1, hours2), 
                  minute = rep(0, length(hours1)+length(hours2)))
  y$minute[which(is.na(y$hour))] <- NA
  return(list(y=y,i=i))  
}

## Pressure
out1 <- get_times(template[,1], template[,2], template[,5], "p")
pres1 <- c(as.numeric(template[,3]) + as.numeric(template[,4])/12,
          as.numeric(template[out1$i,3]) + as.numeric(template[out1$i,4])/12)
out2 <- get_times(template[,1], template[,2], template[,8], "p")
pres2 <- c(as.numeric(template[,6]) + as.numeric(template[,7])/12,
           as.numeric(template[out2$i,6]) + as.numeric(template[out2$i,7])/12)
pres <- c(pres1, pres2)
out <- rbind(out1$y, out2$y)
out$p <- round(convert_pressure(pres, 27.07, as.numeric(metadata$Latitude), 
                                as.numeric(metadata$Altitude)), 1)
out$p_meta <- paste0("orig=", c(template[,3],template[out1$i,3],template[,6],template[out2$i,6]), ".", 
                     c(template[,4],template[out1$i,4],template[,7],template[out2$i,7]), "Pin | ",
                     "orig.time=", out$hour)
out <- out[which(!is.na(out$day)), ]
out <- out[order(out$year,out$month,out$day,out$hour), ]
write_sef(out[, 1:6],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "p",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "hPa",
          stat = "point",
          metaHead = paste0("Observer=", metadata$Observer, " | PTC=N | PGC=Y"),
          meta = out$p_meta,
          period = 0,
          time_offset = as.numeric(metadata$Longitude)*12/180,
          keep_na = TRUE)

## Temperature
out1 <- get_times(template[,1], template[,2], template[,13], "ta")
temp1 <- c(as.numeric(template[,11]), as.numeric(template[out1$i,11])) * 1.25
out2 <- get_times(template[,1], template[,2], template[,16], "ta")
temp2 <- c(as.numeric(template[,14]), as.numeric(template[out2$i,14])) * 1.25
out <- rbind(out1$y, out2$y)
out$ta <- round(c(temp1, temp2), 1)
out$ta_meta <- paste0("orig=", 
                      c(template[,11],template[out1$i,11],template[,14],template[out2$i,14]), 
                      "R | orig.time=", out$hour)
out <- out[which(!is.na(out$day)), ]
out <- out[order(out$year,out$month,out$day,out$hour), ]
write_sef(out[, 1:6],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "ta",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "C",
          stat = "point",
          metaHead = paste0("Observer=", metadata$Observer),
          meta = out$ta_meta,
          period = 0,
          time_offset = as.numeric(metadata$Longitude)*12/180,
          keep_na = TRUE)

## Check SEF files
for (f in list.files(paste0("../../4_Formatted/", name_folder), full.names=TRUE)) {
  print(f)
  check_sef(f)
}
