## 

library(dataresqc)
library(XLConnect)

## Create station code
name_folder <- strsplit(getwd(), "/")[[1]][8]
name_group <- strsplit(getwd(), "/")[[1]][6]
station_code <- paste(name_group, name_folder, sep = "_")

## Create output directory (if necessary)
if (!name_folder %in% list.dirs("../../4_Formatted/",full.names=FALSE)) {
  dir.create(paste0("../../4_Formatted/", name_folder))
}

## Read inventory
inventory <- readWorksheetFromFile("/scratch3/PALAEO-RA/DataRescue/Inventory_PALAEO-RA_digi.xls",
                                   sheet=1, colTypes="character")

## Read metadata
metadata <- inventory[which(inventory$Code == station_code), ]

## Read data
files <- list.files("../../3_Revised/Gdansk/", full.names=TRUE)
template <- readWorksheetFromFile(files, sheet=1, startRow=8, header=FALSE, colTypes="character")


## MONTHLY ##

## Initialize data frame
days <- hours <- minutes <- rep("", nrow(template))
out <- data.frame(year = as.integer(template[,1]), 
                  month = as.integer(template[,2]), 
                  day = days, 
                  hour = hours, 
                  minute = minutes)

## Precipitation
prec <- as.numeric(template[,3]) + as.numeric(template[,4])/12 + as.numeric(template[,5])/(12*36)
out$rr <- round(prec*27.07, 1)
out$rr_meta <- paste0("orig=", template[,3], ".", template[,4], ".", template[,5], "Pin")

## Pressure
pmax <- 25 + as.numeric(template[,6])/12
pmin <- 25 + as.numeric(template[,7])/12
out$px <- round(convert_pressure(pmax, 27.07, as.numeric(metadata$Latitude), 
                                as.numeric(metadata$Altitude)), 1)
out$px_meta <- paste0("orig=", template[,6], "Pl")
out$pn <- round(convert_pressure(pmin, 27.07, as.numeric(metadata$Latitude), 
                                 as.numeric(metadata$Altitude)), 1)
out$pn_meta <- paste0("orig=", template[,7], "Pl")

## Temperature
tmax <- (as.numeric(template[,8])-32) * 5/9
tmin <- (as.numeric(template[,9])-32) * 5/9
out$tx <- round(tmax, 1)
out$tx_meta <- paste0("orig=", template[,8], "F")
out$tn <- round(tmin, 1)
out$tn_meta <- paste0("orig=", template[,9], "F")

## Write SEF files
write_sef(out[, c("year","month","day","hour","minute","rr")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "rr",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "mm",
          stat = "sum",
          metaHead = paste0("Observer=", metadata$Observer),
          meta = out$rr_meta,
          period = "month",
          note = "monthly",
          keep_na = TRUE)

write_sef(out[, c("year","month","day","hour","minute","px")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "p",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "hPa",
          stat = "maximum",
          metaHead = paste0("Observer=", metadata$Observer, " | PTC=N | PGC=Y"),
          meta = out$px_meta,
          period = "month",
          note = "monthly-max",
          keep_na = TRUE)

write_sef(out[, c("year","month","day","hour","minute","pn")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "p",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "hPa",
          stat = "minimum",
          metaHead = paste0("Observer=", metadata$Observer, " | PTC=N | PGC=Y"),
          meta = out$pn_meta,
          period = "month",
          note = "monthly-min",
          keep_na = TRUE)

write_sef(out[, c("year","month","day","hour","minute","tx")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "ta",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "C",
          stat = "maximum",
          metaHead = paste0("Observer=", metadata$Observer),
          meta = out$tx_meta,
          period = "month",
          note = "monthly-max",
          keep_na = TRUE)

write_sef(out[, c("year","month","day","hour","minute","tn")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "ta",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "C",
          stat = "minimum",
          metaHead = paste0("Observer=", metadata$Observer),
          meta = out$tn_meta,
          period = "month",
          note = "monthly-min",
          keep_na = TRUE)


## Check SEF files
for (f in list.files(paste0("../../4_Formatted/", name_folder), full.names=TRUE)) {
  print(f)
  check_sef(f)
}
