## NB: We exclude Dorpat (Estonia) because of the unusually high correlation with stations in Italy (i.e. probably the data are not from Dorpat)

library(dataresqc)
library(XLConnect)

## Create station code
name_folder <- strsplit(getwd(), "/")[[1]][8]
name_group <- strsplit(getwd(), "/")[[1]][6]
station_code <- paste(name_group, name_folder, sep = "_")

## Create output directory (if necessary)
if (!name_folder %in% list.dirs("../../4_Formatted/",full.names=FALSE)) {
  dir.create(paste0("../../4_Formatted/", name_folder))
}

## Read inventory
inventory <- readWorksheetFromFile("/scratch3/PALAEO-RA/DataRescue/Inventory_PALAEO-RA_digi.xls",
                                   sheet=1, colTypes="character")


## Define function to fill missing values with the value on the previous row
fill_variable <- function(timeseries, strings) {
  for (i in 2:length(timeseries)) {
    if (timeseries[i] %in% strings) {
      if(!timeseries[i-1] %in% strings) {
        timeseries[i] <- timeseries[i-1]
      }
    }
  }
  return(timeseries)
}


## Read template
template <- readWorksheetFromFile("../../3_Revised/KNMI/KNMI_Yearbook_1870.xls",
                                  sheet=1, startRow=7, header=FALSE, colTypes="character")
template[,1] <- fill_variable(template[,1], NA)
template[,2] <- fill_variable(template[,2], NA)

## Loop on stations
for (st in unique(template[,1])) {
  
  xs <- template[which(template[,1]==st), ]
  st <- sub(" 1", "", st)
  st <- sub(" 2", "", st)
  metadata <- inventory[which(inventory$Station == st & 
                                inventory$Source == "KNMI Yearbook" &
                                inventory$First.Year == xs[1,3]), ]
  lat <- as.numeric(metadata$Latitude)
  alt <- as.numeric(metadata$Altitude)
  
  ## Loop on units
  for (i in 1:length(unique(xs[,2]))) {
    
    un <- unique(xs[,2])[i]
    xsu <- xs[which(xs[,2]==un), ]
    years <- as.integer(rep(xsu[,3], each=12))
    years[seq(1,length(years),12)] <- years[seq(1,length(years),12)] - 1 # because the first month is December
    months <- rep(c(12,1:11), nrow(xsu))
    
    ## Convert pressure
    pres_orig <- c(t(xsu[,4:15]), recursive=TRUE)
    pres <- as.numeric(pres_orig)
    if (un == "Millimeters 700 +") {
      pres <- convert_pressure(pres+700, 1, lat, alt)
    } else if (un == "Millimeters 600 +") {
      pres <- convert_pressure(pres+600, 1, lat, alt)
    } else if (un == "Engelsche duimen") {
      pres <- convert_pressure(pres, 25.4, lat, alt)
    } else if (un == "300 Parijsche lijnen +") {
      pres <- convert_pressure(pres+300, 27.07/12, lat, alt)
    } else if (un == "290 Parijsche lijnen +") {
      pres <- convert_pressure(pres+290, 27.07/12, lat, alt)
    } else if (un == "500 1 / 2 Russ. lijnen +") {
      pres <- convert_pressure(pres+500.5, 25.4/10/2, lat, alt)
    }
    meta_col <- paste0("orig=", pres_orig, " (", un, ")")
    
    if (i == 1) {
      out <- data.frame(Year = years,
                        Month = months,
                        Day = rep("", length(years)),
                        Hour = rep("", length(years)),
                        Minute = rep("", length(years)),
                        Value = round(pres, 1),
                        Meta = meta_col,
                        stringsAsFactors = FALSE)
    } else {
      out2 <- data.frame(Year = years,
                         Month = months,
                         Day = rep("", length(years)),
                         Hour = rep("", length(years)),
                         Minute = rep("", length(years)),
                         Value = round(pres, 1),
                         Meta = meta_col,
                         stringsAsFactors = FALSE)
      out <- rbind(out, out2)
    }
  
  }
  
  if (metadata$Station != "Dorpat") {
    write_sef(out,
            outpath = paste0("../../4_Formatted/", name_folder),
            variable = "p",
            cod = metadata$Code,
            nam = metadata$Station,
            lat = metadata$Latitude,
            lon = metadata$Longitude,
            alt = metadata$Altitude,
            sou = "PALAEO-RA",
            link = metadata$C3S.Link,
            units = "hPa",
            stat = "mean",
            meta = out$Meta,
            metaHead = "PTC=Y | PGC=Y",
            period = "month",
            note = "monthly",
            keep_na = FALSE)
  }
  
}


## Check SEF files
for (f in list.files(paste0("../../4_Formatted/", name_folder), full.names=TRUE)) {
  print(f)
  check_sef(f)
}
