## NOTE: The indoor temperature was used to reduce pressure

library(dataresqc)
library(XLConnect)

## Define function to fill missing values with the value on the previous row
fill_variable <- function(timeseries, strings) { 
  for (i in 2:length(timeseries)) {
    if (timeseries[i] %in% strings) {
      if(!timeseries[i-1] %in% strings) {
        timeseries[i] <- timeseries[i-1]
      } 
    }
  }
  return(timeseries)
}

## Create station code
name_folder <- strsplit(getwd(), "/")[[1]][8]
name_group <- strsplit(getwd(), "/")[[1]][6]
station_code <- paste(name_group, name_folder, sep = "_")

## Create output directory (if necessary)
if (!name_folder %in% list.dirs("../../4_Formatted/",full.names=FALSE)) {
  dir.create(paste0("../../4_Formatted/", name_folder))
}

## Read inventory
inventory <- readWorksheetFromFile("/scratch3/PALAEO-RA/DataRescue/Inventory_PALAEO-RA_digi.xls",
                                   sheet=1, colTypes="character")

## Read metadata
metadata <- inventory[which(inventory$Code == station_code), ]

## Read data
files <- list.files("../../3_Revised/Lyndon/", full.names=TRUE)
template <- readWorksheetFromFile(files, sheet=1, startRow=8, header=FALSE, colTypes="character")
template[,1] <- fill_variable(template[,1], NA)
template[,2] <- fill_variable(template[,2], NA)

## Time
days <- hours <- minutes <- rep("", nrow(template))


## Pressure
out <- data.frame(year = as.integer(template[,1]), 
                  month = as.integer(template[,2]), 
                  day = days, 
                  hour = hours, 
                  minute = minutes)
pres <- as.numeric(template[,6])
atb <- (as.numeric(template[,9]) - 32) * 5/9
out$p <- round(convert_pressure(pres, 25.4, as.numeric(metadata$Latitude), 
                          as.numeric(metadata$Altitude), atb), 1)
out$p_meta <- paste0("orig=", template[,6], "in|atb=", template[,9], "F")
pmax <- as.numeric(template[,4])
out$pmax <- round(convert_pressure(pmax, 25.4, as.numeric(metadata$Latitude), 
                          as.numeric(metadata$Altitude)), 1)
out$pmax_meta <- paste0("orig=", template[,4], "in")
pmin <- as.numeric(template[,5])
out$pmin <- round(convert_pressure(pmin, 25.4, as.numeric(metadata$Latitude), 
                                   as.numeric(metadata$Altitude)), 1)
out$pmin_meta <- paste0("orig=", template[,5], "in")
out <- out[which(template[,3]=="AM"), ]

write_sef(out[, c("year","month","day","hour","minute","p")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "p",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "hPa",
          stat = "mean",
          metaHead = paste0("Observer=", metadata$Observer, " | PTC=Y | PGC=Y",
                            " | Obs.times=morning"),
          meta = out$p_meta,
          period = "month",
          note = "monthly",
          keep_na = TRUE)

write_sef(out[, c("year","month","day","hour","minute","pmax")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "p",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "hPa",
          stat = "maximum",
          metaHead = paste0("Observer=", metadata$Observer, " | PTC=N | PGC=Y",
                            " | Obs.times=morning"),
          meta = out$pmax_meta,
          period = "month",
          note = "monthly-max",
          keep_na = TRUE)

write_sef(out[, c("year","month","day","hour","minute","pmin")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "p",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "hPa",
          stat = "minimum",
          metaHead = paste0("Observer=", metadata$Observer, " | PTC=N | PGC=Y",
                            " | Obs.times=morning"),
          meta = out$pmin_meta,
          period = "month",
          note = "monthly-min",
          keep_na = TRUE)


## Precipitation
out <- data.frame(year = as.integer(template[,1]), 
                  month = as.integer(template[,2]), 
                  day = days, 
                  hour = hours, 
                  minute = minutes)
out$rr <- round(as.numeric(template[,13]) * 25.4, 1)
out$rr_meta <- paste0("orig=", template[,13], "in")
out <- out[which(template[,3]%in%c("AM",NA)), ]

write_sef(out[, c("year","month","day","hour","minute","rr")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "rr",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "mm",
          stat = "sum",
          metaHead = paste0("Observer=", metadata$Observer),
          meta = out$rr_meta,
          period = "month",
          note = "monthly",
          keep_na = TRUE)


## Temperature
out <- data.frame(year = as.integer(template[,1]), 
                  month = as.integer(template[,2]), 
                  day = days, 
                  hour = hours, 
                  minute = minutes)
out$ta <- (as.numeric(template[,12]) - 32) * 5/9
out$ta_meta <- paste0(template[,12], "F")
out$tmax <- (as.numeric(template[,10]) - 32) * 5/9
out$tmax_meta <- paste0("orig=", template[,10], "F")
out$tmin <- (as.numeric(template[,11]) - 32) * 5/9
out$tmin_meta <- paste0("orig=", template[,11], "F")
out <- out[which(!is.na(template[,3])), ]
dates <- ISOdate(out$year, out$month, 1)
for (d in unique(dates)) {
  i <- which(dates==d)[1]
  out$ta_meta[i] <- paste0("orig=", out$ta_meta[i], "+", out$ta_meta[i+1])
  out$ta[i] <- round(mean(out$ta[c(i,i+1)]), 1)
  if (!is.na(out$tmax[i])) {
    out$tmax_meta[i] <- out$tmax_meta[i+which.max(out$tmax[c(i,i+1)])-1]
    out$tmax[i] <- round(max(out$tmax[c(i,i+1)]), 1)
  }
  if (!is.na(out$tmin[i])) {
    out$tmin_meta[i] <- out$tmin_meta[i+which.min(out$tmin[c(i,i+1)])-1]
    out$tmin[i] <- round(min(out$tmin[c(i,i+1)]), 1)
  }
}
out <- out[!duplicated(dates), ]

write_sef(out[, c("year","month","day","hour","minute","ta")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "ta",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "C",
          stat = "mean",
          metaHead = paste0("Observer=", metadata$Observer, 
                            " | Obs.times=morning,afternoon"),
          meta = out$ta_meta,
          period = "month",
          note = "monthly",
          keep_na = TRUE)

write_sef(out[, c("year","month","day","hour","minute","tmax")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "ta",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "C",
          stat = "maximum",
          metaHead = paste0("Observer=", metadata$Observer, 
                            " | Obs.times=morning,afternoon"),
          meta = out$tmax_meta,
          period = "month",
          note = "monthly-max",
          keep_na = TRUE)

write_sef(out[, c("year","month","day","hour","minute","tmin")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "ta",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "C",
          stat = "minimum",
          metaHead = paste0("Observer=", metadata$Observer, 
                            " | Obs.times=morning,afternoon"),
          meta = out$tmin_meta,
          period = "month",
          note = "monthly-min",
          keep_na = TRUE)


## Check SEF files
for (f in list.files(paste0("../../4_Formatted/", name_folder), full.names=TRUE)) {
  check_sef(f)
}
