## NOTES:
## Temperature (Hawksby's thermometer) was not formatted
## Pressure in 1727-1728 was not formatted because values look incorrect (especially 1728),
## and clearly inhomogeneous with respect to 1731-1732

library(dataresqc)
library(XLConnect)

## Create station code
name_folder <- strsplit(getwd(), "/")[[1]][8]
name_group <- strsplit(getwd(), "/")[[1]][6]
station_code <- paste(name_group, name_folder, sep = "_")

## Create output directory (if necessary)
if (!name_folder %in% list.dirs("../../4_Formatted/",full.names=FALSE)) {
  dir.create(paste0("../../4_Formatted/", name_folder))
}

## Read inventory
inventory <- readWorksheetFromFile("/scratch3/PALAEO-RA/DataRescue/Inventory_PALAEO-RA_digi.xls",
                                 sheet=1, colTypes="character")

## Define function to fill missing values with the value on the previous row
fill_variable <- function(timeseries, strings) {
  for (i in 2:length(timeseries)) {
    if (timeseries[i] %in% strings) {
      if(!timeseries[i-1] %in% strings) {
        timeseries[i] <- timeseries[i-1]
      }
    }
  }
  return(timeseries)
}


## 1727-1732
## Read templates
template <- readWorksheetFromFile("../../3_Revised/Naples/Europe_T3_IT_Naples_1727_monthly.xlsx",
                                  sheet=1, startRow=8, header=FALSE, colTypes="character")
template[,1] <- fill_variable(template[,1], NA)
template2 <- readWorksheetFromFile("../../3_Revised/Naples/Europe_T3_IT_Naples_1728-1732_monthly.xlsx", 
                                   sheet=1, startRow=8, header=FALSE, colTypes="character")
template2 <- template2[which(!is.na(template2[,2])), ]
template2[,1] <- fill_variable(template2[,1], NA)
n <- nrow(template2)

metadata <- inventory[which(inventory$Code == paste0(station_code,"_1")), ]

## Precipitation
prec <- c(as.numeric(template[,10]) / 23,
          as.numeric(template2[1:12,20]) + as.numeric(template2[1:12,21])/23,
          as.numeric(template2[13:n,22]))
prec <- prec * 25.4
meta_prec <- c(paste0("orig=", template[,10], "Nap.meas."),
               paste0("orig=", template2[1:12,20], "in", template2[1:12,21], "Nap.meas."),
               paste0("orig=", template2[13:n,22], "in"))

out <- data.frame(Year = as.integer(c(template[,1], template2[,1])),
                  Month = as.integer(c(template[,2], template2[,2])),
                  Day = rep("", n+12),
                  Hour = rep("", n+12),
                  Minute = rep("", n+12),
                  Value = round(prec, 1),
                  Meta = meta_prec,
                  stringsAsFactors = FALSE)

write_sef(out[, 1:6],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "rr",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "mm",
          stat = "sum",
          meta = out$Meta,
          metaHead = paste0("Observer=", metadata$Observer, " | Calendar=Julian"),
          period = "month",
          note = "monthly",
          keep_na = TRUE)


## Pressure 1731-1732
pres <- convert_pressure(as.numeric(template2[25:n,6]) + as.numeric(template2[25:n,7])/100, 
                         25.4, as.numeric(metadata$Latitude), as.numeric(metadata$Altitude))
j <- which(nchar(template2[,7])==1)
template2[j,7] <- paste0(0, template2[j,7])
meta_pres <- paste0("orig=", template2[25:n,6], ".", template2[25:n,7], "in")

out <- data.frame(Year = as.integer(template2[25:n,1]),
                  Month = as.integer(template2[25:n,2]),
                  Day = rep("", 24),
                  Hour = rep("", 24),
                  Minute = rep("", 24),
                  Value = round(pres, 1),
                  Meta = meta_pres,
                  stringsAsFactors = FALSE)

write_sef(out[, 1:6],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "p",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "hPa",
          stat = "mean",
          meta = out$Meta,
          metaHead = paste0("Observer=", metadata$Observer, 
                            " | PTC=N | PGC=Y | Obs.times=morning | Calendar=Julian"),
          period = "month",
          note = "monthly",
          keep_na = TRUE)


file.rename(list.files(paste0("../../4_Formatted/", name_folder), full.names=TRUE), 
            gsub(" ", "", list.files(paste0("../../4_Formatted/", name_folder), full.names=TRUE)))


## Check SEF files
for (f in list.files(paste0("../../4_Formatted/", name_folder), full.names=TRUE)) check_sef(f)
