## NOTES: 
## Temperature and humidity were not formatted (unknown scales)
## A correction of +0.2 inches was applied to the pressure before 4 May 1682, as per metadata (p. 128)
## Assumed that 28.8 means 28 + 7/8 and 29.1 means 29 + 0/8; not clear what is meant when a round 
## number of inches is given, but this happens very rarely so it was formatted as NA.

library(dataresqc)
library(XLConnect)

## Create output directory (if necessary)
name_folder <- strsplit(getwd(), "/")[[1]][8]
name_group <- strsplit(getwd(), "/")[[1]][6]
if (!name_folder %in% list.dirs("../../4_Formatted/",full.names=FALSE)) {
  dir.create(paste0("../../4_Formatted/", name_folder))
}

## Read inventory
inventory <- readWorksheetFromFile("/scratch3/PALAEO-RA/DataRescue/Inventory_PALAEO-RA_digi.xls",
                                 sheet=1, colTypes="character")

## Define function to fill missing values with the value on the previous row
fill_variable <- function(timeseries, strings) {
  for (i in 2:length(timeseries)) {
    if (timeseries[i] %in% strings) {
      if(!timeseries[i-1] %in% strings) {
        timeseries[i] <- timeseries[i-1]
      }
    }
  }
  return(timeseries)
}

## Locke (1666-1683)
station_code <- paste(name_group, name_folder, 1, sep = "_")
metadata <- inventory[which(inventory$Code == station_code & inventory$Resolution=="subdaily"), ]

## Read data
template <- readWorksheetFromFile("../../3_Revised/Oxford/Europe_T3_GB_Oxford_Locke_1666-1683_subdaily.xls",
                                  sheet=1, startRow=9, header=FALSE, colTypes="character")
template[,1] <- fill_variable(template[,1], NA)
template[,2] <- fill_variable(template[,2], NA)
template[,3] <- fill_variable(template[,3], NA)

## Time
dates <- as.Date(paste(template[,1], template[,2], template[,3], sep="-"))
dates <- dates + 10 # convert from Julian to Gregorian calendar
years <- as.integer(format(dates,"%Y"))
months <- as.integer(format(dates,"%m"))
days <- as.integer(format(dates,"%d"))
hours <- as.integer(substr(template[,4], 12, 13))
minutes <- as.integer(substr(template[,4], 15, 16))
meta_time <- paste0("orig.date=", dates-10, "|orig.time=", substr(template[,4],12,16))

## Pressure
template[,6] <- fill_variable(template[,6], NA)
pres <- as.numeric(template[,6]) + (as.numeric(template[,7]) - 1) / 8
j <- which(dates < (as.Date("1682-05-04")+10))
pres[j] <- pres[j] + 0.2
pres <- convert_pressure(pres, 25.4, as.numeric(metadata$Latitude), as.numeric(metadata$Altitude))
meta_col <- paste0(meta_time, "|orig=", template[,6], ".", template[,7], "in") 
meta_col[j] <- sub("in", "+0.2in", meta_col[j])
out <- data.frame(Year = years,
                  Month = months,
                  Day = days,
                  Hour = hours,
                  Minute = minutes,
                  Value = round(pres, 1),
                  Meta = meta_col,
                  stringsAsFactors = FALSE)
write_sef(out[, 1:6],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "p",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "hPa",
          stat = "point",
          meta = out$Meta,
          metaHead = paste0("Observer=", metadata$Observer, " | PTC=N | PGC=Y"),
          period = 0,
          time_offset = as.numeric(metadata$Longitude)*12/180,
          keep_na = TRUE)

## Wind direction
directions <- c("N", "NNE", "NE", "ENE", "E", "ESE", "SE", "SSE", "S",
                "SSW", "SW", "WSW", "W", "WNW", "NW", "NNW")
wdir <- 22.5 * (match(toupper(template[,9]), directions) - 1)
out <- data.frame(Year = years,
                  Month = months,
                  Day = days,
                  Hour = hours,
                  Minute = minutes,
                  Value = round(wdir, 0),
                  Meta = paste0(meta_time, "|orig=", template[,9]),
                  stringsAsFactors = FALSE)
write_sef(out[, 1:6],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "dd",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "degree",
          stat = "point",
          meta = out$Meta,
          metaHead = paste0("Observer=", metadata$Observer),
          period = 0,
          time_offset = as.numeric(metadata$Longitude)*12/180,
          keep_na = TRUE)

## Wind force
out <- data.frame(Year = years,
                  Month = months,
                  Day = days,
                  Hour = hours,
                  Minute = minutes,
                  Value = as.integer(template[,10]),
                  Meta = paste0(meta_time, "|orig=", template[,10]),
                  stringsAsFactors = FALSE)
write_sef(out[, 1:6],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "w",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "0-4",
          stat = "point",
          meta = out$Meta,
          metaHead = paste0("Observer=", metadata$Observer),
          period = 0,
          time_offset = as.numeric(metadata$Longitude)*12/180,
          keep_na = TRUE)


## Plot (1683-1684)
station_code <- paste(name_group, name_folder, 2, sep = "_")
metadata <- inventory[which(inventory$Code == station_code & inventory$Resolution=="subdaily"), ]

## Read data
template <- read.table("../../3_Revised/Oxford/Europe_T3_GB_Oxford_Plot_1683-1684_subdaily-pressure.txt",
                       sep="\t", header=TRUE)

## Time
dates <- as.Date(paste(template[,1], template[,2], template[,3], sep="-"))
dates <- dates + 10 # convert from Julian to Gregorian calendar
years <- as.integer(format(dates,"%Y"))
months <- as.integer(format(dates,"%m"))
days <- as.integer(format(dates,"%d"))
hours <- template$Hour
minutes <- rep(0, nrow(template))
meta_time <- paste0("orig.date=", dates-10, "|orig.time=", template$Hour)

## Pressure
pres <- convert_pressure(template$Pressure, 25.4, as.numeric(metadata$Latitude), as.numeric(metadata$Altitude))
out <- data.frame(Year = years,
                  Month = months,
                  Day = days,
                  Hour = hours,
                  Minute = minutes,
                  Value = round(pres, 1),
                  Meta = paste0(meta_time, "|orig=", template$Pressure, "in"),
                  stringsAsFactors = FALSE)
write_sef(out[, 1:6],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "p",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "hPa",
          stat = "point",
          meta = out$Meta,
          metaHead = paste0("Observer=", metadata$Observer, " | PTC=N | PGC=Y"),
          period = 0,
          time_offset = as.numeric(metadata$Longitude)*12/180,
          keep_na = TRUE)


## Check SEF files
for (f in list.files(paste0("../../4_Formatted/", name_folder), full.names=TRUE)) check_sef(f)
