## NOTES:

library(dataresqc)
library(XLConnect)

## Create station code
name_folder <- strsplit(getwd(), "/")[[1]][8]
name_group <- strsplit(getwd(), "/")[[1]][6]
station_code <- paste(name_group, name_folder, sep = "_")

## Create output directory (if necessary)
if (!name_folder %in% list.dirs("../../4_Formatted/",full.names=FALSE)) {
  dir.create(paste0("../../4_Formatted/", name_folder))
}

## Read inventory
inventory <- readWorksheetFromFile("/scratch3/PALAEO-RA/DataRescue/Inventory_PALAEO-RA_digi.xls",
                                 sheet=1, colTypes="character")

## Define function to fill missing values with the value on the previous row
fill_variable <- function(timeseries, strings) {
  for (i in 2:length(timeseries)) {
    if (timeseries[i] %in% strings) {
      if(!timeseries[i-1] %in% strings) {
        timeseries[i] <- timeseries[i-1]
      }
    }
  }
  return(timeseries)
}


## Read template
template <- readWorksheetFromFile("../../3_Revised/Risinge/Europe_T3_SE_Risinge_1728-1730_monthly.xlsx",
                                  sheet=1, startRow=8, header=FALSE, colTypes="character")
template[,1] <- fill_variable(template[,1], NA)
metadata <- inventory[which(inventory$Code == station_code), ]
for (j in 3:8) { # add leading zeros
  k <- which(nchar(template[,j])==1)
  template[k,j] <- paste0("0", template[k,j])
}

## Precipitation
prec <- as.numeric(template[13:36,9]) * 25.4
meta_prec <- paste0("orig=", template[13:36,9], "in")

out <- data.frame(Year = as.integer(template[13:36,1]),
                  Month = as.integer(template[13:36,2]),
                  Day = rep("", 24),
                  Hour = rep("", 24),
                  Minute = rep("", 24),
                  Value = round(prec, 1),
                  Meta = meta_prec,
                  stringsAsFactors = FALSE)

write_sef(out[, 1:6],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "rr",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "mm",
          stat = "sum",
          meta = out$Meta,
          metaHead = paste0("Observer=", metadata$Observer, " | Calendar=Julian"),
          period = "month",
          note = "monthly",
          keep_na = TRUE)

## Mid-range pressure
pres <- (as.numeric(template[1:12,3]) + as.numeric(template[1:12,4])/100 + 
  as.numeric(template[1:12,7]) + as.numeric(template[1:12,8])/100) / 2
pres <- convert_pressure(pres, 25.4, as.numeric(metadata$Latitude),
                         as.numeric(metadata$Altitude))
meta_pres <- paste0("orig=", template[1:12,3], ".", template[1:12,4], "+",
                    template[1:12,7], ".", template[1:12,8], "in")

out <- data.frame(Year = as.integer(template[1:12,1]),
                  Month = as.integer(template[1:12,2]),
                  Day = rep("", 12),
                  Hour = rep("", 12),
                  Minute = rep("", 12),
                  Value = round(pres, 1),
                  Meta = meta_pres,
                  stringsAsFactors = FALSE)

write_sef(out[, 1:6],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "p",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "hPa",
          stat = "mid_range",
          meta = out$Meta,
          metaHead = paste0("Observer=", metadata$Observer, " | PGC=Y | PTC=N | Calendar=Julian"),
          period = "month",
          note = "monthly",
          keep_na = TRUE)

## Mean pressure
pres <- as.numeric(template[13:36,5]) + as.numeric(template[13:36,6])/100
pres <- convert_pressure(pres, 25.4, as.numeric(metadata$Latitude),
                         as.numeric(metadata$Altitude))
meta_pres <- paste0("orig=", template[13:36,5], ".", template[13:36,6], "in")

out <- data.frame(Year = as.integer(template[13:36,1]),
                  Month = as.integer(template[13:36,2]),
                  Day = rep("", 24),
                  Hour = rep("", 24),
                  Minute = rep("", 24),
                  Value = round(pres, 1),
                  Meta = meta_pres,
                  stringsAsFactors = FALSE)

write_sef(out[, 1:6],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "p",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "hPa",
          stat = "mean",
          meta = out$Meta,
          metaHead = paste0("Observer=", metadata$Observer, " | PGC=Y | PTC=N | Obs.times=morning | Calendar=Julian"),
          period = "month",
          note = "monthly",
          keep_na = TRUE)

file.rename(list.files(paste0("../../4_Formatted/", name_folder), full.names=TRUE), 
            gsub(" ", "", list.files(paste0("../../4_Formatted/", name_folder), full.names=TRUE)))


## Check SEF files
for (f in list.files(paste0("../../4_Formatted/", name_folder), full.names=TRUE)) check_sef(f)