## Note: Observation times assumed to be at 0800 and 1600 UTC 

library(dataresqc)
library(XLConnect)
library(suncalc)
library(lubridate)

## Create station code
name_folder <- strsplit(getwd(), "/")[[1]][8]
name_group <- strsplit(getwd(), "/")[[1]][6]
station_code <- paste(name_group, name_folder, 1, sep = "_")

## Create output directory (if necessary)
if (!name_folder %in% list.dirs("../../4_Formatted/",full.names=FALSE)) {
  dir.create(paste0("../../4_Formatted/", name_folder))
}

## Read inventory
inventory <- readWorksheetFromFile("/scratch3/PALAEO-RA/DataRescue/Inventory_PALAEO-RA_digi.xls",
                                   sheet=1, colTypes="character")

## Define function to fill missing values with the value on the previous row
fill_variable <- function(timeseries, strings) { 
  for (i in 2:length(timeseries)) {
    if (timeseries[i] %in% strings) {
      if(!timeseries[i-1] %in% strings) {
        timeseries[i] <- timeseries[i-1]
      } 
    }
  }
  return(timeseries)
}


## Read filenames
files <- list.files("../../3_Revised/Rovereto/", pattern="xls", full.names=TRUE)
files <- files[c(1:3,5,4)]
out <- list()


## Rovereto_1 (1800-1839)

## Read metadata
metadata <- inventory[which(inventory$Code == station_code), ]

## Loop over files
for (i in 1:4) {
  
  template <- readWorksheetFromFile(files[i], sheet=1, startRow=8, header=FALSE, colTypes="character")
  
  ## Time
  dates <- as.Date(paste(template[,1], template[,2], template[,3], sep="-"))
#  template$t1 <- getSunlightTimes(dates, as.numeric(metadata$Latitude), 
#                                  as.numeric(metadata$Longitude), keep="sunrise", tz="GMT")$sunrise
#  template$t2 <-  getSunlightTimes(dates, as.numeric(metadata$Latitude), 
#                                   as.numeric(metadata$Longitude), keep="sunset", tz="GMT")$sunset - 3600
  ko <- which(is.na(dates))
  if (length(ko) > 0) template <- template[-ko, ] # Remove false February 29ths
#  hours <- c(hour(template$t1), hour(template$t2))
#  minutes <- c(minute(template$t1), minute(template$t2))
  hours <- c(rep(8, nrow(template)), rep(16, nrow(template)))
  minutes <- rep(0, 2*nrow(template))
#  meta_time1 <- rep("orig.time=sunrise", nrow(template))
#  meta_time2 <- rep("orig.time=sunset-1h", nrow(template))
  meta_time1 <- rep("orig.time=mattina", nrow(template))
  meta_time2 <- rep("orig.time=dopo pranzo", nrow(template))
  
  ## Initialize data frames
  out[[i]] <- data.frame(year = rep(as.integer(template[,1]), 2), 
                         month = rep(as.integer(template[,2]), 2), 
                         day = rep(as.integer(template[,3]), 2), 
                         hour = hours, 
                         minute = minutes)
  
  ## Temperature
  template[,5] <- fill_variable(template[,5], NA)
  template[,7] <- fill_variable(template[,7], NA)
  template[which(template[,5]=="NA"), 5] <- NA
  template[which(template[,7]=="NA"), 7] <- NA
  pm <- rep(1, nrow(template)*2)
  pm[which(template[,5]=="-")] <- -1
  pm[nrow(template)+which(template[,7]=="-")] <- -1
  pm[which(is.na(template[,5]))] <- NA
  pm[nrow(template)+which(is.na(template[,7]))] <- NA
  temp <- c(as.numeric(template[,4]), as.numeric(template[,6])) * 1.25 * pm
  out[[i]]$ta <- round(temp, 1)
  out[[i]]$ta_meta <- c(paste0(meta_time1, "|orig=", template[,5], template[,4], "R"),
                        paste0(meta_time2, "|orig=", template[,7], template[,6], "R"))
  
  ## Pressure (assumed to be observed in the morning when there are two temperature observations)
  template[,9] <- fill_variable(template[,9], NA)
  template[which(template[,9]=="NA"), 9] <- NA
  pm <- rep(1, nrow(template)*2)
  pm[which(template[,9]=="-")] <- -1
  pm[which(is.na(template[,9]))] <- NA
  if (i <= 2) {
    template[,11] <- fill_variable(template[,11], NA)
    template[which(template[,11]=="NA"), 11] <- NA
    pm[nrow(template)+which(template[,11]=="-")] <- -1
    pm[nrow(template)+which(is.na(template[,11]))] <- NA
    pres <- 27.5 + c(as.numeric(template[,8]), as.numeric(template[,10])) * pm / 12
    out[[i]]$p_meta <- c(paste0(meta_time1, "|orig=", template[,9], template[,8], "l"),
                         paste0(meta_time2, "|orig=", template[,11], template[,10], "l"))
  } else {
    pres <- 27.5 + c(as.numeric(template[,8]), rep(NA,nrow(template))) * pm / 12
    out[[i]]$p_meta <- c(paste0(meta_time1, "|orig=", template[,9], template[,8], "l"),
                         rep(NA,nrow(template)))
  }
  out[[i]]$p <- round(convert_pressure(pres, 27.07, as.numeric(metadata$Latitude), 
                                       as.numeric(metadata$Altitude)), 1)
  
}

## Merge data frames and order chronologically
out <- do.call(rbind, out)
out <- out[order(out$year, out$month, out$day, out$hour), ]

## Write SEF files
file.remove(list.files(paste0("../../4_Formatted/", name_folder), full.names=TRUE))

write_sef(out[,c("year","month","day","hour","minute","p")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "p",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "hPa",
          stat = "point",
          metaHead = paste0("Observer=", metadata$Observer, "| PGC=Y | PTC=N"),
          meta = out$p_meta,
          period = 0,
          keep_na = FALSE)

write_sef(out[,c("year","month","day","hour","minute","ta")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "ta",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "C",
          stat = "point",
          metaHead = paste0("Observer=", metadata$Observer),
          meta = out$ta_meta,
          period = 0,
          keep_na = FALSE)


## Rovereto_2 (1827-1829)
i <- i + 1
template <- readWorksheetFromFile(files[i], sheet=1, startRow=8, header=FALSE, colTypes="character")

## Read metadata
station_code <- sub("1", "2", station_code)
metadata <- inventory[which(inventory$Code == station_code), ]

## Time
hours <- c(rep(7,nrow(template)), rep(14,nrow(template)))
minutes <- rep(0, 2*nrow(template))
meta_time <-  c(rep("orig.time=7AM",nrow(template)), rep("orig.time=2PM",nrow(template)))

## Initialize data frames
out <- data.frame(year = rep(as.integer(template[,1]), 2), 
                       month = rep(as.integer(template[,2]), 2), 
                       day = rep(as.integer(template[,3]), 2), 
                       hour = hours, 
                       minute = minutes)

## Correct morning times
j <- which(out$year==1829 & out$month==12 & out$hour==7)
out$minute[j] <- 30
meta_time[j] <- sub("7AM", "7:30AM", meta_time[j])
j <- which((out$month%in%c(3,10) | (out$year==1829 & out$month==2)) & out$hour==7)
out$hour[j] <- 6
out$minute[j] <- 30
meta_time[j] <- sub("7AM", "6:30AM", meta_time[j])
j <- which(out$month%in%c(4,9) & out$hour==7)
out$hour[j] <- 6
meta_time[j] <- sub("7AM", "6AM", meta_time[j])
j <- which(out$month%in%c(5,8) & out$hour==7)
out$hour[j] <- 5
out$minute[j] <- 30
meta_time[j] <- sub("7AM", "5:30AM", meta_time[j])
j <- which(out$month%in%c(6,7) & out$hour==7)
out$hour[j] <- 5
meta_time[j] <- sub("7AM", "5AM", meta_time[j])

## Temperature
temp <- c(as.numeric(template[,6]), as.numeric(template[,11])) * 1.25
out$ta <- round(temp, 1)
out$ta_meta <- paste0(meta_time, "|orig=", c(template[,6], template[,11]), "R")

## Pressure
template[which(!is.na(template[,4]) & is.na(template[,5])), 5] <- 0
template[which(!is.na(template[,9]) & is.na(template[,10])), 10] <- 0
pres <- c(as.numeric(template[,4])+as.numeric(template[,5])/12, 
          as.numeric(template[,9])+as.numeric(template[,10])/12)
out$p <- round(convert_pressure(pres, 27.07, as.numeric(metadata$Latitude), 
                                     as.numeric(metadata$Altitude)), 1)
out$p_meta <- c(paste0(meta_time[1:nrow(template)], "|orig=", 
                       template[,4], ".", template[,5], "Pin"),
                paste0(meta_time[(nrow(template)+1):(2*nrow(template))], "|orig=", 
                       template[,9], ".", template[,10], "Pin"))

## Wind
directions <- c("N", "NNE", "NE", "ENE", "E", "ESE", "SE", "SSE", "S", 
                "SSO", "SO", "OSO", "O", "ONO", "NO", "NNO")
wdir <- c(template[,8], template[,13])
out$dd <- round(22.5 * (match(toupper(wdir), directions) - 1), 0)
out$dd_meta <- paste0(meta_time, "|orig=", wdir)

## Write SEF files
out <- out[order(out$year, out$month, out$day, out$hour), ]

write_sef(out[,c("year","month","day","hour","minute","p")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "p",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "hPa",
          stat = "point",
          metaHead = paste0("Observer=", metadata$Observer, "| PGC=Y | PTC=N"),
          meta = out$p_meta,
          period = 0,
          time_offset = as.numeric(metadata$Longitude) * 12 / 180,
          keep_na = TRUE)

write_sef(out[,c("year","month","day","hour","minute","ta")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "ta",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "C",
          stat = "point",
          metaHead = paste0("Observer=", metadata$Observer),
          meta = out$ta_meta,
          period = 0,
          time_offset = as.numeric(metadata$Longitude) * 12 / 180,
          keep_na = TRUE)

write_sef(out[,c("year","month","day","hour","minute","dd")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "dd",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "degree",
          stat = "point",
          metaHead = paste0("Observer=", metadata$Observer),
          meta = out$dd_meta,
          period = 0,
          time_offset = as.numeric(metadata$Longitude) * 12 / 180,
          keep_na = TRUE)


## Rovereto_3 (1860-1868)
file.copy(list.files("../../3_Revised/Rovereto/",pattern="tsv",full.names=TRUE), "../../4_Formatted/Rovereto/")


## Check SEF files
for (f in list.files(paste0("../../4_Formatted/", name_folder), full.names=TRUE)) {
  print(f)
  check_sef(f)
}
