## NOTES: 
## In case of duplicated years only the first one appearing was kept
## For precipitation we assumed that the Russian inch was identical to the British inch

library(dataresqc)
library(XLConnect)


## Create output directory (if necessary)
name_folder <- strsplit(getwd(), "/")[[1]][8]
name_group <- strsplit(getwd(), "/")[[1]][6]
if (!name_folder %in% list.dirs("../../4_Formatted/",full.names=FALSE)) {
  dir.create(paste0("../../4_Formatted/", name_folder))
}

## Read inventory
inventory <- readWorksheetFromFile("/scratch3/PALAEO-RA/DataRescue/Inventory_PALAEO-RA_digi.xls",
                                   sheet=1, colTypes="character")

## Define function to fill missing values with the value on the previous row
fill_variable <- function(timeseries, strings) {
  for (i in 2:length(timeseries)) {
    if (timeseries[i] %in% strings) {
      if(!timeseries[i-1] %in% strings) {
        timeseries[i] <- timeseries[i-1]
      }
    }
  }
  return(timeseries)
}


### TEMPERATURE ###

## Read template
template <- readWorksheetFromFile("../../3_Revised/Russia/Temperature_Russian_Empire_T3.xlsx",
                                  sheet=1, startRow=8, header=FALSE, colTypes="character")
for (i in c(1:8,24)) template[,i] <- fill_variable(template[,i], NA)
template[,24] <- sub("J", "Julian", template[,24])
template[,24] <- sub("I2", "Interpolated (1/2 + 1/2)", template[,24])
template[,24] <- sub("I3", "Interpolated (1/3 + 2/3)", template[,24])

## Loop on stations
codes <- unique(template[,1])
# n <- length(codes)
# x <- data.frame(id=rep(NA,n),st=rep(NA,n),lat=rep(NA,n),lon=rep(NA,n),
#                 ele=rep(NA,n),y1=rep(NA,n),y2=rep(NA,n))
for (i in 1:length(codes)) {

  cals <- template[which(template[,1]==codes[i]), 24]
  for (cal in unique(cals)) { ## Loop on calendar styles
  
    ## Read data
    xs <- template[which(template[,1]==codes[i] & template[,24]==cal), ]
    xs <- xs[which(substr(xs[,9],1,1) == "1"), ] # Remove means
    xs <- xs[!duplicated(xs[,9]), ] # Remove duplicated years
  
    ## Read metadata
    id <- paste0(name_group, "_", name_folder, "-", codes[i])
  # x$id[i] <- id
  # x$st[i] <- xs[1,8]
  # x$lat[i] <- as.numeric(xs[1,2]) + as.numeric(xs[1,3]) / 60
  # x$lon[i] <- as.numeric(xs[1,4]) + as.numeric(xs[1,5]) / 60 - 20 + 2.35
  # x$ele[i] <- as.numeric(xs[1,6]) * 0.3048
  # if (!is.na(x$ele[i]) & xs[1,7] == "f.") x$ele[i] <- x$ele[i] * 0.3248/0.3048
  # x$lat[i] <- round(x$lat[i], 2)
  # x$lon[i] <- round(x$lon[i], 2)
  # x$ele[i] <- round(x$ele[i], 0)
  # x$y1[i] <- xs[1,9]
  # x$y2[i] <- xs[which.max(as.integer(xs[,9])),9]
    metadata <- inventory[which(inventory$Code == id), ]
  
    temp <- as.numeric(c(t(xs[,10:21]),recursive=FALSE)) * 1.25
    meta_col <- paste0("orig=", c(t(xs[,10:21]),recursive=FALSE), "R")
    meta_head <- ifelse(cal == "G", "", paste0("Calendar=",cal))
  
    out <- data.frame(Year = rep(as.integer(xs[,9]), each=12),
                    Month = rep(1:12, nrow(xs)),
                    Day = rep("", nrow(xs)),
                    Hour = rep("", nrow(xs)),
                    Minute = rep("", nrow(xs)),
                    Value = round(temp, 2),
                    Meta = meta_col,
                    stringsAsFactors = FALSE)
   
    write_sef(out,
            outpath = paste0("../../4_Formatted/", name_folder),
            variable = "ta",
            cod = metadata$Code,
            nam = metadata$Station,
            lat = metadata$Latitude,
            lon = metadata$Longitude,
            alt = metadata$Altitude,
            sou = "PALAEO-RA",
            link = metadata$C3S.Link,
            units = "C",
            stat = "mean",
            metaHead = meta_head,
            meta = out$Meta,
            period = "month",
            note = "monthly",
            keep_na = TRUE)
  
  }

}


### PRECIPITATION ###

## Read template
template <- readWorksheetFromFile("../../3_Revised/Russia/Precipitation_Russian_Empire_T3.xlsx",
                                  sheet=1, startRow=8, header=FALSE, colTypes="character")
for (i in c(1,17:18)) template[,i] <- fill_variable(template[,i], NA)
template[,18] <- sub("J", "Julian", template[,18])
template[,18] <- sub("I2", "Interpolated (1/2 + 1/2)", template[,18])
template[,18] <- sub("I3", "Interpolated (1/3 + 2/3)", template[,18])

## Loop on stations
codes <- unique(template[,1])
for (i in 1:length(codes)) {
  
  ## Read data
  xs <- template[which(template[,1]==codes[i]), ]
  xs <- xs[which(substr(xs[,2],1,1) == "1"), ] # Remove means
  xs <- xs[!duplicated(xs[,2]), ] # Remove duplicated years
  cal <- unique(xs[,18]) # Read calendar style
  if (length(cal) > 1) stop("More than one calendar style")

  ## Read metadata
  id <- paste0(name_group, "_", name_folder, "-", xs[1,17])
  metadata <- inventory[which(inventory$Code == id), ]
  
  prec <- as.numeric(c(t(xs[,3:14]),recursive=FALSE)) * 25.4
  meta_col <- paste0("orig=", c(t(xs[,3:14]),recursive=FALSE), "in")
  meta_head <- ifelse(cal == "G", "", paste0("Calendar=",cal))
  
  out <- data.frame(Year = rep(as.integer(xs[,2]), each=12),
                    Month = rep(1:12, nrow(xs)),
                    Day = rep("", nrow(xs)),
                    Hour = rep("", nrow(xs)),
                    Minute = rep("", nrow(xs)),
                    Value = round(prec, 1),
                    Meta = meta_col,
                    stringsAsFactors = FALSE)
  
  write_sef(out,
            outpath = paste0("../../4_Formatted/", name_folder),
            variable = "rr",
            cod = metadata$Code,
            nam = metadata$Station,
            lat = metadata$Latitude,
            lon = metadata$Longitude,
            alt = metadata$Altitude,
            sou = "PALAEO-RA",
            link = metadata$C3S.Link,
            units = "mm",
            stat = "sum",
            metaHead = meta_head,
            meta = out$Meta,
            period = "month",
            note = "monthly",
            keep_na = TRUE)
  
}


### WET DAYS ###

## Read template
template <- readWorksheetFromFile("../../3_Revised/Russia/WetDays_Russian_Empire_T3.xlsx",
                                  sheet=1, startRow=8, header=FALSE, colTypes="character")
for (i in c(1,17:18)) template[,i] <- fill_variable(template[,i], NA)
template[,18] <- sub("J", "Julian", template[,18])
template[,18] <- sub("I2", "Interpolated (1/2 + 1/2)", template[,18])
template[,18] <- sub("I3", "Interpolated (1/3 + 2/3)", template[,18])

## Loop on stations
codes <- unique(template[,1])
for (i in 1:length(codes)) {
  
  ## Read data
  xs <- template[which(template[,1]==codes[i]), ]
  xs <- xs[which(substr(xs[,2],1,1) == "1"), ] # Remove means
  xs <- xs[!duplicated(xs[,2]), ] # Remove duplicated years
  cal <- unique(xs[,18]) # Read calendar style
  if (length(cal) > 1) stop("More than one calendar style")
  
  ## Read metadata
  if (xs[1,17] == "NA") {
    id <- paste0(name_group, "_", name_folder, "-", xs[1,1], "r")
  } else {
    id <- paste0(name_group, "_", name_folder, "-", xs[1,17])
  }
  metadata <- inventory[which(inventory$Code == id), ]

  wetdays <- as.integer(c(t(xs[,3:14]),recursive=FALSE))
  meta_head <- ifelse(cal == "G", "", paste0("Calendar=",cal))
  
  out <- data.frame(Year = rep(as.integer(xs[,2]), each=12),
                    Month = rep(1:12, nrow(xs)),
                    Day = rep("", nrow(xs)),
                    Hour = rep("", nrow(xs)),
                    Minute = rep("", nrow(xs)),
                    Value = wetdays,
                    stringsAsFactors = FALSE)
  
  write_sef(out,
            outpath = paste0("../../4_Formatted/", name_folder),
            variable = "nr",
            cod = metadata$Code,
            nam = metadata$Station,
            lat = metadata$Latitude,
            lon = metadata$Longitude,
            alt = metadata$Altitude,
            sou = "PALAEO-RA",
            link = metadata$C3S.Link,
            units = "days",
            stat = "sum",
	    metaHead = meta_head,
            period = "month",
            keep_na = TRUE)
  
}


## Check SEF files
for (f in list.files(paste0("../../4_Formatted/", name_folder), full.names=TRUE)) {
  print(f)
  check_sef(f)
}
