## NOTE: For monthly only version 2 (pressure and temperature) is formatted

library(dataresqc)
library(XLConnect)
#library(suncalc)
#library(lubridate)

## Create station code
name_folder <- strsplit(getwd(), "/")[[1]][8]
name_group <- strsplit(getwd(), "/")[[1]][6]
station_code <- paste(name_group, name_folder, sep = "_")

## Create output directory (if necessary)
if (!name_folder %in% list.dirs("../../4_Formatted/",full.names=FALSE)) {
  dir.create(paste0("../../4_Formatted/", name_folder))
}

## Read inventory
inventory <- readWorksheetFromFile("/scratch3/PALAEO-RA/DataRescue/Inventory_PALAEO-RA_digi.xls",
                                   sheet=1, colTypes="character")

## Define function to fill missing values with the value on the previous row
fill_variable <- function(timeseries, strings) {
  for (i in 2:length(timeseries)) {
    if (timeseries[i] %in% strings) {
      if(!timeseries[i-1] %in% strings) {
        timeseries[i] <- timeseries[i-1]
      }
    }
  }
  return(timeseries)
}


## MONTHLY

## Read filenames
files <- list.files("../../3_Revised/StPetersburg/", pattern="monthly-version2", full.names=TRUE)
out <- list()

## Read metadata
metadata <- inventory[which(inventory$Code == station_code & inventory$Resolution=="monthly")[2], ]

## Loop over files
for (i in 1:length(files)) {
  
  ## Read data
  template <- readWorksheetFromFile(files[i], sheet=1, startRow=8, header=FALSE, colTypes="character")
  template[,1] <- fill_variable(template[,1], NA)
  
  ## Mid-range pressure
  mrp <- (as.numeric(template[,3]) + as.numeric(template[,4])) / 2
  mrp <- round(convert_pressure(mrp, ifelse(i==1,25.4,27.07), as.numeric(metadata$Latitude),
                                as.numeric(metadata$Altitude)), 1)
  mrp_meta <- paste0("orig=", template[,3], ifelse(i==1,"in+","Pin+"), 
                     template[,4], ifelse(i==1,"in","Pin"))
  
  ## Mean pressure
  pres <- round(convert_pressure(as.numeric(template[,ifelse(i>2,6,7)]), 27.07,
                                 as.numeric(metadata$Latitude), as.numeric(metadata$Altitude)), 1)
  pres_meta <- paste0("orig=", template[,ifelse(i>2,6,7)], "Pin")
  
  ## Minimum temperature
  if (i > 2) {
    tmin <- round(as.numeric(template[,11]) * 1.25, 1)
    tmin_meta <- paste0("orig=", template[,11], "R")
  } else {
    tmin <- round(100 - as.numeric(template[,8]) * 2/3, 1)
    tmin_meta <- paste0("orig=", template[,8], "D")
  }
  
  ## Maximum temperature
  if (i > 2) {
    tmax <- round(as.numeric(template[,10]) * 1.25, 1)
    tmax_meta <- paste0("orig=", template[,10], "R")
  } else {
    tmax <- round(100 - as.numeric(template[,9]) * 2/3, 1)
    tmax_meta <- paste0("orig=", template[,9], "D")
  }
  
  ## Mean temperature
  if (i == 3) {
    temp <- round(as.numeric(template[,12]) * 1.25, 1)
    temp_meta <- paste0("orig=", template[,12], "R")
  } else if (i == 2) {
    temp <- round(100 - as.numeric(template[,14]) * 2/3, 1)
    temp_meta <- paste0("orig=", template[,14], "D")
  } else {
    temp <- NA
    temp_meta <- NA
  }
  
  ## Create output data frame
  out[[i]] <- data.frame(year=as.integer(template[,1]),
                         month=as.integer(template[,2]),
                         day=rep(NA,nrow(template)),
                         hour=rep(NA,nrow(template)),
                         minute=rep(NA,nrow(template)),
                         mrp, mrp_meta,
                         pres, pres_meta,
                         tmin, tmin_meta,
                         tmax, tmax_meta,
                         temp, temp_meta)
}

## Add 1724-1725
i <- i + 1
template <- readWorksheetFromFile("../../3_Revised/StPetersburg/Europe_T3_RU_StPetersburg_1724-1725_monthly.xlsx", 
                                  sheet=1, startRow=8, header=FALSE, colTypes="character")
template[,1] <- fill_variable(template[,1], NA)
mrp <- (as.numeric(template[,3]) + as.numeric(template[,5])) / 2
mrp <- round(convert_pressure(mrp, 25.4, as.numeric(metadata$Latitude),
                              as.numeric(metadata$Altitude)), 1)
mrp_meta <- paste0("orig=", template[,3], "in+", template[,5], "in")
out[[i]] <- data.frame(year=as.integer(template[,1]),
                       month=as.integer(template[,2]),
                       day=rep(NA,nrow(template)),
                       hour=rep(NA,nrow(template)),
                       minute=rep(NA,nrow(template)),
                       mrp, mrp_meta,
                       pres=rep(NA,nrow(template)), 
                       pres_meta=rep(NA,nrow(template)),
                       tmin=rep(NA,nrow(template)), 
                       tmin_meta=rep(NA,nrow(template)),
                       tmax=rep(NA,nrow(template)), 
                       tmax_meta=rep(NA,nrow(template)),
                       temp=rep(NA,nrow(template)), 
                       temp_meta=rep(NA,nrow(template)))

## Merge and order data frames and then separate Julian from Gregorian calendar
out <- do.call(rbind, out)
out <- out[order(out$year,out$month), ]
out_julian <- out[which(out$year<=1771), ]
out_gregorian <- out[which(out$year>1771), ]

## Write SEF files
write_sef(out_julian[,c("year","month","day","hour","minute","mrp")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "p",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "hPa",
          stat = "mid_range",
          meta = out_julian$mrp_meta,
          metaHead = paste0("Observer=", metadata$Observer, " | Calendar=Julian | PGC=Y | PTC=N"),
          period = "month",
          note = "monthly-midrange",
          keep_na = FALSE)

write_sef(out_gregorian[,c("year","month","day","hour","minute","mrp")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "p",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "hPa",
          stat = "mid_range",
          meta = out_gregorian$mrp_meta,
          metaHead = paste0("Observer=", metadata$Observer, " | PGC=Y | PTC=N"),
          period = "month",
          note = "monthly-midrange",
          keep_na = FALSE)

write_sef(out_julian[,c("year","month","day","hour","minute","pres")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "p",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "hPa",
          stat = "mean",
          meta = out_julian$pres_meta,
          metaHead = paste0("Observer=", metadata$Observer, " | Calendar=Julian | PGC=Y | PTC=N"),
          period = "month",
          note = "monthly-mean",
          keep_na = FALSE)

write_sef(out_gregorian[,c("year","month","day","hour","minute","pres")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "p",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "hPa",
          stat = "mean",
          meta = out_gregorian$pres_meta,
          metaHead = paste0("Observer=", metadata$Observer, " | PGC=Y | PTC=N"),
          period = "month",
          note = "monthly-mean",
          keep_na = FALSE)

write_sef(out_julian[,c("year","month","day","hour","minute","tmin")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "ta",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "C",
          stat = "minimum",
          meta = out_julian$tmin_meta,
          metaHead = paste0("Observer=", metadata$Observer, " | Calendar=Julian"),
          period = "month",
          note = "monthly-min",
          keep_na = FALSE)

write_sef(out_gregorian[,c("year","month","day","hour","minute","tmin")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "ta",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "C",
          stat = "minimum",
          meta = out_gregorian$tmin_meta,
          metaHead = paste0("Observer=", metadata$Observer),
          period = "month",
          note = "monthly-min",
          keep_na = FALSE)

write_sef(out_julian[,c("year","month","day","hour","minute","tmax")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "ta",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "C",
          stat = "maximum",
          meta = out_julian$tmax_meta,
          metaHead = paste0("Observer=", metadata$Observer, " | Calendar=Julian"),
          period = "month",
          note = "monthly-max",
          keep_na = FALSE)

write_sef(out_gregorian[,c("year","month","day","hour","minute","tmax")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "ta",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "C",
          stat = "maximum",
          meta = out_gregorian$tmax_meta,
          metaHead = paste0("Observer=", metadata$Observer),
          period = "month",
          note = "monthly-max",
          keep_na = FALSE)

write_sef(out_julian[,c("year","month","day","hour","minute","temp")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "ta",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "C",
          stat = "mean",
          meta = out_julian$temp_meta,
          metaHead = paste0("Observer=", metadata$Observer, " | Calendar=Julian | ",
                            "Obs.times=matin,midi"),
          period = "month",
          note = "monthly-mean",
          keep_na = FALSE)

write_sef(out_gregorian[which(out_gregorian$year<1803), 
                        c("year","month","day","hour","minute","temp")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "ta",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "C",
          stat = "mean",
          meta = out_gregorian$temp_meta[which(out_gregorian$year<1803)],
          metaHead = paste0("Observer=", metadata$Observer, " | Obs.times=matin,midi"),
          period = "month",
          note = "monthly-mean",
          keep_na = FALSE)

write_sef(out_gregorian[which(out_gregorian$year%in%1803:1820), 
                        c("year","month","day","hour","minute","temp")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "ta",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "C",
          stat = "mean",
          meta = out_gregorian$temp_meta[which(out_gregorian$year%in%1803:1820)],
          metaHead = paste0("Observer=", metadata$Observer, " | Obs.times=matin,midi,soir"),
          period = "month",
          note = "monthly-mean",
          keep_na = FALSE)

write_sef(out_gregorian[which(out_gregorian$year>1820), 
                        c("year","month","day","hour","minute","temp")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "ta",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "C",
          stat = "mean",
          meta = out_gregorian$temp_meta[which(out_gregorian$year>1820)],
          metaHead = paste0("Observer=", metadata$Observer, " | Obs.times=7,2,9"),
          period = "month",
          note = "monthly-mean",
          keep_na = FALSE)

file.rename(list.files(paste0("../../4_Formatted/", name_folder), full.names=TRUE),
            gsub("NA","",list.files(paste0("../../4_Formatted/", name_folder), full.names=TRUE)))


## SUBDAILY

## Read filenames
files <- list.files("../../3_Revised/StPetersburg/", pattern="subdaily", full.names=TRUE)
out <- list()

## Read metadata
metadata <- inventory[which(inventory$Code == station_code & inventory$Resolution=="subdaily")[1], ]

## Loop over files
for (i in 1:length(files)) {
  
  ## Read data
  template <- readWorksheetFromFile(files[i], sheet=1, startRow=8, header=FALSE, colTypes="character")

  ## Read time
  if (i == 6) template[,4] <- substr(template[,4], 12, 19)
  time_meta <- paste0("orig.time=", template[,4])
  
  ## Temperature
  temp <- round(as.numeric(template[,5]) * 1.25, 1)
  temp_meta <- paste0(time_meta, "|orig=", template[,5], "R")
  
  ## Pressure
  template[,6] <- fill_variable(template[,6], NA)
  pres <- as.integer(template[,6]) + as.integer(template[,7])/100
  pres <- round(convert_pressure(pres, 27.07, as.numeric(metadata$Latitude), 
                                 as.numeric(metadata$Altitude)), 1)
  j <- which(as.integer(template[,7])<10)
  template[j, 7] <- paste0("0", template[j, 7])
  pres_meta <- paste0(time_meta, "|orig=", template[,6], ".", template[,7])
  
  ## Create output data frame
  out[[i]] <- data.frame(year=as.integer(template[,1]),
                         month=as.integer(template[,2]),
                         day=as.integer(template[,3]),
                         hour=rep(NA,nrow(template)),
                         minute=rep(NA,nrow(template)),
                         pres, pres_meta,
                         temp, temp_meta)

}

## Merge data frames and add time where available
out <- do.call(rbind, out)
out$hour[grep("7:00",out$temp_meta)] <- 7
out$hour[grep("14:00",out$temp_meta)] <- 14
out$hour[grep("21:00",out$temp_meta)] <- 21
out$minute[grep("7:00",out$temp_meta)] <- 0
out$minute[grep("14:00",out$temp_meta)] <- 0
out$minute[grep("21:00",out$temp_meta)] <- 0
out_gregorian <- out[which(out$year>1771), ]

## Write SEF files
write_sef(out[,c("year","month","day","hour","minute","temp")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "ta",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "C",
          stat = "point",
          meta = out$temp_meta,
          metaHead = paste0("Observer=", metadata$Observer),
          period = "0",
          time_offset = as.numeric(metadata$Longitude) * 12 / 180,
          keep_na = TRUE)

write_sef(out[,c("year","month","day","hour","minute","pres")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "p",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "hPa",
          stat = "point",
          meta = out$pres_meta,
          metaHead = paste0("Observer=", metadata$Observer, " | PGC=Y | PTC=N"),
          period = "0",
          time_offset = as.numeric(metadata$Longitude) * 12 / 180,
          keep_na = TRUE)


## Check SEF files
for (f in list.files(paste0("../../4_Formatted/", name_folder), full.names=TRUE)) {
  print(f)
  check_sef(f)
}