## NOTES:
## - Wind direction was not formatted since it is not clear to what time it refers
## - Pressure unit assumed to be Paris lines, zero assumed to be at 318 lines (=26.5 inches),
##   so that the average (uncorrected) pressure is about 1007 hPa

library(dataresqc)
library(XLConnect)

## Create station code
name_folder <- strsplit(getwd(), "/")[[1]][8]
name_group <- strsplit(getwd(), "/")[[1]][6]
station_code <- paste(name_group, name_folder, sep = "_")

## Create output directory (if necessary)
if (!name_folder %in% list.dirs("../../4_Formatted/",full.names=FALSE)) {
  dir.create(paste0("../../4_Formatted/", name_folder))
}

## Read inventory
inventory <- readWorksheetFromFile("/scratch3/PALAEO-RA/DataRescue/Inventory_PALAEO-RA_digi.xls",
                                   sheet=1, colTypes="character")

## Define function to fill missing values with the value on the previous row
fill_variable <- function(timeseries, strings) { 
  for (i in 2:length(timeseries)) {
    if (timeseries[i] %in% strings) {
      if(!timeseries[i-1] %in% strings) {
        timeseries[i] <- timeseries[i-1]
      } 
    }
  }
  return(timeseries)
}

## Read metadata
metadata <- inventory[which(inventory$Code == station_code), ]

## Read data
template <- readWorksheetFromFile("../../3_Revised/Warsaw/Europe_T3_PL_Warsaw_1725-1728_subdaily.xlsx", 
                                  sheet=1, startRow=8, header=FALSE, colTypes="character")
n <- nrow(template)

## Time
for (i in 1:3) template[,i] <- fill_variable(template[,i], NA)
hours <- minutes <- rep(NA, 2*n)
meta_time <- paste0("orig.time=", c(rep("mane",n),rep("vesperi",n)))

## Initialize data frame
out <- data.frame(year = rep(as.integer(template[,1]), 2),
                  month = rep(as.integer(template[,2]), 2),
                  day = rep(as.integer(template[,3]), 2),
                  hour = hours, 
                  minute = minutes)

## Pressure
template[which(is.na(template[,5])),5] <- 0
template[which(is.na(template[,7])),7] <- 0
pres <- c(as.numeric(template[,4]) + as.numeric(template[,5])/4,
          as.numeric(template[,6]) + as.numeric(template[,7])/4) + 318
pres <- convert_pressure(pres, 27.07/12, 
                         as.numeric(metadata$Latitude), as.numeric(metadata$Altitude))
out$p <- round(pres, 1)
out$p_meta <- paste0(meta_time, "|orig=", 
                     c(paste0(template[,4],".",template[,5]),
                       paste0(template[,6],".",template[,7])),
                     "in")

## Write SEF file
out <- out[which(!is.na(out$day)), ]
out <- out[order(out$year,out$month,out$day), ]
write_sef(out[, c("year","month","day","hour","minute","p")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "p",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "hPa",
          stat = "point",
          metaHead = paste0("Observer=", metadata$Observer, " | PTC=N | PGC=Y"),
          meta = out$p_meta,
          period = 0,
          keep_na = TRUE)


## Check SEF files
for (f in list.files(paste0("../../4_Formatted/", name_folder), full.names=TRUE)) {
  print(f)
  check_sef(f)
}