## NOTES:
## Temperature (Hawksby's thermometer) was not formatted
## Monthly pressure data for 1729-1730 are from a different observer (Wittenberg_2)
## From 1732 onward the Julian calendar ('old style') is used

library(dataresqc)
library(XLConnect)
library(lubridate)

## Create station code
name_folder <- strsplit(getwd(), "/")[[1]][8]
name_group <- strsplit(getwd(), "/")[[1]][6]
station_code <- paste(name_group, name_folder, sep = "_")

## Create output directory (if necessary)
if (!name_folder %in% list.dirs("../../4_Formatted/",full.names=FALSE)) {
  dir.create(paste0("../../4_Formatted/", name_folder))
}

## Read inventory
inventory <- readWorksheetFromFile("/scratch3/PALAEO-RA/DataRescue/Inventory_PALAEO-RA_digi.xls",
                                 sheet=1, colTypes="character")

## Define function to fill missing values with the value on the previous row
fill_variable <- function(timeseries, strings) {
  for (i in 2:length(timeseries)) {
    if (timeseries[i] %in% strings) {
      if(!timeseries[i-1] %in% strings) {
        timeseries[i] <- timeseries[i-1]
      }
    }
  }
  return(timeseries)
}


## MONTHLY
## Read data
template <- readWorksheetFromFile("../../3_Revised/Wittenberg/Europe_T3_DE_Wittenberg_1728-1734_monthly.xlsx",
                                  sheet=1, startRow=8, header=FALSE, colTypes="character")
template <- template[which(!is.na(template[,2])), ]
template[,1] <- fill_variable(template[,1], NA)
template2 <- readWorksheetFromFile("../../3_Revised/Wittenberg/Europe_T3_DE_Wittenberg_1728-1729_subdaily.xls", 
                                   sheet=1, startRow=8, header=FALSE, colTypes="character")


## Observer 1
metadata <- inventory[which(inventory$Code == paste0(station_code,"_1") & inventory$Resolution=="monthly"), ]

## Pressure 1728
pres <- (as.numeric(template[1:12,4]) + as.numeric(template[1:12,5])/10 +
          as.numeric(template[1:12,10]) + as.numeric(template[1:12,11])/10) / 2
pres <- convert_pressure(pres, 25.4, as.numeric(metadata$Latitude), as.numeric(metadata$Altitude))
out <- data.frame(Year = as.integer(template[1:12,1]),
                  Month = as.integer(template[1:12,2]),
                  Day = rep("", 12),
                  Hour = rep("", 12),
                  Minute = rep("", 12),
                  Value = round(pres, 1),
                  Meta = paste0("orig=", template[1:12,4], ".", template[1:12,5], "+", 
                                template[1:12,10], ".", template[1:11,11], "in"),
                  stringsAsFactors = FALSE)
write_sef(out[, 1:6],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "p",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "hPa",
          stat = "mid_range",
          meta = out$Meta,
          metaHead = paste0("Observer=", metadata$Observer, " | PTC=N | PGC=Y"),
          period = "month",
          note = "monthly",
          keep_na = TRUE)

## Pressure 1731-1734
n <- nrow(template)
pres <- as.numeric(template[37:n,6]) + as.numeric(template[37:n,7])/100
pres <- convert_pressure(pres, 25.4, as.numeric(metadata$Latitude), as.numeric(metadata$Altitude))
out <- data.frame(Year = as.integer(template[37:n,1]),
                  Month = as.integer(template[37:n,2]),
                  Day = rep("", n-37+1),
                  Hour = rep("", n-37+1),
                  Minute = rep("", n-37+1),
                  Value = round(pres, 1),
                  Meta = paste0("orig=", template[37:n,6], ".", template[37:n,7], "in"),
                  stringsAsFactors = FALSE)
j <- which(out$Year < 1732)
write_sef(out[j, 1:6],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "p",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "hPa",
          stat = "mean",
          meta = out$Meta[j],
          metaHead = paste0("Observer=", metadata$Observer, " | PTC=N | PGC=Y | Obs.times=morning"),
          period = "month",
          note = "monthly",
          keep_na = TRUE)
j <- which(out$Year >= 1732)
write_sef(out[j, 1:6],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "p",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "hPa",
          stat = "mean",
          meta = out$Meta[j],
          metaHead = paste0("Observer=", metadata$Observer, 
                            " | PTC=N | PGC=Y | Obs.times=morning | Calendar=Julian"),
          period = "month",
          note = "monthly",
          keep_na = TRUE)

## Precipitation 1728-1734 (read from two different templates in different units)
prec <- c((as.numeric(template2[1:3,10]) + as.numeric(template2[1:3,11])/24) * 27.07/72,
          as.numeric(template[4:n,16]) * 25.4)
out <- data.frame(Year = c(as.integer(template2[1:3,1]), as.integer(template[4:n,1])),
                  Month = c(as.integer(template2[1:3,2]), as.integer(template[4:n,2])),
                  Day = rep("", n),
                  Hour = rep("", n),
                  Minute = rep("", n),
                  Value = round(prec, 1),
                  Meta = paste0("orig=", c(paste0(template2[1:3,10],".",template2[1:3,11],"Cub."),
                                           paste0(template[4:n,16], "in"))),
                  stringsAsFactors = FALSE)
j <- which(out$Year < 1732)
write_sef(out[j, 1:6],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "rr",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "mm",
          stat = "sum",
          meta = out$Meta[j],
          metaHead = paste0("Observer=", metadata$Observer),
          period = "month",
          note = "monthly",
          keep_na = TRUE)
j <- which(out$Year >= 1732)
write_sef(out[j, 1:6],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "rr",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "mm",
          stat = "sum",
          meta = out$Meta[j],
          metaHead = paste0("Observer=", metadata$Observer, " | Calendar=Julian"),
          period = "month",
          note = "monthly",
          keep_na = TRUE)


## Observer 2
metadata <- inventory[which(inventory$Code == paste0(station_code,"_2")), ]

## Pressure 1729-1730
n <- nrow(template)
pres <- as.numeric(template[13:36,6]) + as.numeric(template[13:36,7])/100
pres <- convert_pressure(pres, 25.4, as.numeric(metadata$Latitude), as.numeric(metadata$Altitude))
out <- data.frame(Year = as.integer(template[13:36,1]),
                  Month = as.integer(template[13:36,2]),
                  Day = rep("", 24),
                  Hour = rep("", 24),
                  Minute = rep("", 24),
                  Value = round(pres, 1),
                  Meta = paste0("orig=", template[13:36,6], ".", template[13:36,7], "in"),
                  stringsAsFactors = FALSE)
write_sef(out[, 1:6],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "p",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "hPa",
          stat = "mean",
          meta = out$Meta,
          metaHead = paste0("Observer=", metadata$Observer, " | PTC=N | PGC=Y | Obs.times=morning"),
          period = "month",
          note = "monthly",
          keep_na = TRUE)


file.rename(list.files(paste0("../../4_Formatted/", name_folder), full.names=TRUE), 
            gsub(" ", "", list.files(paste0("../../4_Formatted/", name_folder), full.names=TRUE)))


## SUBDAILY
template <- template2[-(1:3),] # ignore monthly data
metadata <- inventory[which(inventory$Code == paste0(station_code,"_1") & inventory$Resolution=="subdaily"), ]

## Time (assuming morning observations were always at 6AM)
for (i in 1:3) template[,i] <- fill_variable(template[,i], NA)
times <- strptime(template[,4], format="%Y-%m-%d %H:%M")
hours <- hour(times)
hours[which(is.na(hours))] <- 6
minutes <- minute(times)
minutes[which(is.na(minutes))] <- 0
times <- paste(template[,1], template[,2], template[,3], hours, minutes)
hours[duplicated(times)] <- NA
minutes[duplicated(times)] <- NA
j <- which(nchar(template[,4]) < 10)
meta_time <- paste0("orig.time=", substr(template[,4],12,16))
meta_time[j] <- paste0("orig.time=", template[j,4])

## Pressure
template[,5] <- fill_variable(template[,5], NA)
pres <- as.numeric(template[,5]) + as.numeric(template[,6])/12
pres <- convert_pressure(pres, 27.07, as.numeric(metadata$Latitude), as.numeric(metadata$Altitude))
  
## Wind direction
directions <- c("N", "NNO", "NO", "ONO", "O", "OSO", "SO", "SSO", "S",
                "SSW", "SW", "WSW", "W", "WNW", "NW", "NNW")
wdir <- 22.5 * (match(toupper(template[,8]), directions) - 1)

## Precipitation
template[which(is.na(template[,10])), 10] <- 0
template[which(is.na(template[,11])), 11] <- 0
prec <- (as.numeric(template[,10]) + as.numeric(template[,11])/24) * 27.07/72
  
## Create output data frame
out <- data.frame(Year = as.integer(template[,1]),
                  Month = as.integer(template[,2]),
                  Day = as.integer(template[,3]),
                  Hour = hours,
                  Minute = minutes,
                  p = round(pres, 1),
                  p_meta = paste0(meta_time, "|orig=", template[,5], ".", template[,6], "Pin"),
                  dd = round(wdir, 0),
                  dd_meta = paste0(meta_time, "|orig=", template[,8]),
                  w = as.integer(template[,9]),
                  w_meta = paste0(meta_time, "|orig=", template[,9]),
                  rr = round(prec, 1),
                  rr_meta = paste0(meta_time, "|orig=", template[,10], ".", template[,11], "Cub."),
                  stringsAsFactors = FALSE)

## Write SEF files
write_sef(out[, 1:6],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "p",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "hPa",
          stat = "point",
          meta = out$p_meta,
          metaHead = paste0("Observer=", metadata$Observer, " | PTC=N | PGC=Y"),
          period = 0,
          time_offset = as.numeric(metadata$Longitude)*12/180,
          keep_na = TRUE)

write_sef(out[, c(1:5,8)],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "dd",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "degree",
          stat = "point",
          meta = out$dd_meta,
          metaHead = paste0("Observer=", metadata$Observer),
          period = 0,
          time_offset = as.numeric(metadata$Longitude)*12/180,
          keep_na = TRUE)

write_sef(out[, c(1:5,10)],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "wind_force",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "0-4",
          stat = "point",
          meta = out$w_meta,
          metaHead = paste0("Observer=", metadata$Observer),
          period = 0,
          time_offset = as.numeric(metadata$Longitude)*12/180,
          keep_na = TRUE)

write_sef(out[, c(1:5,12)],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "rr",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "mm",
          stat = "sum",
          meta = out$rr_meta,
          metaHead = paste0("Observer=", metadata$Observer),
          period = "p",
          time_offset = as.numeric(metadata$Longitude)*12/180,
          keep_na = TRUE)


## Check SEF files
for (f in list.files(paste0("../../4_Formatted/", name_folder), full.names=TRUE)) check_sef(f)
