## NOTE: Dominant wind not formatted

library(dataresqc)
library(XLConnect)

## Create station code
name_folder <- strsplit(getwd(), "/")[[1]][8]
name_group <- strsplit(getwd(), "/")[[1]][6]
station_code <- paste(name_group, name_folder, sep = "_")

## Create output directory (if necessary)
if (!name_folder %in% list.dirs("../../4_Formatted/",full.names=FALSE)) {
  dir.create(paste0("../../4_Formatted/", name_folder))
}

## Read inventory
inventory <- readWorksheetFromFile("/scratch3/PALAEO-RA/DataRescue/Inventory_PALAEO-RA_digi.xls",
                                   sheet=1, colTypes="character")
inv_srm <- read.csv("SRM_stations.csv", stringsAsFactors=FALSE, header=FALSE)

## Read filenames
files <- list.files("../../3_Revised/SRM", full.names=TRUE)
years <- 1776:1786

## Loop on files
Data <- list()
for (i in 1:length(files)) {
  
  ## Initialize data frame
  Data[[years[i]]] <- data.frame(year=as.character(), month=as.character(),
                                 station=as.character(),
                                 tmax_day=as.character(), tmin_day=as.character(), 
                                 tmax=as.character(), tmin=as.character(), 
                                 tmean=as.character(), pmax_day=as.character(), 
                                 pmin_day=as.character(), pmax=as.character(),
                                 pmax_orig=as.character(), pmin=as.character(), 
                                 pmin_orig=as.character(), pmean=as.character(),
                                 pmean_orig=as.character(), nr=as.character(), 
                                 rr=as.character(), rr_orig=as.character())
  
  ## Loop on sheets
  for (j in 1:12) {
    
    ## Read data
    template <- readWorksheetFromFile(files[i], sheet=j, startRow=11, header=FALSE, colTypes="character")
    k1 <- ifelse(years[i]%in%1777:1778, 21, 18)
    k2 <- ifelse(years[i]%in%1777:1778, 18, 19)
    tmp <- cbind(years[i], j, template[,c(1:8)],
                 as.numeric(template[,9])+as.numeric(template[,10])/12+as.numeric(template[,11])/144,
                 paste(template[,9],template[,10],template[,11],sep="."),
                 as.numeric(template[,12])+as.numeric(template[,13])/12+as.numeric(template[,14])/144,
                 paste(template[,12],template[,13],template[,14],sep="."),
                 as.numeric(template[,15])+as.numeric(template[,16])/12+as.numeric(template[,17])/144,
                 paste(template[,15],template[,16],template[,17],sep="."),
                 template[,k1],
                 as.numeric(template[,k2])+as.numeric(template[,k2+1])/12+as.numeric(template[,k2+2])/144,
                 paste(template[,k2],template[,k2+1],template[,k2+2],sep="."),
                 stringsAsFactors=FALSE)
    Data[[years[i]]] <- rbind(Data[[years[i]]], tmp, stringsAsFactors=FALSE)
    
  }
  
}

## Merge data frames
Data <- Reduce(rbind, Data)
names(Data) <- c("year", "month", "station", "tmax_day", "tmin_day", "tmax", "tmin", "tmean",
                 "pmax_day", "pmin_day", "pmax", "pmax_orig", "pmin", "pmin_orig", "pmean",
                 "pmean_orig", "nr", "rr", "rr_orig")
Data <- Data[which(!is.na(Data$station)), ]

## Assign unique name to each station
Data$station[grep("Vienne.+D", Data$station)] <- "Vienne Dauphiné" # avoid confusion between Vienne, France and Vienne, Austria
Data$station <- sapply(Data$station, function(x) strsplit(x,",")[[1]][1])
for (i in 1:nrow(inv_srm)) {
  Data$station[grep(inv_srm[i,1],Data$station,fixed=TRUE)] <- inv_srm[i,2] 
}

## Loop on stations
for(st in unique(Data$station)) {
  print(paste(which(unique(Data$station)==st),st), quote=FALSE)
  i <- which(Data$station == st)
  
  ## Read metadata
  metadata <- inventory[which(inventory$Station==st & inventory$Source=="Histoire de la SRM"), ]
  
  ## Initialize output data frame
  out <- data.frame(year=Data$year[i],
                    month=Data$month[i],
                    day=rep("", length(i)),
                    hour=rep("", length(i)),
                    minute=rep("", length(i)))
  
  ## Check for duplicates
  for (iy in out$year) {
    if (sum(out$year==iy) > 12) stop("Duplicate found")
  }
  
  ## Write data into data frame
  out$ta <- round(as.numeric(Data$tmean[i])*1.25, 1)
  out$ta_meta <- paste0("orig=", Data$tmean[i], "R")
  out$p <- round(convert_pressure(Data$pmean[i],27.07,as.numeric(metadata$Latitude),
                                  as.numeric(metadata$Altitude)), 1)
  out$p_meta <- paste0("orig=", Data$pmean_orig[i], "Pin")
  out$nr <- as.integer(as.character(Data$nr[i]))
  out$nr_meta <- paste0("orig=", Data$nr[i])
  out$rr <- round(as.numeric(Data$rr[i])*27.07, 1)
  out$rr_meta <- paste0("orig=", Data$rr_orig[i], "Pin")
  out$tx <- round(as.numeric(Data$tmax[i])*1.25, 1)
  out$tx_meta <- paste0("orig=", Data$tmax[i], "R|day=", Data$tmax_day[i])
  out$tn <- round(as.numeric(Data$tmin[i])*1.25, 1)
  out$tn_meta <- paste0("orig=", Data$tmin[i], "R|day=", Data$tmin_day[i])
  out$px <- round(convert_pressure(Data$pmax[i],27.07,as.numeric(metadata$Latitude),
                                   as.numeric(metadata$Altitude)), 1)
  out$px_meta <- paste0("orig=", Data$pmax_orig[i], "Pin|day=", Data$pmax_day[i])
  out$pn <- round(convert_pressure(Data$pmin[i],27.07,as.numeric(metadata$Latitude),
                                   as.numeric(metadata$Altitude)), 1)
  out$pn_meta <- paste0("orig=", Data$pmin_orig[i], "Pin|day=", Data$pmin_day[i])
  
  ## Write SEF files
  if (sum(!is.na(out$p)) > 0) {
    write_sef(out[, c("year","month","day","hour","minute","p")],
              outpath = paste0("../../4_Formatted/", name_folder),
              variable = "p",
              cod = metadata$Code,
              nam = metadata$Station,
              lat = metadata$Latitude,
              lon = metadata$Longitude,
              alt = metadata$Altitude,
              sou = "PALAEO-RA",
              link = metadata$C3S.Link,
              units = "hPa",
              stat = "mean",
              meta = out$p_meta,
              metaHead = "PTC=N | PGC=Y",
              period = "month",
              note = "monthly",
              keep_na = TRUE)
  }
  
  if (sum(!is.na(out$px)) > 0) {
    write_sef(out[, c("year","month","day","hour","minute","px")],
              outpath = paste0("../../4_Formatted/", name_folder),
              variable = "p",
              cod = metadata$Code,
              nam = metadata$Station,
              lat = metadata$Latitude,
              lon = metadata$Longitude,
              alt = metadata$Altitude,
              sou = "PALAEO-RA",
              link = metadata$C3S.Link,
              units = "hPa",
              stat = "maximum",
              meta = out$px_meta,
              metaHead = "PTC=N | PGC=Y",
              period = "month",
              note = "monthly-max",
              keep_na = TRUE)
  }  
  
  if (sum(!is.na(out$pn)) > 0) {
    write_sef(out[, c("year","month","day","hour","minute","pn")],
              outpath = paste0("../../4_Formatted/", name_folder),
              variable = "p",
              cod = metadata$Code,
              nam = metadata$Station,
              lat = metadata$Latitude,
              lon = metadata$Longitude,
              alt = metadata$Altitude,
              sou = "PALAEO-RA",
              link = metadata$C3S.Link,
              units = "hPa",
              stat = "minimum",
              meta = out$pn_meta,
              metaHead = "PTC=N | PGC=Y",
              period = "month",
              note = "monthly-min",
              keep_na = TRUE)
  }  
  
  if (sum(!is.na(out$ta)) > 0) {
    write_sef(out[, c("year","month","day","hour","minute","ta")],
              outpath = paste0("../../4_Formatted/", name_folder),
              variable = "ta",
              cod = metadata$Code,
              nam = metadata$Station,
              lat = metadata$Latitude,
              lon = metadata$Longitude,
              alt = metadata$Altitude,
              sou = "PALAEO-RA",
              link = metadata$C3S.Link,
              units = "C",
              stat = "mean",
              meta = out$ta_meta,
              period = "month",
              note = "monthly",
              keep_na = TRUE)
  }
  
  if (sum(!is.na(out$tx)) > 0) {
    write_sef(out[, c("year","month","day","hour","minute","tx")],
              outpath = paste0("../../4_Formatted/", name_folder),
              variable = "ta",
              cod = metadata$Code,
              nam = metadata$Station,
              lat = metadata$Latitude,
              lon = metadata$Longitude,
              alt = metadata$Altitude,
              sou = "PALAEO-RA",
              link = metadata$C3S.Link,
              units = "C",
              stat = "maximum",
              meta = out$tx_meta,
              period = "month",
              note = "monthly-max",
              keep_na = TRUE)
  }
  
  if (sum(!is.na(out$tn)) > 0) {
    write_sef(out[, c("year","month","day","hour","minute","tn")],
              outpath = paste0("../../4_Formatted/", name_folder),
              variable = "ta",
              cod = metadata$Code,
              nam = metadata$Station,
              lat = metadata$Latitude,
              lon = metadata$Longitude,
              alt = metadata$Altitude,
              sou = "PALAEO-RA",
              link = metadata$C3S.Link,
              units = "C",
              stat = "minimum",
              meta = out$tn_meta,
              period = "month",
              note = "monthly-min",
              keep_na = TRUE)
  }
  
  if (sum(!is.na(out$rr)) > 0) {
    write_sef(out[, c("year","month","day","hour","minute","rr")],
              outpath = paste0("../../4_Formatted/", name_folder),
              variable = "rr",
              cod = metadata$Code,
              nam = metadata$Station,
              lat = metadata$Latitude,
              lon = metadata$Longitude,
              alt = metadata$Altitude,
              sou = "PALAEO-RA",
              link = metadata$C3S.Link,
              units = "mm",
              stat = "sum",
              meta = out$rr_meta,
              period = "month",
              note = "monthly",
              keep_na = TRUE)
  }
  
  if (sum(!is.na(out$nr)) > 0) {
    write_sef(out[, c("year","month","day","hour","minute","nr")],
              outpath = paste0("../../4_Formatted/", name_folder),
              variable = "nr",
              cod = metadata$Code,
              nam = metadata$Station,
              lat = metadata$Latitude,
              lon = metadata$Longitude,
              alt = metadata$Altitude,
              sou = "PALAEO-RA",
              link = metadata$C3S.Link,
              units = "days",
              stat = "sum",
              meta = out$nr_meta,
              period = "month",
              keep_na = TRUE)
  }
  
}


## Check SEF files
for (f in list.files(paste0("../../4_Formatted/", name_folder), full.names=TRUE)) {
  print(f)
  check_sef(f)
}
