## NOTES:
## - Temperature was not formatted (Florentine thermometer)
## - Pressure units assumed to be Breslau inch (=23.996 mm), however values are too low
## - Wind direction was not formatted

library(dataresqc)
library(XLConnect)

## Create station code
name_folder <- strsplit(getwd(), "/")[[1]][8]
name_group <- strsplit(getwd(), "/")[[1]][6]
station_code <- paste(name_group, name_folder, sep = "_")

## Create output directory (if necessary)
if (!name_folder %in% list.dirs("../../4_Formatted/",full.names=FALSE)) {
  dir.create(paste0("../../4_Formatted/", name_folder))
}

## Read inventory
inventory <- readWorksheetFromFile("/scratch3/PALAEO-RA/DataRescue/Inventory_PALAEO-RA_digi.xls",
                                   sheet=1, colTypes="character")

## Define function to fill missing values with the value on the previous row
fill_variable <- function(timeseries, strings) {
  for (i in 2:length(timeseries)) {
    if (timeseries[i] %in% strings) {
      if(!timeseries[i-1] %in% strings) {
        timeseries[i] <- timeseries[i-1]
      }
    }
  }
  return(timeseries)
}

## Loop on files
files <- list.files("../../3_Revised/Loebau/", full.names=TRUE)
out <- list()
for (i in 1:length(files)) {
  
  ## Read data
  template <- readWorksheetFromFile(files[i], sheet=1, startRow=9, header=FALSE, 
                                    colTypes="character")
  metadata <- inventory[which(inventory$Code == station_code), ]
  
  ## Initialize data frame
  out[[i]] <- data.frame(Year = as.integer(template[,1]),
                         Month = as.integer(template[,2]),
                         Day = as.integer(template[,3]),
                         Hour = rep(NA, nrow(template)),
                         Minute = rep(NA, nrow(template)))
  
  ## Pressure
  template[,4] <- fill_variable(template[,4], c("-",".",NA))
  pres <- as.numeric(template[,4]) + as.numeric(template[,5])/12
  pres <- convert_pressure(pres, 23.996, as.numeric(metadata$Latitude), as.numeric(metadata$Altitude))
  out[[i]]$p <- round(pres, 1)
  out[[i]]$p_meta <- paste0("orig=", template[,4], ".", template[,5], "Br.in")
  
}

## Merge data frames
out <- Reduce(rbind, out)
out <- out[order(out$Year,out$Month,out$Day), ]

## Write SEF file
write_sef(out[, 1:6],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "p",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "hPa",
          stat = "point",
          meta = out$p_meta,
          metaHead = paste0("Observer=", metadata$Observer, " | PTC=N | PGC=Y"),
          period = "0",
          keep_na = TRUE)


## Check SEF files
for (f in list.files(paste0("../../4_Formatted/", name_folder), full.names=TRUE)) check_sef(f)