## NOTES:
## - Temperature was not formatted (unknown scale)
## - Pressure was formatted only for Barometer maj.

library(dataresqc)
library(XLConnect)
library(lubridate)

## Create station code
name_folder <- strsplit(getwd(), "/")[[1]][8]
name_group <- strsplit(getwd(), "/")[[1]][6]
station_code <- paste(name_group, name_folder, sep = "_")

## Create output directory (if necessary)
if (!name_folder %in% list.dirs("../../4_Formatted/",full.names=FALSE)) {
  dir.create(paste0("../../4_Formatted/", name_folder))
}

## Read inventory
inventory <- readWorksheetFromFile("/scratch3/PALAEO-RA/DataRescue/Inventory_PALAEO-RA_digi.xls",
                                   sheet=1, colTypes="character")

## Define function to fill missing values with the value on the previous row
fill_variable <- function(timeseries, strings) {
  for (i in 2:length(timeseries)) {
    if (timeseries[i] %in% strings) {
      if(!timeseries[i-1] %in% strings) {
        timeseries[i] <- timeseries[i-1]
      }
    }
  }
  return(timeseries)
}

## Loop on files
files <- list.files("../../3_Revised/Nuernberg/", full.names=TRUE)
out <- list()
for (i in 1:length(files)) {
  
  ## Read data
  template <- readWorksheetFromFile(files[i], sheet=1, startRow=9, header=FALSE, 
                                    colTypes="character")
  for (j in 1:3) template[,j] <- fill_variable(template[,j], NA)
  metadata <- inventory[which(inventory$Code == station_code), ]
  
  ## Time
  hours <- as.numeric(template[,4])
  minutes <- as.integer((as.numeric(template[,4])-as.integer(template[,4]))*60)
  dates <- ISOdate(template[,1], template[,2], template[,3])
  nobs <- table(dates)
  hours[grep("n",template[,5])] <- hours[grep("n",template[,5])] + 12
  hours[which(dates%in%names(nobs[nobs==2]))] <- hours[which(dates%in%names(nobs[nobs==2]))] + 12
  hours[which(hours>=24)] <- hours[which(hours>=24)] - 12
  for (j in 1:10) {
    dh <- c(1, diff(hours))
    dd <- c(0, diff(dates))
    hours[which(dh<=0 & dd==0)] <- hours[which(dh<=0 & dd==0)] + 12
  }
  i_pm <- which(hours<12 & dd==0 & c(dd[2:length(dd)],1)>0)
  hours[i_pm] <- hours[i_pm] + 12
  hours[which(hours>24)] <- hours[which(hours>24)] - 12
  hours <- as.integer(hours)
  meta_time <- paste0("orig.date=", substr(dates,1,10), "|orig.time=", template[,4])
  meta_time[grep("n",template[,5])] <- paste0(meta_time[grep("n",template[,5])], "n")
  dates[which(hours==24)] <- dates[which(hours==24)] + 24*3600
  
  ## Initialize data frame
  out[[i]] <- data.frame(Year = year(dates),
                         Month = month(dates),
                         Day = day(dates),
                         Hour = hours,
                         Minute = minutes)
  
  ## Pressure
  j <- c(13, 14, 12, 11, 10, 7, 7, 7)
  template[,j[i]] <- fill_variable(template[,j[i]], c('"','-',NA))
  template[,j[i]+1] <- fill_variable(template[,j[i]+1], c('"','-'))
  pres <- as.numeric(template[,j[i]]) + as.numeric(template[,j[i]+1])/12
  if (i < 5) {
    template[,j[i]+2] <- fill_variable(template[,j[i]+2], c('"','-'))
    template[,j[i]+3] <- fill_variable(template[,j[i]+3], c('"','-'))
    k <- which(!is.na(template[,j[i]+2]))
    pres[k] <- pres[k] + (as.numeric(template[k,j[i]+2])/as.numeric(template[k,j[i]+3])) / 12
  }
  pres <- convert_pressure(pres, 26.154, as.numeric(metadata$Latitude), as.numeric(metadata$Altitude))
  out[[i]]$p <- round(pres, 1)
  out[[i]]$p_meta <- paste0(meta_time, "|orig=", template[,j[i]], ".", template[,j[i]+1], "Rh.in")
  if (i < 5) {
    out[[i]]$p_meta[k] <- paste0(meta_time[k], "|orig=", template[k,j[i]], ".", template[k,j[i]+1], ".",
                                 template[k,j[i]+2], "/", template[k,j[i]+3], "Rh.in")
  }
  
  ## Wind
  j <- c(18, 18, 20, 20, 13, 10, 12, 12)
  directions <- c("N", "NgO", "NNO", "NOgN", "NO", "NOgO", "ONO", "OgN", "O", "OgS", "OSO", 
                  "SOgO", "SO", "SOgS", "SSO", "SgO", "S", "SgW", "SSW", "SWgS", "SW", "SWgW", 
                  "WSW", "WgS", "W", "WgN", "WNW", "NWgW", "NW", "NWgN", "NNW", "NgW")
  out[[i]]$dd <- round(11.25 * (match(template[,j[i]], directions) - 1), 0)
  out[[i]]$dd_meta <- paste0(meta_time, "|orig=", template[,j[i]])
  
}

## Merge data frames
out <- Reduce(rbind, out)
out <- out[order(out$Year,out$Month,out$Day), ]

## Write SEF file
write_sef(out[, 1:6],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "p",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "hPa",
          stat = "point",
          meta = out$p_meta,
          metaHead = paste0("Observer=", metadata$Observer, " | PTC=N | PGC=Y"),
          period = "0",
          time_offset = as.numeric(metadata$Longitude)*12/180,
          keep_na = TRUE)

write_sef(out[, c("Year","Month","Day","Hour","Minute","dd")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "dd",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "degree",
          stat = "point",
          meta = out$dd_meta,
          metaHead = paste0("Observer=", metadata$Observer),
          period = "0",
          time_offset = as.numeric(metadata$Longitude)*12/180,
          keep_na = TRUE)


## Check SEF files
for (f in list.files(paste0("../../4_Formatted/", name_folder), full.names=TRUE)) check_sef(f)
