## NOTES:
## - There are 2 distinct records from different observers, so we use 2 station codes (USA_Andover_1 and USA_Andover_2)

library(dataresqc)
library(XLConnect)

## Create station code
name_folder <- strsplit(getwd(), "/")[[1]][8]
name_group <- strsplit(getwd(), "/")[[1]][6]
station_code <- paste(name_group, name_folder, sep = "_")

## Create output directory (if necessary)
if (!name_folder %in% list.dirs("../../4_Formatted/",full.names=FALSE)) {
  dir.create(paste0("../../4_Formatted/", name_folder))
}

## Read inventory
inventory <- readWorksheetFromFile("/scratch3/PALAEO-RA/DataRescue/Inventory_PALAEO-RA_digi.xls",
                                 sheet=1, colTypes="character")


## SUBDAILY
template <- readWorksheetFromFile("../../3_Revised/Andover/USA_T3_US_Andover_1775_subdaily.xls",
                                  sheet=1, startRow=8, header=FALSE, colTypes="character")
metadata <- inventory[which(inventory$Code == paste(station_code, 1, sep="_") & 
                              inventory$Resolution == "subdaily"), ]

## Time
time <- as.numeric(template[,4])
time[which(template[,5]=="e")] <- time[which(template[,5]=="e")] + 12
hours <- as.integer(time)
minutes <- as.integer((time - hours) * 60)
dates <- as.Date(paste(template[,1], template[,2], template[,3], sep="-"))
meta_time <- paste0("orig.date=", dates, "|orig.time=", template[,4], template[,5])

## Temperature (Fahrenheit)
temp <- round((as.numeric(template[,8]) - 32) * 5 / 9, 1)
out <- data.frame(as.integer(template[,1]), 
                  as.integer(template[,2]), 
                  as.integer(template[,3]), 
                  hours, minutes, temp)
meta_col <- paste0(meta_time, "|orig=", template[,8], "F")
write_sef(out,
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "ta",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "C",
          stat = "point",
          metaHead = ifelse(is.na(metadata$Observer), "", paste0("Observer=", metadata$Observer)),
          meta = meta_col,
          period = 0,
          time_offset = as.numeric(metadata$Longitude) * 12 / 180,
          keep_na = TRUE)
          
## Wind direction
directions <- c("N", "NNE", "NE", "ENE", "E", "ESE", "SE", "SSE", "S",
                "SSW", "SW", "WSW", "W", "WNW", "NW", "NNW")
wdir <- 22.5 * (match(toupper(template[,9]), directions) - 1)
out <- data.frame(as.integer(template[,1]), 
                  as.integer(template[,2]), 
                  as.integer(template[,3]), 
                  hours, minutes, wdir)
meta_col <- paste0(meta_time, "|orig=", template[,9])
write_sef(out,
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "dd",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "degree",
          stat = "point",
          metaHead = ifelse(is.na(metadata$Observer), "", paste0("Observer=", metadata$Observer)),
          meta = meta_col,
          period = 0,
          time_offset = as.numeric(metadata$Longitude) * 12 / 180,
          keep_na = TRUE)

## Wind force
wforce <- as.integer(template[,10])
out <- data.frame(as.integer(template[,1]), 
                  as.integer(template[,2]), 
                  as.integer(template[,3]), 
                  hours, minutes, wforce)
meta_col <- paste0(meta_time, "|orig=", template[,10])
write_sef(out,
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "w",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "0-2",
          stat = "point",
          metaHead = ifelse(is.na(metadata$Observer), "", paste0("Observer=", metadata$Observer)),
          meta = meta_col,
          period = 0,
          time_offset = as.numeric(metadata$Longitude) * 12 / 180,
          keep_na = TRUE)


## MONTHLY
metadata <- inventory[which(inventory$Code == paste(station_code, 2, sep="_") & 
                              inventory$Resolution == "monthly"), ]

## Temperature
out <- data.frame(Year = rep(1798:1808, each=12),
                  Month = rep(1:12, 11),
                  Day = rep(NA, 11*12),
                  Hour = rep("", 11*12),
                  Minute = rep("", 11*12),
                  Tmax = rep(NA, 11*12),
                  Tavg = rep(NA, 11*12),
                  Tmin = rep(NA, 11*12),
                  stringsAsFactors = FALSE)
## Loop on sheets
for (sh in 1:11) {
  template <- readWorksheetFromFile("../../3_Revised/Andover/USA_T3_US_Andover_1798-1808_monthly.xls",
                                    sheet=sh, startRow=7, header=FALSE, colTypes="character")
  ## Convert to Celsius
  temp <- template[,3:5]
  for (i in 1:3) temp[,i] <- round((as.numeric(temp[,i]) - 32) * 5 / 9, 1)
  ## Calculate monthly max,average,min
  out$Tmax[(12*sh-11):(12*sh)] <- apply(temp[seq(1,34,3),], 1, max)
  out$Tavg[(12*sh-11):(12*sh)] <- round(apply(temp[seq(2,35,3),], 1, mean), 1)
  out$Tmin[(12*sh-11):(12*sh)] <- apply(temp[seq(3,36,3),], 1, min)
}
## Write SEF files (loop on stat)
stats <- c("maximum","mean","minimum")
for (st in stats) {
  write_sef(out[,c(1:5,5+which(stats==st))],
            outpath = paste0("../../4_Formatted/", name_folder),
            variable = "ta",
            cod = metadata$Code,
            nam = metadata$Station,
            lat = metadata$Latitude,
            lon = metadata$Longitude,
            alt = metadata$Altitude,
            sou = "PALAEO-RA",
            link = metadata$C3S.Link,
            units = "C",
            stat = st,
            metaHead = paste0("Observation times=sunrise,warmest pt of day,sundown",
                              ifelse(is.na(metadata$Observer), 
                                     "", paste0("|Observer=", metadata$Observer))),
            period = "month",
            note = ifelse(st=="mean", "monthly", paste0("monthly-",st)),
            keep_na = TRUE)
}

## Precipitation
template <- readWorksheetFromFile("../../3_Revised/Andover/USA_T3_US_Andover_1782-1803_monthly-prec.xls",
                                  sheet=1, startRow=8, header=FALSE, colTypes="character")
out <- data.frame(Year = rep(as.integer(template[,1]), each=12),
                  Month = rep(1:12, nrow(template)),
                  Day = rep(NA, nrow(template)*12),
                  Hour = rep("", nrow(template)*12),
                  Minute = rep("", nrow(template)*12),
                  Value = round(25.4 * as.numeric(c(t(template[,2:13]))), 1),
                  Meta = paste0("orig=", c(t(template[,2:13])), "in"))
write_sef(out[,1:6],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "rr",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "mm",
          stat = "sum",
          metaHead = ifelse(is.na(metadata$Observer), "", paste0("Observer=", metadata$Observer)),
          meta = out[,7],
          period = "month",
          note = "monthly",
          keep_na = TRUE)


file.rename(list.files(paste0("../../4_Formatted/", name_folder), full.names=TRUE), 
            gsub("NA", "", list.files(paste0("../../4_Formatted/", name_folder), full.names=TRUE)))


## Check SEF files
for (f in list.files(paste0("../../4_Formatted/", name_folder), full.names=TRUE)) check_sef(f)
