## NOTES:
## Two thermometers are used: one Fahrenheit mercury thermometer and one Royal Society thermometer.
## We convert the latter to Fahrenheit by linear fitting of the parallel observations 
## (done separately for each file).
## Values above 100 degrees (given as 100+) are converted to NA.

library(dataresqc)
library(XLConnect)
#library(pracma)

## Create station code
name_folder <- strsplit(getwd(), "/")[[1]][8]
name_group <- strsplit(getwd(), "/")[[1]][6]
station_code <- paste(name_group, name_folder, sep = "_")

## Create output directory (if necessary)
if (!name_folder %in% list.dirs("../../4_Formatted/",full.names=FALSE)) {
  dir.create(paste0("../../4_Formatted/", name_folder))
}

## Read inventory
inventory <- readWorksheetFromFile("/scratch3/PALAEO-RA/DataRescue/Inventory_PALAEO-RA_digi.xls",
                                   sheet=1, colTypes="character")

## Define function to fill missing values with the value on the previous row
fill_variable <- function(timeseries, strings) { 
  for (i in 2:length(timeseries)) {
    if (timeseries[i] %in% strings) {
      if(!timeseries[i-1] %in% strings) {
        timeseries[i] <- timeseries[i-1]
      } 
    }
  }
  return(timeseries)
}

## Define function that does the fitting of the parallel observations
convert_rs <- function(rs, f) {
  i <- which(!is.na(rs) & !is.na(f))
  #  model <- polyfit(rs[i], f[i], 2)  
  #  out <- model[3] + model[2] * rs + model[1] * rs**2
  model <- lm(f[i] ~ rs[i])$coef
  out <- model[1] + model[2] * rs
  #  print(model)
  return(out)
}

## Define wind directions
directions <- c("N", "NbE", "NNE", "NEbN", "NE", "NEbE", "ENE", "EbN", "E", 
                "EbS", "ESE", "SEbE", "SE", "SEbS", "SSE", "SbE", "S",
                "SbW", "SSW", "SWbS", "SW", "SWbW", "WSW", "WbS", "W",
                "WbN", "WNW", "NWbW", "NW", "NWbN", "NNW", "NbW")


## SUBDAILY

## 1742-1779

files <- c("USA_T3_US_Cambridge_1742-1748_subdaily.xls",
           "USA_T3_US_Cambridge_1749-1753_subdaily.xls",
           "USA_T3_US_Cambridge_1754-1758_subdaily.xls",
           "USA_T3_US_Cambridge_1759-1763_subdaily.xls",
           "USA_T3_US_Cambridge_1764-1768_subdaily.xls",
           "USA_T3_US_Cambridge_1769-1773_subdaily.xls",
           "USA_T3_US_Cambridge_1774-1779_subdaily.xls")

## Read metadata
metadata <- inventory[which(inventory$Code == station_code & 
                              inventory$Resolution == "subdaily" &
                              inventory$First.Year == "1742"), ]

## Loop on files
out <- list()
for (i in 1:length(files)) {
  
  ## Read data
  template <- readWorksheetFromFile(paste0("../../3_Revised/Cambridge/",files[i]),
                                    sheet=1, startRow=8, header=FALSE, colTypes="character")
  
  ## Time
  time <- as.numeric(template[,4])
  time[which(template[,4]=="noon" | template[,5]=="n")] <- 12
  time[which(template[,4]=="midnight" | template[,5]=="midnight")] <- 24
  time[which(template[,5]=="e")] <- time[which(template[,5]=="e")] + 12
  i_nextday <- which(time >= 24)
  if (length(i_nextday) > 0) time[i_nextday] <- time[i_nextday] - 24
  hours <- as.integer(time)
  minutes <- as.integer((time - hours) * 60)
  template[,1] <- fill_variable(template[,1], NA)
  template[,2] <- fill_variable(template[,2], NA)
  template[,3] <- fill_variable(template[,3], NA)
  dates <- as.Date(paste(template[,1], template[,2], template[,3], sep="-"))
  meta_time <- paste0("orig.date=", dates, "|orig.time=", template[,4], template[,5])
  if (length(i_nextday) > 0) {
    dates[i_nextday] <- dates[i_nextday] + 1
    template[i_nextday,1] <- format(dates[i_nextday], "%Y")
    template[i_nextday,2] <- format(dates[i_nextday], "%m")
    template[i_nextday,3] <- format(dates[i_nextday], "%d")
  }
  
  ## Initialize data frame
  out[[i]] <- data.frame(year = as.integer(template[,1]), 
                         month = as.integer(template[,2]), 
                         day = as.integer(template[,3]), 
                         hour = hours, 
                         minute = minutes)
  
  ## Pressure
  template[,6] <- fill_variable(template[,6], "-")
  out[[i]]$p <- round(convert_pressure(as.numeric(template[,6]), 25.4, 
                                       as.numeric(metadata$Latitude), 
                                       as.numeric(metadata$Altitude)), 
                      1)
  out[[i]]$p_meta <- paste0(meta_time, "|orig=", template[,6], "in")
  
  ## Temperature
  template[,7] <- fill_variable(template[,7], "-")
  template[,8] <- fill_variable(template[,8], "-")
  temp_rs <- as.numeric(template[,7])
  temp_f <- as.numeric(template[,8])
  out[[i]]$ta1 <- round((temp_f - 32) * 5 / 9, 1)
  temp_rs <- round(convert_rs(temp_rs, temp_f), 1)
  out[[i]]$ta2 <- round((temp_rs - 32) * 5 / 9, 1)
  out[[i]]$ta1_meta <- paste0(meta_time, "|orig=", template[,8], "F")
  out[[i]]$ta2_meta <- paste0(meta_time, "|orig=", template[,7], "H")
  
  ## Wind direction
  wdir <- fill_variable(template[,9], "-")
  wdir <- sub("by", "b", wdir)
  out[[i]]$dd <- round(11.25 * (match(wdir, directions) - 1), 0)
  out[[i]]$dd_meta <- paste0(meta_time, "|orig=", template[,9])
  
  
  ## Wind force
  template[,10] <- fill_variable(template[,10], "-")
  out[[i]]$w <- as.integer(template[,10])
  out[[i]]$w_meta <- paste0(meta_time, "|orig=", template[,10])
  
  ## Precipitation
  if (i > 1) {
    template[which(is.na(template[,11])),11] <- 0
    out[[i]]$rr <- round(as.numeric(template[,11]) * 25.4, 1)
    out[[i]]$rr_meta <- paste0(meta_time, "|orig=", template[,11], "in")
  } else {
    out[[i]]$rr <- NA
    out[[i]]$rr_meta <- ""
  }
  
}

## Merge data frames
out <- Reduce(rbind, out)

## Write SEF files
write_sef(out[,c("year","month","day","hour","minute","p")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "p",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "hPa",
          stat = "point",
          metaHead = paste0("Observer=", metadata$Observer, " | PGC=Y | PTC=N"),
          meta = out$p_meta,
          period = 0,
          time_offset = as.numeric(metadata$Longitude) * 12 / 180,
          keep_na = TRUE)

write_sef(out[,c("year","month","day","hour","minute","ta1")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "ta",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "C",
          stat = "point",
          metaHead = paste0("Observer=", metadata$Observer, " | Instrument=Mercury Fahrenheit thermometer"),
          meta = out$ta1_meta,
          period = 0,
          time_offset = as.numeric(metadata$Longitude) * 12 / 180,
          keep_na = TRUE,
          note = "F")

write_sef(out[,c("year","month","day","hour","minute","ta2")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "ta",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "C",
          stat = "point",
          metaHead = paste0("Observer=", metadata$Observer, " | Instrument=Royal Society's (Hauksbee) thermometer"),
          meta = out$ta2_meta,
          period = 0,
          time_offset = as.numeric(metadata$Longitude) * 12 / 180,
          keep_na = TRUE,
          note = "RS")

write_sef(out[,c("year","month","day","hour","minute","dd")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "dd",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "degree",
          stat = "point",
          metaHead = paste0("Observer=", metadata$Observer),
          meta = out$dd_meta,
          period = 0,
          time_offset = as.numeric(metadata$Longitude) * 12 / 180,
          keep_na = TRUE)

write_sef(out[,c("year","month","day","hour","minute","w")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "w",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "0-4",
          stat = "point",
          metaHead = paste0("Observer=", metadata$Observer),
          meta = out$w_meta,
          period = 0,
          time_offset = as.numeric(metadata$Longitude) * 12 / 180,
          keep_na = TRUE)

write_sef(out[,c("year","month","day","hour","minute","rr")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "rr",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "mm",
          stat = "sum",
          metaHead = paste0("Observer=", metadata$Observer),
          meta = out$rr_meta,
          period = "p",
          time_offset = as.numeric(metadata$Longitude) * 12 / 180,
          keep_na = TRUE)


## 1785-1787 (Palatina)

files <- c("USA_T3_US_Cambridge_1785_subdaily.xls",
           "USA_T3_US_Cambridge_1786_subdaily.xls",
           "USA_T3_US_Cambridge_1787_subdaily.xls")

## Read metadata
metadata <- inventory[which(inventory$Code == station_code & 
                              inventory$Resolution == "subdaily" &
                              inventory$First.Year == "1785"), ]

## Loop on files
out <- list()
for (i in 1:length(files)) {
  
  ## Read data
  template <- readWorksheetFromFile(paste0("../../3_Revised/Cambridge/",files[i]),
                                    sheet=1, startRow=8, header=FALSE, colTypes="character")
  
  ## Time
  time <- as.numeric(template[,4])
  hours <- as.integer(time)
  minutes <- as.integer((time - hours) * 60)
  dates <- as.Date(paste(template[,1], template[,2], template[,3], sep="-"))
  meta_time <- paste0("orig.date=", dates, "|orig.time=", template[,4])
  
  ## Initialize data frame
  out[[i]] <- data.frame(year = as.integer(template[,1]), 
                         month = as.integer(template[,2]), 
                         day = as.integer(template[,3]), 
                         hour = hours, 
                         minute = minutes)
  
  ## Pressure
  if (i < 3) {
    template[,5] <- fill_variable(template[,5], NA)
    pres <- as.numeric(template[,5]) + as.numeric(template[,6]) / 12
    out[[i]]$p <- round(convert_pressure(pres, 27.07, 
                                         as.numeric(metadata$Latitude), 
                                         as.numeric(metadata$Altitude),
                                         as.numeric(template[,7])), 
                        1)
    i_miss <- which(is.na(as.numeric(template[,7])))
    out[[i]]$p[i_miss] <- round(convert_pressure(pres[i_miss], 27.07, 
                                                 as.numeric(metadata$Latitude), 
                                                 as.numeric(metadata$Altitude)),
                                1)
    out[[i]]$p_meta <- paste0(meta_time, "|orig=", template[,5], ".", template[,6],
                              "Pin|atb=", template[,7], "R")
    out[[i]]$p_meta[i_miss] <- paste0(meta_time[i_miss], "|orig=", template[i_miss,5], ".", 
                                      template[i_miss,6], "Pin|atb=", template[i_miss,7], "|PTC=N")
  } else {
    out[[i]]$p <- NA
    out[[i]]$p_meta <- ""
  }
  
  ## Temperature
  if (i < 3) {
    out[[i]]$ta <- round(as.numeric(template[,8]) * 1.25, 1)
    out[[i]]$ta_meta <- paste0(meta_time, "|orig=", template[,8], "R")
  } else {
    out[[i]]$ta <- NA
    out[[i]]$ta_meta <- ""
  }
  
  ## Wind direction
  j <- ifelse(i < 3, 12, 5)
  directions <- gsub("E", "O", directions)
  out[[i]]$dd <- round(11.25 * (match(template[,j], directions) - 1), 0)
  out[[i]]$dd_meta <- paste0(meta_time, "|orig=", template[,j])
  
  ## Wind force
  j <- ifelse(i < 3, 13, 6)
  out[[i]]$w <- as.integer(template[,j])
  out[[i]]$w_meta <- paste0(meta_time, "|orig=", template[,j])
  
  ## Precipitation
  if (i < 3) {
    template[which(is.na(template[,14])),14] <- 0
    template[which(time!=14),14] <- NA
    out[[i]]$rr <- round(as.numeric(template[,14]) * 25.4, 1)
    out[[i]]$rr_meta <- paste0(meta_time, "|orig=", template[,14], "in")
  } else {
    out[[i]]$rr <- NA
    out[[i]]$rr_meta <- ""
  }
  
}

## Merge data frames
out <- Reduce(rbind, out)

## Read file with monthly extremes for 1787
template <- readWorksheetFromFile("../../3_Revised/Cambridge/USA_T3_US_Cambridge_1787_monthly.xls",
                                  sheet=1, startRow=8, header=FALSE, colTypes="character")

## Loop over months
for (im in 1:12) {
  ## Max pressure
  pmax <- as.numeric(template[im,3]) + as.numeric(template[im,4]) / 12
  pmax <- convert_pressure(pmax, 27.07, as.numeric(metadata$Latitude), 
                           as.numeric(metadata$Altitude))
  daymax <- as.integer(strsplit(template[im,5], "&")[[1]])
  hourmax <- as.integer(strsplit(template[im,6], "&")[[1]])
  for (d in daymax) {
    date_meta <- as.Date(paste(1787, im, d, sep="-"))
    for (h in hourmax) {
      out$p[which(out$day==d & out$month==im & out$year==1787 & out$hour==h)] <- round(pmax, 1)
      out$p_meta[which(out$day==d & out$month==im & out$year==1787 & out$hour==h)] <- 
        paste0("orig.date=", date_meta, "|orig.time=", h, 
               "|orig=", template[im,3], ".", template[im,4], "Pin|PTC=N")
    }
  }
  ## Min pressure
  pmin <- as.numeric(template[im,7]) + as.numeric(template[im,8]) / 12
  pmin <- convert_pressure(pmin, 27.07, as.numeric(metadata$Latitude), 
                           as.numeric(metadata$Altitude))
  daymin <- as.integer(strsplit(template[im,9], "&")[[1]])
  hourmin <- as.integer(strsplit(template[im,10], "&")[[1]])
  for (d in daymin) {
    date_meta <- as.Date(paste(1787, im, d, sep="-"))
    for (h in hourmin) {
      out$p[which(out$day==d & out$month==im & out$year==1787 & out$hour==h)] <- round(pmin, 1)
      out$p_meta[which(out$day==d & out$month==im & out$year==1787 & out$hour==h)] <- 
        paste0("orig.date=", date_meta, "|orig.time=", h, 
               "|orig=", template[im,7], ".", template[im,8], "Pin|PTC=N")
    }
  }
  ## Max temperature
  tmax <- as.numeric(template[im,13]) * 1.25
  daymax <- as.integer(strsplit(template[im,14], "&")[[1]])
  hourmax <- as.integer(strsplit(template[im,15], "&")[[1]])
  for (d in daymax) {
    date_meta <- as.Date(paste(1787, im, d, sep="-"))
    for (h in hourmax) {
      out$ta[which(out$day==d & out$month==im & out$year==1787 & out$hour==h)] <- round(tmax, 1)
      out$ta_meta[which(out$day==d & out$month==im & out$year==1787 & out$hour==h)] <- 
        paste0("orig.date=", date_meta, "|orig.time=", h, "|orig=", template[im,13], "R")
    }
  }
  ## Min temperature
  tmin <- as.numeric(template[im,16]) * 1.25
  daymin <- as.integer(strsplit(template[im,17], "&")[[1]])
  hourmin <- as.integer(strsplit(template[im,18], "&")[[1]])
  for (d in daymin) {
    date_meta <- as.Date(paste(1787, im, d, sep="-"))
    for (h in hourmin) {
      out$ta[which(out$day==d & out$month==im & out$year==1787 & out$hour==h)] <- round(tmin, 1)
      out$ta_meta[which(out$day==d & out$month==im & out$year==1787 & out$hour==h)] <- 
        paste0("orig.date=", date_meta, "|orig.time=", h, "|orig=", template[im,16], "R")
    }
  }
}

## Write SEF files
write_sef(out[,c("year","month","day","hour","minute","p")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "p",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "hPa",
          stat = "point",
          metaHead = paste0("Observer=", metadata$Observer, 
                            " | PGC=Y | PTC=Y | Instrument=Palatine Society"),
          meta = out$p_meta,
          period = 0,
          time_offset = as.numeric(metadata$Longitude) * 12 / 180,
          keep_na = FALSE)

write_sef(out[,c("year","month","day","hour","minute","ta")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "ta",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "C",
          stat = "point",
          metaHead = paste0("Observer=", metadata$Observer, " | Instrument=Palatine Society"),
          meta = out$ta_meta,
          period = 0,
          time_offset = as.numeric(metadata$Longitude) * 12 / 180,
          keep_na = FALSE)

write_sef(out[,c("year","month","day","hour","minute","dd")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "dd",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "degree",
          stat = "point",
          metaHead = paste0("Observer=", metadata$Observer),
          meta = out$dd_meta,
          period = 0,
          time_offset = as.numeric(metadata$Longitude) * 12 / 180,
          keep_na = TRUE)

write_sef(out[,c("year","month","day","hour","minute","w")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "w",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "0-4",
          stat = "point",
          metaHead = paste0("Observer=", metadata$Observer),
          meta = out$w_meta,
          period = 0,
          time_offset = as.numeric(metadata$Longitude) * 12 / 180,
          keep_na = TRUE)

write_sef(out[,c("year","month","day","hour","minute","rr")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "rr",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "mm",
          stat = "sum",
          metaHead = paste0("Observer=", metadata$Observer),
          meta = out$rr_meta,
          period = "day",
          time_offset = as.numeric(metadata$Longitude) * 12 / 180,
          keep_na = FALSE)



## MONTHLY

## 1742-1775
metadata <- inventory[which(inventory$Code == station_code & 
                              inventory$Resolution == "monthly" &
                              inventory$First.Year == "1742"), ]
template <- readWorksheetFromFile("../../3_Revised/Cambridge/USA_T3_US_Cambridge_1742-1775_monthly.xls",
                                  sheet=1, startRow=8, header=FALSE, colTypes="character")

out <- data.frame(Year = as.integer(template[,1]),
                  Month = as.integer(template[,2]),
                  Day = rep(NA, nrow(template)),
                  Hour = rep("", nrow(template)),
                  Minute = rep("", nrow(template)),
                  stringsAsFactors = FALSE)

out$p <- convert_pressure(as.numeric(template[,3]),  25.4, 
                          as.numeric(metadata$Latitude), as.numeric(metadata$Altitude))
out$p <- round(out$p, 2)
out$p_meta <- paste0("orig=", template[,3], "in")

write_sef(out[,1:6],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "p",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "hPa",
          stat = "mean",
          metaHead = paste0("Observer=", metadata$Observer, 
                            " | PGC=Y | PTC=N"),
          meta = out$p_meta,
          period = "month",
          keep_na = TRUE,
          note = "monthly")


## 1780-1784
metadata <- inventory[which(inventory$Code == station_code & 
                              inventory$Resolution == "monthly" &
                              inventory$First.Year == "1780"), ]

template <- readWorksheetFromFile("../../3_Revised/Cambridge/USA_T3_US_Cambridge_1780-1783_monthly.xls",
                                  sheet=1, startRow=8, header=FALSE, colTypes="character")

out <- data.frame(Year = as.integer(template[,1]),
                  Month = as.integer(template[,2]),
                  Day = rep(NA, nrow(template)),
                  Hour = rep("", nrow(template)),
                  Minute = rep("", nrow(template)),
                  stringsAsFactors = FALSE)

out$pmax <- convert_pressure(as.numeric(template[,7]),  25.4, 
                             as.numeric(metadata$Latitude), as.numeric(metadata$Altitude))
out$pmax <- round(out$pmax, 1)
out$pmax_meta <- paste0("orig=", template[,7], "in")

out$pmin <- convert_pressure(as.numeric(template[,8]),  25.4, 
                             as.numeric(metadata$Latitude), as.numeric(metadata$Altitude))
out$pmin <- round(out$pmin, 1)
out$pmin_meta <- paste0("orig=", template[,8], "in")

out$p <- convert_pressure(as.numeric(template[,10]),  25.4, 
                          as.numeric(metadata$Latitude), as.numeric(metadata$Altitude))
out$p <- round(out$p, 1)
out$p_meta <- paste0("orig=", template[,10], "in")

out$tmax <- round((as.numeric(template[,3]) - 32) * 5 / 9, 1)
out$tmax_meta <- paste0("orig=", template[,3], "F")

out$tmin <- round((as.numeric(template[,4]) - 32) * 5 / 9, 1)
out$tmin_meta <- paste0("orig=", template[,4], "F")

out$ta <- round((as.numeric(template[,6]) - 32) * 5 / 9, 1)
out$ta_meta <- paste0("orig=", template[,6], "F")

template <- readWorksheetFromFile("../../3_Revised/Cambridge/USA_T3_US_Cambridge_1784_monthly.xls",
                                  sheet=1, startRow=8, header=FALSE, colTypes="character")

out2 <- data.frame(Year = as.integer(template[,1]),
                   Month = as.integer(template[,2]),
                   Day = rep(NA, nrow(template)),
                   Hour = rep("", nrow(template)),
                   Minute = rep("", nrow(template)),
                   stringsAsFactors = FALSE)

out2$pmax <- convert_pressure(as.numeric(template[,3]),  25.4, 
                              as.numeric(metadata$Latitude), as.numeric(metadata$Altitude))
out2$pmax <- round(out2$pmax, 1)
out2$pmax_meta <- paste0("orig=", template[,3], "in")

out2$pmin <- convert_pressure(as.numeric(template[,4]),  25.4, 
                              as.numeric(metadata$Latitude), as.numeric(metadata$Altitude))
out2$pmin <- round(out2$pmin, 1)
out2$pmin_meta <- paste0("orig=", template[,4], "in")

out2$p <- NA
out2$p_meta <- ""

out2$tmax <- round((as.numeric(template[,6]) - 32) * 5 / 9, 1)
out2$tmax_meta <- paste0("orig=", template[,6], "F")

out2$tmin <- round((as.numeric(template[,7]) - 32) * 5 / 9, 1)
out2$tmin_meta <- paste0("orig=", template[,7], "F")

out2$ta <- round((as.numeric(template[,9]) - 32) * 5 / 9, 1)
out2$ta_meta <- paste0("orig=", template[,9], "F")

out <- rbind(out, out2)

write_sef(out[,c("Year","Month","Day","Hour","Minute","pmax")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "p",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "hPa",
          stat = "maximum",
          metaHead = paste0("Observer=", metadata$Observer, 
                            " | PGC=Y | PTC=N"),
          meta = out$pmax_meta,
          period = "month",
          keep_na = TRUE,
          note = "monthly-max")

write_sef(out[,c("Year","Month","Day","Hour","Minute","pmin")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "p",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "hPa",
          stat = "minimum",
          metaHead = paste0("Observer=", metadata$Observer, 
                            " | PGC=Y | PTC=N"),
          meta = out$pmin_meta,
          period = "month",
          keep_na = TRUE,
          note = "monthly-min")

write_sef(out[,c("Year","Month","Day","Hour","Minute","p")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "p",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "hPa",
          stat = "mean",
          metaHead = paste0("Observer=", metadata$Observer, 
                            " | PGC=Y | PTC=N"),
          meta = out$p_meta,
          period = "month",
          keep_na = TRUE,
          note = "monthly")

write_sef(out[,c("Year","Month","Day","Hour","Minute","tmax")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "ta",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "C",
          stat = "maximum",
          metaHead = paste0("Observer=", metadata$Observer),
          meta = out$tmax_meta,
          period = "month",
          keep_na = TRUE,
          note = "monthly-max")

write_sef(out[,c("Year","Month","Day","Hour","Minute","tmin")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "ta",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "C",
          stat = "minimum",
          metaHead = paste0("Observer=", metadata$Observer),
          meta = out$tmin_meta,
          period = "month",
          keep_na = TRUE,
          note = "monthly-min")

write_sef(out[,c("Year","Month","Day","Hour","Minute","ta")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "ta",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "C",
          stat = "mean",
          metaHead = paste0("Observer=", metadata$Observer),
          meta = out$ta_meta,
          period = "month",
          keep_na = TRUE,
          note = "monthly")


## 1790-1812
metadata <- inventory[which(inventory$Code == station_code & 
                              inventory$Resolution == "monthly" &
                              inventory$First.Year == "1790"), ]
template <- readWorksheetFromFile("../../3_Revised/Cambridge/USA_T3_US_Cambridge_1790-1812_monthly.xls",
                                  sheet=1, startRow=8, header=FALSE, colTypes="character")
template <- template[1:22,]

out <- data.frame(Year = as.integer(rep(template[,1], each=12)),
                  Month = rep(1:12, nrow(template)),
                  Day = rep(NA, nrow(template)*12),
                  Hour = rep("", nrow(template)*12),
                  Minute = rep("", nrow(template)*12),
                  Value = as.numeric(c(t(template[,2:13]))),
                  Meta = paste0("orig=", c(t(template[,2:13])), "in|atb==55F"),
                  stringsAsFactors = FALSE)

## Pressure was reduced to 55F
out$Value <- convert_pressure(out$Value,  25.4, as.numeric(metadata$Latitude), 
                              as.numeric(metadata$Altitude), rep((55-32)*5/9, nrow(out)))
out$Value <- round(out$Value, 1)

## Add metadata on barometer
out$Meta[1:(12*12+7)] <- paste0(out$Meta[1:(12*12+7)], "|Instrument=Palatine Society")
out$Meta[(12*12+8):(19*12+1)] <- paste0(out$Meta[(12*12+8):(19*12+1)], 
                                        "|Instrument=made by Champney")
out$Meta[(20*12+1):(22*12)] <- paste0(out$Meta[(20*12+1):(22*12)], 
                                      "|Instrument=made by W. & S. Jones, London, ",
                                      "provided with a floating gage and a scale of correction")

## Write file
write_sef(out[,1:6],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "p",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "hPa",
          stat = "mean",
          metaHead = paste0("Observer=", metadata$Observer, " | PGC=Y | PTC=Y"),
          meta = out$Meta,
          period = "month",
          keep_na = TRUE,
          note = "monthly")


file.rename(list.files(paste0("../../4_Formatted/", name_folder), full.names=TRUE), 
            gsub("NA", "", list.files(paste0("../../4_Formatted/", name_folder), full.names=TRUE)))


## Check SEF files
for (f in list.files(paste0("../../4_Formatted/", name_folder), full.names=TRUE)) {
  print(f)
  check_sef(f)
}
