library(dataresqc)
library(XLConnect)

## Create station code
name_folder <- strsplit(getwd(), "/")[[1]][8]
name_group <- strsplit(getwd(), "/")[[1]][6]
station_code <- paste(name_group, name_folder, sep = "_")

## Create output directory (if necessary)
if (!name_folder %in% list.dirs("../../4_Formatted/",full.names=FALSE)) {
  dir.create(paste0("../../4_Formatted/", name_folder))
}

## Read inventory
inventory <- readWorksheetFromFile("/scratch3/PALAEO-RA/DataRescue/Inventory_PALAEO-RA_digi.xls",
                                 sheet=1, colTypes="character")

## Read data and metadata
template <- readWorksheetFromFile("../../3_Revised/Charleston/USA_T3_US_Charleston_1738-1759_monthly.xls",
                                  sheet=1, startRow=8, header=FALSE, colTypes="character")
metadata <- inventory[which(inventory$Code == station_code & 
                              inventory$Resolution == "monthly"), ]

## Fill in missing years
for (i in 2:nrow(template)) {
  if (is.na(template[i,1])) template[i,1] <- template[i-1,1]
}

## Create data frame with SEF structure
out <- data.frame(Year = as.integer(template[,1]),
                  Month = as.integer(template[,2]),
                  Day = rep(NA, nrow(template)),
                  Hour = rep("", nrow(template)),
                  Minute = rep(0, nrow(template)),
                  Ta = round((as.numeric(template[,3])-32)*5/9, 1),
                  Rr = round(25.4 * as.numeric(template[,4]), 1),
                  Meta_ta = paste0("orig=", template[,3], "F"),
                  Meta_rr = paste0("orig=", template[,4], "in"),
                  stringsAsFactors = FALSE)

## Write SEF file for temperature
write_sef(out[1:60,1:6],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "ta",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "C",
          stat = "mean",
          metaHead = paste0("Observer=", metadata$Observer),
          meta = out$Meta_ta[1:60],
          period = "month",
          note = "monthly",
          keep_na = TRUE)

## Write SEF file for precipitation
write_sef(out[,c(1:5,7)],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "rr",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "mm",
          stat = "sum",
          metaHead = paste0("Observer=", metadata$Observer),
          meta = out$Meta_rr,
          period = "month",
          note = "monthly",
          keep_na = TRUE)

file.rename(list.files(paste0("../../4_Formatted/", name_folder), full.names=TRUE), 
            gsub("NA", "", list.files(paste0("../../4_Formatted/", name_folder), full.names=TRUE)))

## Check SEF files
for (f in list.files(paste0("../../4_Formatted/", name_folder), full.names=TRUE)) check_sef(f)