library(dataresqc)
library(XLConnect)
# package "zoo" is needed to replace all "ditto" values in the wind directions data frame to previous value
library(zoo)

# Create station code
name_folder <- strsplit(getwd(), "/")[[1]][8]
name_group <- strsplit(getwd(), "/")[[1]][6]
station_code <- paste(name_group, name_folder, sep = "_")

# Create output directory (if necessary)
if (!name_folder %in% list.dirs("../../4_Formatted/",full.names=FALSE)) {
  dir.create(paste0("../../4_Formatted/", name_folder))
}

# Read inventory
inventory <- readWorksheetFromFile("/scratch3/PALAEO-RA/DataRescue/Inventory_PALAEO-RA_digi.xls",
                                   sheet=1, colTypes="character")


# SUBDAILY
template <- readWorksheetFromFile("/scratch3/PALAEO-RA/DataRescue/Projects/USA/3_Revised/Detroit/USA_T3_US_Detroit_1781-1786_subdaily.xls",
                                  sheet=1, startRow=8, endRow=1772, header=FALSE, colTypes="character")
metadata <- inventory[which(inventory$Code == station_code & 
                              inventory$Resolution == "subdaily"), ]

## Change data frame structure

# Create empty list
sefdata <- list()

# Create data frame for temperature (under name 'ta')
sefdata$ta <- data.frame(Year = rep(as.integer(template[,1]), each=3),
                         Month = rep(as.integer(template[,2]), each=3),
                         Day = rep(as.integer(template[,3]), each=3),
                         Value = as.integer(c(t(template[,4:6]))))

# TIME
hours <- rep(c(7, 12, 20), times = 1765)
for (v in names(sefdata)) {
  sefdata[[v]]$Hour <- hours
  sefdata[[v]] <- sefdata[[v]][, c("Year", "Month", "Day",
                                   "Hour", "Value")]
}

minutes <- rep(0, nrow(template))
for (v in names(sefdata)) {
  sefdata[[v]]$Minute <- minutes
  sefdata[[v]] <- sefdata[[v]][, c("Year", "Month", "Day",
                                   "Hour","Minute", "Value")]
}

# Add Meta column
original_units <- c("F")
obs_times <- c("7", "12", "8")
dates <- as.Date(paste(as.integer(template[,1]), as.integer(template[,2]), as.integer(template[,3]), sep="-"))
names(original_units) <- names(sefdata)

for (v in names(sefdata)) {
  sefdata[[v]]$Meta <- paste0("orig=", sefdata[[v]]$Value, original_units[v],
                              "|orig.time=", rep(obs_times, times=1765),
                              "|orig.date=", rep(dates, each=3))
}


# Fahrenheit to Celsius
sefdata$ta$Value <- round((sefdata$ta$Value - 32) * 5 / 9, 1)


# Write SEF files

units <- c("C")
names(units) <- names(sefdata)
longitude <- as.numeric(metadata$Longitude)
for (v in names(sefdata)) {
  write_sef(Data = sefdata[[v]][, 1:7],
            outpath = paste0("/scratch3/PALAEO-RA/DataRescue/Projects/USA/4_Formatted/", name_folder),
            variable = v,
            cod = metadata$Code,
            nam = metadata$Station,
            lat = metadata$Latitude,
            lon = metadata$Longitude,
            alt = metadata$Altitude,
            sou = "PALAEO-RA",
            units = units[v],
            stat = "point",
            metaHead = paste0("Observer=Anthon",
                              ifelse(v=="p", "|PTC=N|PGC=Y", "")),
            meta=sefdata$ta$Meta,
            period = 0,
            time_offset = longitude*12/180,
            keep_na = TRUE)
}

## Check SEF files
for (f in list.files(paste0("../../4_Formatted/", name_folder), full.names=TRUE)) check_sef(f)


# same procedure for wind direction

# Create data frame for wind directions (under name 'dd')
directions <- c("N", "NNE", "NE", "ENE", "E", "ESE", "SE", "SSE", "S",
                "SSW", "SW", "WSW", "W", "WNW", "NW", "NNW")
wdir <- template[,7]

# replacing all "ditto" with previous value -> need to install package "zoo"
is.na(wdir) <- wdir == "ditto"
wdir        <- na.locf(wdir,na.rm=FALSE)

# change wind directions to decimal value
wdir2 <- 22.5 * (match(toupper(wdir), directions) - 1)

# Create empty list
sefdata2 <- list()

# add data to list
sefdata2$dd <- data.frame(Year = rep(as.integer(template[,1]), each=1),
                         Month = rep(as.integer(template[,2]), each=1),
                         Day = rep(as.integer(template[,3]), each=1),
                         Hour = as.numeric(rep("",nrow(template))),
                         Minute = as.numeric(rep(NA,nrow(template))),
                         Value = wdir2)

# Add Meta column
dates <- as.Date(paste(as.integer(template[,1]), as.integer(template[,2]), as.integer(template[,3]), sep="-"))
names(original_units) <- names(sefdata2)


for (v in names(sefdata2)) {
  sefdata2[[v]]$Meta <- paste0("orig=", template[,7],
                              "|orig.time=", rep(NA, times=1765),
                              "|orig.date=", rep(dates, each=1))
}

# Write SEF Files

units2 <- c("degree")
names(units2) <- names(sefdata2)
for (v in names(sefdata2)) {
  write_sef(Data = sefdata2[[v]][, 1:7],
            outpath = paste0("/scratch3/PALAEO-RA/DataRescue/Projects/USA/4_Formatted/", name_folder),
            variable = v,
            cod = metadata$Code,
            nam = metadata$Station,
            lat = metadata$Latitude,
            lon = metadata$Longitude,
            alt = metadata$Altitude,
            sou = "PALAEO-RA",
            units = units2[v],
            stat = "point",
            metaHead = paste0("Observer=Anthon",
                              ifelse(v=="p", "|PTC=N|PGC=Y", "")),
            meta = sefdata2$dd$Meta,
            period = 0,
            time_offset = longitude*12/180,
            keep_na=TRUE)
}

## Check SEF files
for (f in list.files(paste0("../../4_Formatted/", name_folder), full.names=TRUE)) check_sef(f)
