## NOTE: Tmax was not formatted (only occasionally measured)

library(dataresqc)
library(XLConnect)
library(lubridate)
library(suncalc)

## Create station code
name_folder <- strsplit(getwd(), "/")[[1]][8]
name_group <- strsplit(getwd(), "/")[[1]][6]
station_code <- paste(name_group, name_folder, sep = "_")

## Create output directory (if necessary)
if (!name_folder %in% list.dirs("../../4_Formatted/",full.names=FALSE)) {
  dir.create(paste0("../../4_Formatted/", name_folder))
}

## Read inventory
inventory <- readWorksheetFromFile("/scratch3/PALAEO-RA/DataRescue/Inventory_PALAEO-RA_digi.xls",
                                   sheet=1, colTypes="character")

## Define function to fill missing values with the value on the previous row
fill_variable <- function(timeseries, strings) { 
  for (i in 2:length(timeseries)) {
    if (timeseries[i] %in% strings) {
      if(!timeseries[i-1] %in% strings) {
        timeseries[i] <- timeseries[i-1]
      } 
    }
  }
  return(timeseries)
}

## Define wind directions
directions <- c("N", "N b E", "NNE", "NE b N", "NE", "NE b E", "ENE", "E b N", "E", 
                "E b S", "ESE", "SE b E", "SE", "SE b S", "SSE", "S b E", "S", 
                "S b W", "SSW", "SW b S", "SW", "SW b W", "WSW", "W b S", "W",
                "W b N", "WNW", "NW b W", "NW", "NW b N", "NNW", "N b W¨")

## Read metadata
metadata <- inventory[which(inventory$Code == station_code), ]

## Read data
files <- list.files("../../3_Revised/Orange/", full.names=TRUE)
template1 <- readWorksheetFromFile(files[1], sheet=1, startRow=8, header=FALSE, colTypes="character")
template2 <- readWorksheetFromFile(files[2], sheet=1, startRow=8, header=FALSE, colTypes="character")
template1 <- template1[, c(1:5,8,6,7)]
template2 <- template2[, 1:8]
names(template1) <- names(template2)
template <- rbind(template1, template2)
n <- nrow(template)

## Time
dates <- as.Date(paste(template[,1], template[,2], template[,3], sep="-"))
sunrises <- getSunlightTimes(dates, as.numeric(metadata$Latitude), 
                             as.numeric(metadata$Longitude), keep="sunrise", tz="GMT")$sunrise
sunrises <- sunrises + 3600 * as.numeric(metadata$Longitude) * 12 / 180
hours_ta <- c(hour(sunrises), rep(14,n), rep(16,n))
minutes_ta <- c(minute(sunrises), rep(0,2*n))
hours_dd <- c(hour(sunrises), rep(16,n))
minutes_dd <- c(minute(sunrises), rep(0,n))
meta_time_ta <- paste0("orig.time=", c(rep("sunrise",n), rep("2pm",n), rep("4pm",n)))
meta_time_dd <- paste0("orig.time=", c(rep("sunrise",n), rep("4pm",n)))


## TEMPERATURE ##
out <- data.frame(year = as.integer(template[,1]), 
                  month = as.integer(template[,2]), 
                  day = as.integer(template[,3]), 
                  hour = hours_ta, 
                  minute = minutes_ta)

temp <- c(template[,c(4,6,7)], recursive=TRUE)
out$ta <- round((as.numeric(temp) - 32) * 5/9, 1)
out$ta_meta <- paste0(meta_time_ta, "|orig=", temp, "F")
out <- out[order(out$year,out$month,out$day,out$hour), ]
write_sef(out,
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "ta",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "C",
          stat = "point",
          metaHead = paste0("Observer=", metadata$Observer),
          meta = out$ta_meta,
          period = 0,
          time_offset = as.numeric(metadata$Longitude) * 12 / 180,
          keep_na = TRUE)


## WIND ##
out <- data.frame(year = as.integer(template[,1]), 
                  month = as.integer(template[,2]), 
                  day = as.integer(template[,3]), 
                  hour = hours_dd, 
                  minute = minutes_dd)
out <- out[order(out$year,out$month,out$day,out$hour), ]
template[,5] <- fill_variable(template[,5], c("ibidem","ib","idem"))
template[,8] <- fill_variable(template[,8], c("ibidem","ib","idem"))
wdir <- c(template[,c(5,8)], recursive=TRUE)
wdir <- sub("by", "b", wdir)
out$dd <- round(11.25 * (match(wdir, directions) - 1), 0)
out$dd_meta <- paste0(meta_time_dd, "|orig=", wdir)

write_sef(out,
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "dd",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "degree",
          stat = "point",
          metaHead = paste0("Observer=", metadata$Observer),
          meta = out$dd_meta,
          period = 0,
          time_offset = as.numeric(metadata$Longitude) * 12 / 180,
          keep_na = TRUE)


## Check SEF files
for (f in list.files(paste0("../../4_Formatted/", name_folder), full.names=TRUE)) {
  print(f)
  check_sef(f)
}