## NOTES:
## There are 4 different stations in this folder:
## Near Philadelphia, Philadelphia (Pemberton), Philadelphia (Rittenhouse), Philadelphia (2 miles W of)

library(dataresqc)
library(XLConnect)
library(lubridate)

## Create station code
name_folder <- strsplit(getwd(), "/")[[1]][8]
name_group <- strsplit(getwd(), "/")[[1]][6]
station_code <- paste(name_group, name_folder, sep = "_")

## Create output directory (if necessary)
if (!name_folder %in% list.dirs("../../4_Formatted/",full.names=FALSE)) {
  dir.create(paste0("../../4_Formatted/", name_folder))
}

## Read inventory
inventory <- readWorksheetFromFile("/scratch3/PALAEO-RA/DataRescue/Inventory_PALAEO-RA_digi.xls",
                                 sheet=1, colTypes="character")

## Define function to fill missing values with the value on the previous row
fill_variable <- function(timeseries, strings) { 
  for (i in 2:length(timeseries)) {
    if (timeseries[i] %in% strings) {
      if(!timeseries[i-1] %in% strings) {
        timeseries[i] <- timeseries[i-1]
      } 
    }
  }
  return(timeseries)
}

## Define wind directions
directions <- c("N", "NbE", "NNE", "NEbN", "NE", "NEbE", "ENE", "EbN", "E", 
                "EbS", "ESE", "SEbE", "SE", "SEbS", "SSE", "SbE", "S",
                "SbW", "SSW", "SWbS", "SW", "SWbW", "WSW", "WbS", "W",
                "WbN", "WNW", "NWbW", "NW", "NWbN", "NNW", "NbW")


## SUBDAILY

## NEAR PHILADELPHIA 

files <- list.files("../../3_Revised/Philadelphia/", pattern="NearPhiladelphia",
                    full.names=TRUE)

## Read metadata
metadata <- inventory[which(inventory$Code == paste0(station_code, "_1")), ]

## Loop on files
out <- list()
for (i in 1:length(files)) {

## Read data
template <- readWorksheetFromFile(files[i],
                                  sheet=1, startRow=8, header=FALSE, colTypes="character")

## Time
times <- strptime(template[,4], format="%Y-%m-%d %H:%M:%S")
hours <- hour(times)
minutes <- minute(times)
template[,1] <- fill_variable(template[,1], NA)
template[,2] <- fill_variable(template[,2], NA)
template[,3] <- fill_variable(template[,3], NA)
dates <- as.Date(paste(template[,1], template[,2], template[,3], sep="-"))
meta_time <- paste0("orig.date=", dates, "|orig.time=", substr(template[,4],12,19))

## Fix mistake in evening observations
if (i == 4) {
  i_evening <- which(hours>=20 & is.na(template[,9]))
  template[i_evening,9] <- template[i_evening,8]
  template[i_evening,8] <- NA
}

## Initialize data frame
out[[i]] <- data.frame(year = as.integer(template[,1]), 
                       month = as.integer(template[,2]), 
                       day = as.integer(template[,3]), 
                       hour = hours, 
                       minute = minutes)

## Pressure
pres <- as.numeric(template[,5]) + 0.1 * (as.integer(template[,6]) / as.integer(template[,7]))
i0 <- which(is.na(template[,6]))
pres[i0] <- as.numeric(template[i0,5])
out[[i]]$p <- round(convert_pressure(pres, 25.4, 
                                     as.numeric(metadata$Latitude), 
                                     as.numeric(metadata$Altitude)), 
                    1)
out[[i]]$p_meta <- paste0(meta_time, "|orig=", template[,5], ".", 
                          template[,6], "/", template[,7], "in")
out[[i]]$p_meta[i0] <- paste0(meta_time[i0], "|orig=", template[i0,5], "in")
if (i == 4) {
  i_atb <- which(!is.na(template[,8]) & !is.na(pres))
  out[[i]]$p[i_atb] <- round(convert_pressure(pres[i_atb], 25.4, 
                                              as.numeric(metadata$Latitude), 
                                              as.numeric(metadata$Altitude),
                                              (as.numeric(template[i_atb,8])-32)*5/9), 
                             1)
  out[[i]]$p_meta[i_atb] <- paste0(out[[i]]$p_meta[i_atb], "|atb=", 
                                   template[i_atb,8], "F|PTC=Y")
}

## Temperature
temp <- as.numeric(template[,ifelse(i<4, 8, 9)])
out[[i]]$ta <- round((temp - 32) * 5 / 9, 1)
out[[i]]$ta_meta <- paste0(meta_time, "|orig=", template[,ifelse(i<4, 8, 9)], "F")
          
## Wind direction
wdir <- template[,ifelse(i<4, 9, 10)]
wdir <- sub("t", "b", wdir)
wdir <- sub("by", "b", wdir)
wdir <- gsub(" ", "", wdir)
out[[i]]$dd <- round(11.25 * (match(wdir, directions) - 1), 0)
out[[i]]$dd_meta <- paste0(meta_time, "|orig=", template[,ifelse(i<4, 9, 10)])

}

## Merge data frames
out <- Reduce(rbind, out)

## Write SEF files
write_sef(out[,c("year","month","day","hour","minute","p")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "p",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "hPa",
          stat = "point",
          metaHead = paste0("Observer=", metadata$Observer, " | PGC=Y | PTC=N"),
          meta = out$p_meta,
          period = 0,
          time_offset = as.numeric(metadata$Longitude) * 12 / 180,
          keep_na = TRUE)

write_sef(out[,c("year","month","day","hour","minute","ta")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "ta",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "C",
          stat = "point",
          metaHead = paste0("Observer=", metadata$Observer),
          meta = out$ta_meta,
          period = 0,
          time_offset = as.numeric(metadata$Longitude) * 12 / 180,
          keep_na = TRUE)

write_sef(out[,c("year","month","day","hour","minute","dd")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "dd",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "degree",
          stat = "point",
          metaHead = paste0("Observer=", metadata$Observer),
          meta = out$dd_meta,
          period = 0,
          time_offset = as.numeric(metadata$Longitude) * 12 / 180,
          keep_na = TRUE)


## WEST OF PHILADELPHIA

## Read metadata
metadata <- inventory[which(inventory$Code == paste0(station_code, "_4") &
                            inventory$Resolution == "subdaily"), ]
## Read data
template <- readWorksheetFromFile("../../3_Revised/Philadelphia/USA_T3_US_WestPhiladelphia_1772_subdaily.xls",
                                  sheet=1, startRow=8, header=FALSE, colTypes="character")

## Time
times <- strptime(template[,4], format="%Y-%m-%d %H:%M:%S")
hours <- hour(times)
minutes <- minute(times)
template[,1] <- fill_variable(template[,1], NA)
template[,2] <- fill_variable(template[,2], NA)
template[,3] <- fill_variable(template[,3], NA)
dates <- as.Date(paste(template[,1], template[,2], template[,3], sep="-"))
meta_time <- paste0("orig.date=", dates, "|orig.time=", substr(template[,4],12,19))

## Initialize data frame
out <- data.frame(year = as.integer(template[,1]), 
                  month = as.integer(template[,2]), 
                  day = as.integer(template[,3]), 
                  hour = hours, 
                  minute = minutes)

## Pressure
pres <- as.numeric(template[,5]) + 0.1 * (as.integer(template[,6]) / as.integer(template[,7]))
i0 <- which(is.na(template[,6]))
pres[i0] <- as.numeric(template[i0,5])
out$p <- round(convert_pressure(pres, 25.4, 
                                as.numeric(metadata$Latitude), 
                                as.numeric(metadata$Altitude),
                                (as.numeric(template[,8])-32)*5/9),
               1)
out$p_meta <- paste0(meta_time, "|orig=", template[,5], ".", 
                     template[,6], "/", template[,7], "in")
out$p_meta[i0] <- paste0(meta_time[i0], "|orig=", template[i0,5], "in")
i_atb <- (!is.na(template[,8]) & !is.na(pres))
out$p_meta[i_atb] <- paste0(out$p_meta[i_atb], "|atb=", template[i_atb,8], "F")
out$p[!i_atb] <- round(convert_pressure(pres[!i_atb], 25.4, 
                                        as.numeric(metadata$Latitude), 
                                        as.numeric(metadata$Altitude)),
                       1)
out$p_meta[!i_atb] <- paste0(out$p_meta[!i_atb], "|PTC=N")

## Temperature
temp <- as.numeric(template[,9])
out$ta <- round((temp - 32) * 5 / 9, 1)
out$ta_meta <- paste0(meta_time, "|orig=", template[,9], "F")
          
## Wind direction
wdir <- template[,10]
wdir <- sub("t", "b", wdir)
wdir <- sub("by", "b", wdir)
wdir <- gsub(" ", "", wdir)
out$dd <- round(11.25 * (match(wdir, directions) - 1), 0)
out$dd_meta <- paste0(meta_time, "|orig=", template[,10])

## Write SEF files
write_sef(out[,c("year","month","day","hour","minute","p")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "p",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "hPa",
          stat = "point",
          metaHead = paste0("Observer=", metadata$Observer, " | PGC=Y | PTC=Y"),
          meta = out$p_meta,
          period = 0,
          time_offset = as.numeric(metadata$Longitude) * 12 / 180,
          keep_na = TRUE)

write_sef(out[,c("year","month","day","hour","minute","ta")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "ta",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "C",
          stat = "point",
          metaHead = paste0("Observer=", metadata$Observer),
          meta = out$ta_meta,
          period = 0,
          time_offset = as.numeric(metadata$Longitude) * 12 / 180,
          keep_na = TRUE)

write_sef(out[,c("year","month","day","hour","minute","dd")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "dd",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "degree",
          stat = "point",
          metaHead = paste0("Observer=", metadata$Observer),
          meta = out$dd_meta,
          period = 0,
          time_offset = as.numeric(metadata$Longitude) * 12 / 180,
          keep_na = TRUE)


## PHILADELPHIA (PEMBERTON) 

files <- list.files("../../3_Revised/Philadelphia/", pattern="Pemberton",
                    full.names=TRUE)
files <- files[-grep("monthly", files)]

## Read metadata
metadata <- inventory[which(inventory$Code == paste0(station_code, "_2")), ]

out <- list()

## 1767
## Read data
template <- readWorksheetFromFile(files[1],
                                  sheet=1, startRow=8, header=FALSE, colTypes="character")

## Time
template[,1] <- fill_variable(template[,1], NA)
template[,2] <- fill_variable(template[,2], NA)
template[,3] <- fill_variable(template[,3], NA)
meta_time <- paste0("orig.time=", c(rep("AM",nrow(template)), rep("PM",nrow(template))))

## Initialize data frame
out[[1]] <- data.frame(year = rep(as.integer(template[,1]), 2), 
                       month = rep(as.integer(template[,2]), 2), 
                       day = rep(as.integer(template[,3]), 2), 
                       hour = rep(NA, 2*nrow(template)), 
                       minute = rep(NA, 2*nrow(template)))

## Pressure
pres <- c(as.numeric(template[,4]), rep(NA,nrow(template)))
out[[1]]$p <- round(convert_pressure(pres, 25.4, 
                                     as.numeric(metadata$Latitude), 
                                     as.numeric(metadata$Altitude)), 
                    1)
out[[1]]$p_meta <- paste0(meta_time, "|orig=", c(template[,4],rep(NA,nrow(template))), "in")
i_atb <- which(!is.na(template[,5]) & !is.na(pres))
out[[1]]$p[i_atb] <- round(convert_pressure(pres[i_atb], 25.4, 
                                            as.numeric(metadata$Latitude), 
                                            as.numeric(metadata$Altitude),
                                            (as.numeric(template[i_atb,5])-32)*5/9), 
                           1)
out[[1]]$p_meta[i_atb] <- paste0(out[[1]]$p_meta[i_atb], "|atb=", 
                                   template[i_atb,5], "F|PTC=Y")

## Temperature
temp <- c(as.numeric(template[,6]), as.numeric(template[,7]))
out[[1]]$ta <- round((temp - 32) * 5 / 9, 1)
out[[1]]$ta_meta <- paste0(meta_time, "|orig=", c(template[,6],template[,7]), "F")
          
## Wind direction
wdir <- c(template[,8], template[,9])
out[[1]]$dd <- round(11.25 * (match(wdir, directions) - 1), 0)
out[[1]]$dd_meta <- paste0(meta_time, "|orig=", wdir)

## Order chronologically
out[[1]] <- out[[1]][order(out[[1]]$year,out[[1]]$month,out[[1]]$day),]

## 1768-1770
## Loop on files
for (i in 2:3) {

## Read data
template <- readWorksheetFromFile(files[i],
                                  sheet=1, startRow=8, header=FALSE, colTypes="character")

## Time
template[,1] <- fill_variable(template[,1], NA)
template[,2] <- fill_variable(template[,2], NA)
template[,3] <- fill_variable(template[,3], NA)
dates <- as.Date(paste(template[,1], template[,2], template[,3], sep="-"))
if (i == 2) {
  times <- strptime(template[,4], format="%Y-%m-%d %H:%M:%S")
  hours <- hour(times)
  minutes <- minute(times)
  meta_time <- paste0("orig.date=", dates, "|orig.time=", substr(template[,4],12,19))
} else {
  hours <- as.integer(substr(template[,4],1,1))
  i_pm <- which(substr(template[,4],2,3) == "pm")
  hours[i_pm] <- hours[i_pm] + 12
  minutes <- rep(0, nrow(template))
  minutes[which(is.na(hours))] <- NA
  meta_time <- paste0("orig.date=", dates, "|orig.time=", template[,4])
}


## Initialize data frame
out[[i]] <- data.frame(year = as.integer(template[,1]), 
                       month = as.integer(template[,2]), 
                       day = as.integer(template[,3]), 
                       hour = hours, 
                       minute = minutes)

## Pressure
pres <- as.numeric(template[,5]) + 0.1 * (as.integer(template[,6]) / as.integer(template[,7]))
i0 <- which(is.na(template[,6]))
pres[i0] <- as.numeric(template[i0,5])
out[[i]]$p <- round(convert_pressure(pres, 25.4, 
                                     as.numeric(metadata$Latitude), 
                                     as.numeric(metadata$Altitude)), 
                    1)
out[[i]]$p_meta <- paste0(meta_time, "|orig=", template[,5], ".", 
                          template[,6], "/", template[,7], "in")
out[[i]]$p_meta[i0] <- paste0(meta_time[i0], "|orig=", template[i0,5], "in")

## Temperature
temp <- as.numeric(template[,8])
out[[i]]$ta <- round((temp - 32) * 5 / 9, 1)
out[[i]]$ta_meta <- paste0(meta_time, "|orig=", template[,8], "F")
          
## Wind direction
wdir <- template[,9]
wdir <- sub("t", "b", wdir)
out[[i]]$dd <- round(11.25 * (match(wdir, directions) - 1), 0)
out[[i]]$dd_meta <- paste0(meta_time, "|orig=", template[,9])

}

## 1772
## Read data
template <- readWorksheetFromFile(files[4],
                                  sheet=1, startRow=8, header=FALSE, colTypes="character")

## Time
template[,1] <- fill_variable(template[,1], NA)
template[,2] <- fill_variable(template[,2], NA)
template[,3] <- fill_variable(template[,3], NA)
dates <- as.Date(paste(template[,1], template[,2], template[,3], sep="-"))
j <- which(nchar(template[,4]) == 4)
hours <- as.integer(substr(template[,4],1,1))
hours[j] <- as.integer(substr(template[j,4],1,2))
i_pm <- c(which(substr(template[,4],2,3) == "pm"), which(substr(template[,4],3,4) == "pm"))
hours[i_pm] <- hours[i_pm] + 12
minutes <- rep(0, nrow(template))
minutes[which(is.na(hours))] <- NA
meta_time <- paste0("orig.date=", dates, "|orig.time=", template[,4])

## Initialize data frame
out[[4]] <- data.frame(year = as.integer(template[,1]), 
                       month = as.integer(template[,2]), 
                       day = as.integer(template[,3]), 
                       hour = hours, 
                       minute = minutes)

## Pressure
pres <- as.numeric(template[,5]) + 0.1 * (as.integer(template[,6]) / as.integer(template[,7]))
i0 <- which(is.na(template[,6]))
pres[i0] <- as.numeric(template[i0,5])
out[[4]]$p <- round(convert_pressure(pres, 25.4, 
                                     as.numeric(metadata$Latitude), 
                                     as.numeric(metadata$Altitude)), 
                    1)
out[[4]]$p_meta <- paste0(meta_time, "|orig=", template[,5], ".", 
                          template[,6], "/", template[,7], "in")
out[[4]]$p_meta[i0] <- paste0(meta_time[i0], "|orig=", template[i0,5], "in")
i_atb <- which(!is.na(template[,9]) & !is.na(pres))
out[[4]]$p[i_atb] <- round(convert_pressure(pres[i_atb], 25.4, 
                                            as.numeric(metadata$Latitude), 
                                            as.numeric(metadata$Altitude),
                                            (as.numeric(template[i_atb,9])-32)*5/9), 
                           1)
out[[4]]$p_meta[i_atb] <- paste0(out[[4]]$p_meta[i_atb], "|atb=", 
                                   template[i_atb,9], "F|PTC=Y")

## Temperature
temp <- as.numeric(template[,8])
out[[4]]$ta <- round((temp - 32) * 5 / 9, 1)
out[[4]]$ta_meta <- paste0(meta_time, "|orig=", template[,8], "F")
          
## Wind direction
wdir <- template[,10]
wdir <- sub("t", "b", wdir)
out[[4]]$dd <- round(11.25 * (match(wdir, directions) - 1), 0)
out[[4]]$dd_meta <- paste0(meta_time, "|orig=", template[,10])

## 1773
## Read data
template <- readWorksheetFromFile(files[5],
                                  sheet=1, startRow=8, header=FALSE, colTypes="character")

## Time
template[,1] <- fill_variable(template[,1], NA)
template[,2] <- fill_variable(template[,2], NA)
template[,3] <- fill_variable(template[,3], NA)
dates <- as.Date(paste(template[,1], template[,2], template[,3], sep="-"))
j <- which(nchar(template[,4]) == 4)
hours <- as.integer(substr(template[,4],1,1))
hours[j] <- as.integer(substr(template[j,4],1,2))
i_pm <- c(which(substr(template[,4],2,3) == "pm"), which(substr(template[,4],3,4) == "pm"))
hours[i_pm] <- hours[i_pm] + 12
minutes <- rep(0, nrow(template))
minutes[which(is.na(hours))] <- NA
meta_time <- paste0("orig.date=", dates, "|orig.time=", template[,4])

## Initialize data frame
out[[5]] <- data.frame(year = as.integer(template[,1]), 
                       month = as.integer(template[,2]), 
                       day = as.integer(template[,3]), 
                       hour = hours, 
                       minute = minutes)

## Pressure
out[[5]]$p <- NA
out[[5]]$p_meta <- ""

## Temperature
temp <- as.numeric(template[,6])
out[[5]]$ta <- round((temp - 32) * 5 / 9, 1)
out[[5]]$ta_meta <- paste0(meta_time, "|orig=", template[,6], "F")
          
## Wind direction
wdir <- template[,7]
wdir <- sub("t", "b", wdir)
out[[5]]$dd <- round(11.25 * (match(wdir, directions) - 1), 0)
out[[5]]$dd_meta <- paste0(meta_time, "|orig=", template[,7])

## Merge data frames
out <- Reduce(rbind, out)

## Write SEF files
write_sef(out[,c("year","month","day","hour","minute","p")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "p",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "hPa",
          stat = "point",
          metaHead = paste0("Observer=", metadata$Observer, " | PGC=Y | PTC=N"),
          meta = out$p_meta,
          period = 0,
          time_offset = as.numeric(metadata$Longitude) * 12 / 180,
          keep_na = TRUE)

write_sef(out[,c("year","month","day","hour","minute","ta")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "ta",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "C",
          stat = "point",
          metaHead = paste0("Observer=", metadata$Observer),
          meta = out$ta_meta,
          period = 0,
          time_offset = as.numeric(metadata$Longitude) * 12 / 180,
          keep_na = TRUE)

write_sef(out[,c("year","month","day","hour","minute","dd")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "dd",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "degree",
          stat = "point",
          metaHead = paste0("Observer=", metadata$Observer),
          meta = out$dd_meta,
          period = 0,
          time_offset = as.numeric(metadata$Longitude) * 12 / 180,
          keep_na = TRUE)


## PHILADELPHIA (RITTENHOUSE) 

files <- list.files("../../3_Revised/Philadelphia/", pattern="Rittenhouse",
                    full.names=TRUE)

## Read metadata
metadata <- inventory[which(inventory$Code == paste0(station_code, "_3")), ]

out <- list()

## 1788-1791
## Read data
template <- readWorksheetFromFile(files[1],
                                  sheet=1, startRow=8, header=FALSE, colTypes="character")

## Time
template[,1] <- fill_variable(template[,1], NA)
template[,2] <- fill_variable(template[,2], NA)
template[,3] <- fill_variable(template[,3], NA)
dates <- as.Date(paste(template[,1], template[,2], template[,3], sep="-"))
times <- strptime(template[,4], format="%Y-%m-%d %H:%M:%S")
hours <- hour(times)
minutes <- minute(times)
meta_time <- substr(template[,4],12,19)
meta_time[which(meta_time=="")] <- template[which(meta_time==""),4]
meta_time <- paste0("orig.date=", dates, "|orig.time=", meta_time)
i_max <- grepl("highest", meta_time)

## Initialize data frames
out[[1]] <- data.frame(year = as.integer(template[!i_max,1]), 
                       month = as.integer(template[!i_max,2]), 
                       day = as.integer(template[!i_max,3]), 
                       hour = hours[!i_max], 
                       minute = minutes[!i_max])

out_max <- data.frame(year = as.integer(template[i_max,1]), 
                      month = as.integer(template[i_max,2]), 
                      day = as.integer(template[i_max,3]), 
                      hour = rep(NA, sum(i_max)), 
                      minute = rep(NA, sum(i_max)))

## Pressure
pres <- as.numeric(template[!i_max,6]) + 0.1 * as.numeric(template[!i_max,7])
out[[1]]$p <- round(convert_pressure(pres, 25.4, 
                                     as.numeric(metadata$Latitude), 
                                     as.numeric(metadata$Altitude)), 
                    1)
out[[1]]$p_meta <- paste0(meta_time[!i_max], "|orig=", template[!i_max,6], 
                          ".", template[!i_max,7], "in")

## Temperature
temp <- as.numeric(template[!i_max,5])
out[[1]]$ta <- round((temp - 32) * 5 / 9, 1)
out[[1]]$ta_meta <- paste0(meta_time[!i_max], "|orig=", template[!i_max,5], "F")

## Max temperature
tmax <- as.numeric(template[i_max,5])
out_max$Tx <- round((tmax - 32) * 5 / 9, 1)
out_max$Tx_meta <- paste0(meta_time[i_max], "|orig=", template[i_max,5], "F")
          
## Wind direction
out[[1]]$dd <- NA
out[[1]]$dd_meta <- ""

## 1791-1795
## Read data
template <- readWorksheetFromFile(files[2],
                                  sheet=1, startRow=8, header=FALSE, colTypes="character")

## Time
template[,1] <- fill_variable(template[,1], NA)
template[,2] <- fill_variable(template[,2], NA)
template[,3] <- fill_variable(template[,3], NA)
dates <- as.Date(paste(template[,1], template[,2], template[,3], sep="-"))
hours <- as.integer(template[,4])
hours[duplicated(dates)] <- hours[duplicated(dates)] + 12
hours[which(hours>=24)] <- hours[which(hours>=24)] - 12
minutes <- rep(0, nrow(template))
meta_time <- paste0("orig.date=", dates, "|orig.time=", template[,4])

## Initialize data frames
out[[2]] <- data.frame(year = as.integer(template[,1]), 
                       month = as.integer(template[,2]), 
                       day = as.integer(template[,3]), 
                       hour = hours, 
                       minute = minutes)

## Pressure
pres <- as.numeric(template[,6])
out[[2]]$p <- round(convert_pressure(pres, 25.4, 
                                     as.numeric(metadata$Latitude), 
                                     as.numeric(metadata$Altitude)), 
                    1)
out[[2]]$p_meta <- paste0(meta_time, "|orig=", template[,6], "in")

## Temperature
temp <- as.numeric(template[,5])
out[[2]]$ta <- round((temp - 32) * 5 / 9, 1)
out[[2]]$ta_meta <- paste0(meta_time, "|orig=", template[,5], "F")

## Wind direction
wdir <- fill_variable(template[,7], "ditto")
out[[2]]$dd <- round(11.25 * (match(wdir, directions) - 1), 0)
out[[2]]$dd_meta <- paste0(meta_time, "|orig=", template[,7])


## 1796-1800
## Read data
template <- readWorksheetFromFile(files[3],
                                  sheet=1, startRow=8, header=FALSE, colTypes="character")

## Time
template[,1] <- fill_variable(template[,1], NA)
template[,2] <- fill_variable(template[,2], NA)
template[,3] <- fill_variable(template[,3], NA)
dates <- as.Date(paste(template[,1], template[,2], template[,3], sep="-"))
hours <- as.integer(template[,4])
hours[duplicated(dates)] <- hours[duplicated(dates)] + 12
hours[which(hours>=24)] <- hours[which(hours>=24)] - 12
minutes <- rep(0, nrow(template))
meta_time <- paste0("orig.date=", dates, "|orig.time=", template[,4])

## Initialize data frames
out[[3]] <- data.frame(year = as.integer(template[,1]), 
                       month = as.integer(template[,2]), 
                       day = as.integer(template[,3]), 
                       hour = hours, 
                       minute = minutes)

## Pressure
pres <- as.numeric(template[,5]) + 0.1 * as.numeric(template[,6])
out[[3]]$p <- round(convert_pressure(pres, 25.4, 
                                     as.numeric(metadata$Latitude), 
                                     as.numeric(metadata$Altitude)), 
                    1)
out[[3]]$p_meta <- paste0(meta_time, "|orig=", template[,5], 
                          ".", template[,6], "in")

## Temperature
temp <- as.numeric(template[,7])
out[[3]]$ta <- round((temp - 32) * 5 / 9, 1)
out[[3]]$ta_meta <- paste0(meta_time, "|orig=", template[,7], "F")

## Wind direction
wdir <- fill_variable(template[,8], "ditto")
out[[3]]$dd <- round(11.25 * (match(wdir, directions) - 1), 0)
out[[3]]$dd_meta <- paste0(meta_time, "|orig=", template[,8])

## 1801-1805
## Read data
template <- readWorksheetFromFile(files[4],
                                  sheet=1, startRow=8, header=FALSE, colTypes="character")

## Time
template[,1] <- fill_variable(template[,1], NA)
template[,2] <- fill_variable(template[,2], NA)
template[,3] <- fill_variable(template[,3], NA)
dates <- as.Date(paste(template[,1], template[,2], template[,3], sep="-"))
hours <- as.integer(template[,4])
hours[duplicated(dates)] <- hours[duplicated(dates)] + 12
hours[which(hours>=24)] <- hours[which(hours>=24)] - 12
minutes <- rep(0, nrow(template))
meta_time <- paste0("orig.date=", dates, "|orig.time=", template[,4])

## Initialize data frames
out[[4]] <- data.frame(year = as.integer(template[,1]), 
                       month = as.integer(template[,2]), 
                       day = as.integer(template[,3]), 
                       hour = hours, 
                       minute = minutes)

## Pressure
out[[4]]$p <- NA
out[[4]]$p_meta <- ""

## Temperature
temp <- as.numeric(template[,5])
out[[4]]$ta <- round((temp - 32) * 5 / 9, 1)
out[[4]]$ta_meta <- paste0(meta_time, "|orig=", template[,5], "F")

## Wind direction
wdir <- template[,6]
out[[4]]$dd <- round(11.25 * (match(wdir, directions) - 1), 0)
out[[4]]$dd_meta <- paste0(meta_time, "|orig=", template[,6])

## Merge data frames
out <- Reduce(rbind, out)

## Write SEF files
write_sef(out[,c("year","month","day","hour","minute","p")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "p",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "hPa",
          stat = "point",
          metaHead = paste0("Observer=", metadata$Observer, " | PGC=Y | PTC=N"),
          meta = out$p_meta,
          period = 0,
          time_offset = as.numeric(metadata$Longitude) * 12 / 180,
          keep_na = TRUE)

write_sef(out[,c("year","month","day","hour","minute","ta")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "ta",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "C",
          stat = "point",
          metaHead = paste0("Observer=", metadata$Observer),
          meta = out$ta_meta,
          period = 0,
          time_offset = as.numeric(metadata$Longitude) * 12 / 180,
          keep_na = TRUE)

write_sef(out[,c("year","month","day","hour","minute","dd")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "dd",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "degree",
          stat = "point",
          metaHead = paste0("Observer=", metadata$Observer),
          meta = out$dd_meta,
          period = 0,
          time_offset = as.numeric(metadata$Longitude) * 12 / 180,
          keep_na = TRUE)

write_sef(out_max[,c("year","month","day","hour","minute","Tx")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "Tx",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "C",
          stat = "maximum",
          metaHead = paste0("Observer=", metadata$Observer),
          meta = out_max$Tx_meta,
          period = "p1day",
          keep_na = TRUE)


## MONTHLY (1773-1777)

metadata <- inventory[which(inventory$Code == paste0(station_code, "_4") & 
                              inventory$Resolution == "monthly"), ]
template <- readWorksheetFromFile("../../3_Revised/Philadelphia/USA_T3_US_Philadelphia_Pemberton_1773-1777_monthly.xls",
                                  sheet=1, startRow=8, header=FALSE, colTypes="character")

## Time
template[,1] <- fill_variable(template[,1], NA)
template[,2] <- fill_variable(template[,2], NA)
time_meta <- matrix(template[,3], nrow=nrow(template)/2, ncol=2, byrow=TRUE)
time_meta <- paste0("obs.times=", apply(time_meta, 1, paste, collapse=","))

## Initialize data frame
out <- data.frame(Year = as.integer(template[,1]),
                  Month = as.integer(template[,2]),
                  Day = rep(NA, nrow(template)),
                  Hour = rep("", nrow(template)),
                  Minute = rep("", nrow(template)),
                  stringsAsFactors = FALSE)
out <- out[!duplicated(paste(out$Year,out$Month)),]

## Pressure
pres <- filter(as.numeric(template[,12]), c(0.5,0.5))[seq(1,nrow(template)-1,2)]
atb <- filter(as.numeric(template[,9]), c(0.5,0.5))[seq(1,nrow(template)-1,2)]
out$p <- round(convert_pressure(pres, 25.4, as.numeric(metadata$Latitude), 
                                 as.numeric(metadata$Altitude), atb), 
                1)
meta <- matrix(template[,12], nrow=nrow(template)/2, ncol=2, byrow=TRUE)
meta_atb <- matrix(template[,9], nrow=nrow(template)/2, ncol=2, byrow=TRUE)
out$p_meta <- paste0(time_meta, "|orig=", apply(meta, 1, paste, collapse="in+"), "in",
                     "|atb=", apply(meta_atb, 1, paste, collapse="F+"), "F")

## Temperature
out$ta <- filter(as.numeric(template[,6]), c(0.5,0.5))[seq(1,nrow(template)-1,2)]
out$ta <- round((out$ta - 32) * 5 / 9, 1)
meta <- matrix(template[,6], nrow=nrow(template)/2, ncol=2, byrow=TRUE)
out$ta_meta <- paste0(time_meta, "|orig=", apply(meta, 1, paste, collapse="F+"), "F")

## Wet days
out$nr <- as.integer(template[seq(1,nrow(template)-1,2),14]) + 
            as.integer(template[seq(1,nrow(template)-1,2),15])
out$nr_meta <- paste0("orig=", template[seq(1,nrow(template)-1,2),14], "+", 
                      template[seq(1,nrow(template)-1,2),15])

## Write SEF files
write_sef(out[,c("Year","Month","Day","Hour","Minute","p")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "p",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "hPa",
          stat = "mean",
          metaHead = paste0("Observer=", metadata$Observer, 
                            " | PGC=Y | PTC=Y"),
          meta = out$p_meta,
          period = "month",
          keep_na = TRUE,
          note = "monthly")

write_sef(out[,c("Year","Month","Day","Hour","Minute","ta")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "ta",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "C",
          stat = "mean",
          metaHead = paste0("Observer=", metadata$Observer),
          meta = out$ta_meta,
          period = "month",
          keep_na = TRUE,
          note = "monthly")

write_sef(out[,c("Year","Month","Day","Hour","Minute","nr")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "nr",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "count",
          stat = "sum",
          metaHead = paste0("Observer=", metadata$Observer),
          meta = out$nr_meta,
          period = "month",
          keep_na = TRUE)

file.rename(list.files(paste0("../../4_Formatted/", name_folder), full.names=TRUE), 
            gsub("NA", "", list.files(paste0("../../4_Formatted/", name_folder), full.names=TRUE)))


## Check SEF files
for (f in list.files(paste0("../../4_Formatted/", name_folder), full.names=TRUE)) {
  print(f)
  check_sef(f)
}
